Package: GREENREG
Title: Tool for Statistical and Environmental Analysis
Version: 0.1.0
Authors@R: 
    person("Dayron Jared", "Bazan Guzman", , "dbazanguzman@gmail.com", role = c("aut", "cre"))
Description: Provides a set of accessible and automated functions to apply 
    statistical models such as Simple Linear Regression (RLS, from the Spanish 
    'Regresión Lineal Simple'), Multiple Linear Regression (RLM, from the Spanish 
    'Regresión Lineal Múltiple'), Generalized Linear Models (GLM), and time 
    series analysis through Autoregressive Integrated Moving Average (ARIMA) 
    models. Designed to support teaching at the Universidad Autónoma Chapingo, 
    it facilitates results interpretation and assumption validation through 
    automatic graphical diagnostics. Methods for regression and time series 
    are based on Montgomery et al. (2021, ISBN:978-1119570141) and 
    Box & Jenkins (1970, ISBN:978-0816211043).
License: GPL-3
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
Depends: R (>= 3.5)
Imports: ggplot2, ggrepel, stats, lmtest,
Suggests: pROC, broom, car, rmarkdown, tseries
NeedsCompilation: no
Packaged: 2026-05-25 03:56:12 UTC; dbaza
Author: Dayron Jared Bazan Guzman [aut, cre]
Maintainer: Dayron Jared Bazan Guzman <dbazanguzman@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-29 10:10:14 UTC
Built: R 4.6.0; ; 2026-05-29 11:46:22 UTC; unix

Package: GREENREG
Title: Tool for Statistical and Environmental Analysis
Version: 0.1.0
Authors@R: 
    person("Dayron Jared", "Bazan Guzman", , "dbazanguzman@gmail.com", role = c("aut", "cre"))
Description: Provides a set of accessible and automated functions to apply 
    statistical models such as Simple Linear Regression (RLS, from the Spanish 
    'Regresión Lineal Simple'), Multiple Linear Regression (RLM, from the Spanish 
    'Regresión Lineal Múltiple'), Generalized Linear Models (GLM), and time 
    series analysis through Autoregressive Integrated Moving Average (ARIMA) 
    models. Designed to support teaching at the Universidad Autónoma Chapingo, 
    it facilitates results interpretation and assumption validation through 
    automatic graphical diagnostics. Methods for regression and time series 
    are based on Montgomery et al. (2021, ISBN:978-1119570141) and 
    Box & Jenkins (1970, ISBN:978-0816211043).
License: GPL-3
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
Depends: R (>= 3.5)
Imports: ggplot2, ggrepel, stats, lmtest,
Suggests: pROC, broom, car, rmarkdown, tseries
NeedsCompilation: no
Packaged: 2026-05-25 03:56:12 UTC; dbaza
Author: Dayron Jared Bazan Guzman [aut, cre]
Maintainer: Dayron Jared Bazan Guzman <dbazanguzman@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-29 10:10:14 UTC
Built: R 4.6.0; ; 2026-05-29 11:10:25 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(plot,ar_greenreg)
S3method(plot,arima_greenreg)
S3method(plot,arma_greenreg)
S3method(plot,logistico_greenreg)
S3method(plot,ma_greenreg)
S3method(plot,poisson_greenreg)
S3method(plot,rlm_greenreg)
S3method(plot,rls_greenreg)
S3method(print,ar_greenreg)
S3method(print,arima_greenreg)
S3method(print,arma_greenreg)
S3method(print,logistico_greenreg)
S3method(print,ma_greenreg)
S3method(print,poisson_greenreg)
S3method(print,rlm_greenreg)
S3method(print,rls_greenreg)
export(analisis_datos)
export(logistico)
export(modelo_ar)
export(modelo_arima)
export(modelo_arma)
export(modelo_ma)
export(reg_poisson)
export(rlm)
export(rls)
import(ggplot2)
importFrom(ggrepel,geom_text_repel)
importFrom(lmtest,bptest)
importFrom(lmtest,dwtest)
importFrom(lmtest,raintest)
importFrom(stats,Box.test)
importFrom(stats,acf)
importFrom(stats,aggregate)
importFrom(stats,anova)
importFrom(stats,arima)
importFrom(stats,as.formula)
importFrom(stats,binomial)
importFrom(stats,coef)
importFrom(stats,confint)
importFrom(stats,cooks.distance)
importFrom(stats,cor)
importFrom(stats,density)
importFrom(stats,df.residual)
importFrom(stats,dnorm)
importFrom(stats,dpois)
importFrom(stats,filter)
importFrom(stats,fitted)
importFrom(stats,formula)
importFrom(stats,gaussian)
importFrom(stats,glm)
importFrom(stats,hatvalues)
importFrom(stats,lm)
importFrom(stats,logLik)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,na.omit)
importFrom(stats,pacf)
importFrom(stats,pchisq)
importFrom(stats,pf)
importFrom(stats,pnorm)
importFrom(stats,poisson)
importFrom(stats,predict)
importFrom(stats,printCoefmat)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,reorder)
importFrom(stats,resid)
importFrom(stats,residuals)
importFrom(stats,rstandard)
importFrom(stats,sd)
importFrom(stats,shapiro.test)
importFrom(stats,step)
importFrom(utils,globalVariables)
importFrom(utils,menu)

#' Regresion de Poisson (Modelos de Conteo)
#'
#' Esta funcion ajusta un Modelo Lineal Generalizado (GLM) de la familia Poisson con
#' funcion de enlace logaritmica \eqn{\ln(\lambda)}. Esta disenada para modelar variables
#' de respuesta que representan conteos enteros no negativos (0, 1, 2, 3...) y tasas
#' de incidencia en estudios agronomicos, forestales y sociales.
#'
#' @details
#' El flujo interno de la funcion se divide en seis etapas criticas:
#' \enumerate{
#'   \item **Ajuste Log-Lineal:** Estima la relacion entre los predictores y el logaritmo
#'         de la tasa de eventos (\eqn{\lambda}). Utiliza Maxima Verosimilitud (MLE) para el ajuste.
#'   \item **Interpretacion de Tasas (IRR):** Los coeficientes originales (\eqn{\beta}) se
#'         exponencian automaticamente \eqn{e^{\beta}} para obtener los **Ratios de Tasas de Incidencia**.
#'         Esto permite concluir en que porcentaje aumenta o disminuye la frecuencia de eventos
#'         por cada unidad de cambio en X.
#'   \item **Diagnostico de Equidispersion:** Calcula el Ratio de Dispersion basado en el
#'         estadistico de Pearson \eqn{(X^2 / gl)}. En una distribucion Poisson perfecta, este
#'         valor debe ser 1. Valores > 1.5 indican **Sobredispersion**, sugiriendo que los
#'         datos varian mas de lo que el modelo permite.
#'   \item **Inferencia Robusta:** Genera intervalos de confianza para los IRR, permitiendo
#'         identificar factores de riesgo (IRR > 1) y factores protectores (IRR < 1).
#'   \item **Evaluacion de Residuos:** Calcula residuos de devianza y Pearson para identificar
#'         patrones de error sistematicos o falta de ajuste en conteos especificos (como el cero).
#'   \item **Traduccion Didactica:** Genera un reporte que advierte si el modelo es valido
#'         o si se debe migrar a un modelo Binomial Negativa debido a la dispersion.
#' }
#'
#' @section Guia de las 8 Graficas de Diagnostico (`plot`):
#' Al ejecutar \code{plot(modelo)}, se despliega una secuencia de validacion para conteos:
#' \describe{
#'   \item{\strong{1. Distribucion de Conteos (Real vs Modelo)}}{Compara las frecuencias observadas contra las que el modelo "esperaba" predecir. Crucial para detectar el exceso de ceros.}
#'   \item{\strong{2. Precision (Predicho vs Real)}}{Scatterplot entre el Lambda estimado y el conteo real. Los puntos deben seguir la tendencia de la linea roja diagonal.}
#'   \item{\strong{3. Importancia de Variables (IRR)}}{Visualiza el impacto relativo de cada predictor. Los puntos verdes son significativos; si el intervalo cruza el 1, la variable es ruido.}
#'   \item{\strong{4. Verificacion de Sobredispersion}}{Grafica la relacion Media-Varianza. Si la linea verde (datos) se eleva sobre la negra (teoria), el supuesto Poisson se rompe.}
#'   \item{\strong{5. Distancia de Cook}}{Identifica casos atipicos en el conteo que tienen un peso excesivo en la estimacion de la tasa promedio.}
#'   \item{\strong{6. Residuos de Devianza}}{Analiza la calidad del ajuste en escala logaritmica. Busca una nube uniforme sin formas de "U" o de embudo.}
#'   \item{\strong{7. Apalancamiento (Leverage)}}{Detecta observaciones con valores de X inusuales que controlan la pendiente del modelo.}
#'   \item{\strong{8. Densidad de Ajuste Global}}{Superpone las curvas de probabilidad del modelo y la realidad para evaluar el solapamiento total.}
#' }
#'
#' @param formula Una formula de R (ej. \code{numero_frutos ~ riego + fertilizante}).
#' @param data Un \code{data.frame} con las observaciones (el conteo debe ser entero).
#'
#' @return Un objeto de clase \code{"poisson_greenreg"} que contiene:
#' \itemize{
#'   \item \code{modelo}: Objeto \code{glm} original de la familia Poisson.
#'   \item \code{irr_table}: Tabla con los Ratios de Tasas de Incidencia, intervalos y p-valores.
#'   \item \code{dispersion}: El ratio de dispersion calculado (Pearson / GL).
#'   \item \code{metricas}: Lista con AIC, Log-Verosimilitud y Devianzas.
#'   \item \code{predicciones}: Dataframe con valores observados, lambda estimado y diagnosticos (leverage/residuos).
#' }
#'
#' @examples
#' data(datos_tala_ilegal_poisson)
#' modelo_poisson <- reg_poisson(numero_talas ~ densidad_poblacion + 
#'                               superficie_forestal_ha + dias_vigilancia, 
#'                               data = datos_tala_ilegal_poisson)
#' plot(modelo_poisson)
#' @importFrom stats glm poisson coef confint formula na.omit predict fitted residuals df.residual pnorm qnorm logLik pchisq cooks.distance hatvalues rstandard dpois quantile
#' @importFrom ggrepel geom_text_repel
#' @import ggplot2
#' @export
reg_poisson <- function(formula, data) {
  # --- 1. Ajuste del Modelo ---
  modelo_base <- stats::glm(formula, data = data, family = stats::poisson(link = "log"), na.action = stats::na.omit)

  # --- 2. Verificacion de Equidispersion (Ratio de Pearson) ---
  # Si el ratio es > 1.5 o 2, hay sobredispersion
  pearson_chisq <- sum(stats::residuals(modelo_base, type = "pearson")^2)
  ratio_dispersion <- pearson_chisq / modelo_base$df.residual

  # --- 3. Coeficientes e IRR ---
  resumen <- summary(modelo_base)
  coefs_log <- stats::coef(modelo_base)
  irr <- exp(coefs_log)
  ci_irr <- exp(stats::confint(modelo_base))
  tabla_irr <- cbind(IRR = irr, `2.5 %` = ci_irr[,1], `97.5 %` = ci_irr[,2], `P-valor` = resumen$coefficients[,4])

  df_preds <- data.frame(
    obs = modelo_base$y,
    pred = stats::fitted(modelo_base),
    leverage = stats::hatvalues(modelo_base),
    resid_std = stats::rstandard(modelo_base)
  )
  # --- 4. Empaquetado ---
  resultado <- list(
    formula = formula,
    modelo = modelo_base,
    predicciones = df_preds,
    coeficientes = resumen$coefficients,
    irr_table = tabla_irr,
    dispersion = ratio_dispersion,
    metricas = list(
      aic = modelo_base$aic,
      loglik = as.numeric(stats::logLik(modelo_base)),
      null_dev = modelo_base$null.deviance,
      res_dev = modelo_base$deviance
    )
  )
  class(resultado) <- "poisson_greenreg"
  return(resultado)
}

#' @rdname reg_poisson
#' @param x Objeto de clase.
#' @param ... Argumentos adicionales.
#' @export
print.poisson_greenreg <- function(x, ...) {
  
  cat("\n==============================================================\n")
  cat("            REGRESION POISSON (MODELO DE CONTEO) \n")
  cat("==============================================================\n\n")
  
  # --- 1. MODELO Y ECUACION ---
  cat("--- 1. MODELO Y ECUACION ---\n")
  cat("Formula:", deparse(x$formula), "\n")
  
  # CORRECCION 2: Espacio de nombres stats::
  b <- round(stats::coef(x$modelo), 3)
  vars <- names(b)
  ecuacion <- paste0("log(Y) = ", b[1])
  if(length(b) > 1) {
    for(i in 2:length(b)) ecuacion <- paste0(ecuacion, " + (", b[i], " * ", vars[i], ")")
  }
  cat("Ecuacion estimada:\n", ecuacion, "\n\n")
  
  # --- 2. COEFICIENTES ESTIMADOS ---
  cat("--- 2. COEFICIENTES ESTIMADOS (IRR) ---\n")
  cat("Interpretacion en Tasas de Incidencia (Incidence Rate Ratios):\n")
  print(round(x$irr_table, 4))
  cat("\n NOTA: Los IRR indican cuanto se multiplica la tasa de eventos.\n")
  cat("   - IRR > 1: Efecto multiplicativo (Riesgo/Aumento).\n")
  cat("   - IRR < 1: Efecto de reduccion (Proteccion/Disminucion).\n\n")
  
  # --- 3. EVALUACION DEL MODELO ---
  cat("--- 3. EVALUACION DEL MODELO (Ratio Test) ---\n")
  
  chi_cuad <- x$metricas$null_dev - x$metricas$res_dev
  df_chi <- x$modelo$df.null - x$modelo$df.residual
  
  # CORRECCION 3: Blindaje logico contra valores nulos
  if (!is.null(chi_cuad) && !is.na(chi_cuad) && !is.null(df_chi) && !is.na(df_chi) && df_chi > 0) {
    p_val_lrt <- 1 - stats::pchisq(chi_cuad, df_chi)
    p_val_txt <- if(is.na(p_val_lrt)) "NA" else format.pval(p_val_lrt)
    concl_txt <- if(!is.na(p_val_lrt) && p_val_lrt < 0.05) "El modelo explica significativamente los datos" else "El modelo no es mejor que el promedio (nulo)"
  } else {
    chi_cuad <- NA
    df_chi <- NA
    p_val_txt <- "NA"
    concl_txt <- "No disponible o indeterminado"
  }
  
  chi_txt <- if(is.na(chi_cuad)) "NA" else sprintf("%.3f", chi_cuad)
  df_txt <- if(is.na(df_chi)) "NA" else as.character(df_chi)
  
  cat(sprintf(" Chi-cuadrado Global: %s | GL: %s\n", chi_txt, df_txt))
  cat(" P-valor (LRT):", p_val_txt, "\n")
  cat(" Conclusion:", concl_txt, "\n\n")
  
  # --- 4. BONDAD DE AJUSTE ---
  cat("--- 4. BONDAD DE AJUSTE ---\n")
  
  pseudo_r2 <- 1 - (x$metricas$res_dev / x$metricas$null_dev)
  
  # Proteccion estricta para sprintf
  loglik_txt <- if(is.null(x$metricas$loglik) || is.na(x$metricas$loglik)) "NA" else sprintf("%10.2f", x$metricas$loglik)
  resdev_txt <- if(is.null(x$metricas$res_dev) || is.na(x$metricas$res_dev)) "NA" else sprintf("%10.2f", x$metricas$res_dev)
  pseudo_txt <- if(is.null(pseudo_r2) || is.na(pseudo_r2)) "NA" else sprintf("%7.2f%%", pseudo_r2 * 100)
  
  cat(sprintf(" Log-Verosimilitud: %s\n", loglik_txt))
  cat(sprintf(" Devianza Residual: %s\n", resdev_txt))
  cat(sprintf(" Pseudo R2 (McFadden): %s\n", pseudo_txt))
  cat("  (Representa la reduccion de la incertidumbre/error respecto al modelo nulo).\n\n")
  
  # --- 5. SIGNIFICANCIA DE COEFICIENTES ---
  cat("--- 5. SIGNIFICANCIA DE COEFICIENTES (Prueba Wald) ---\n")
  
  # Blindaje extra convirtiendo a matrix
  if (!is.null(x$coeficientes) && nrow(x$coeficientes) > 0) {
    stats::printCoefmat(as.matrix(x$coeficientes), digits = 4, signif.stars = TRUE)
  } else {
    cat(" No se pudieron estimar coeficientes validos.\n")
  }
  cat("\n")
  
  # --- 6. VERIFICACION DE SUPUESTOS ---
  cat("--- 6. VERIFICACION DE SUPUESTOS ---\n")
  
  # A) Equidispersion
  disp <- x$dispersion
  disp_txt <- if(is.null(disp) || is.na(disp)) "NA" else sprintf("%.2f", disp)
  
  cat(sprintf(" Equidispersion (Ratio): %s\n", disp_txt))
  
  if(!is.null(disp) && !is.na(disp)) {
    if(disp > 1.5) {
      cat(" ALERTA: Hay SOBREDISPERSION (Ratio > 1.5). Los p-valores podrian estar subestimados.\n")
    } else {
      cat(" OKEY: Se cumple razonablemente el supuesto de equidispersion (Media = Varianza).\n")
    }
  }
  
  # B) Otros supuestos
  cat(" Independencia: Asumida por el diseno de muestreo.\n")
  cat(" Linealidad: Se asume que log(Media) es lineal respecto a las variables X.\n\n")
  
  # --- 7. RESUMEN DEL MODELO ---
  cat("--- 7. RESUMEN DEL MODELO ---\n")
  
  # CORRECCION 4: Validacion estructural de la tabla antes de hacer el filtro
  if (!is.null(x$irr_table) && "P-valor" %in% colnames(x$irr_table)) {
    p_vals <- x$irr_table[, "P-valor"]
    sig_vars <- rownames(x$irr_table)[!is.na(p_vals) & p_vals < 0.05]
    
    if(length(sig_vars) > 0) {
      cat(" Variables significativas identificadas:", paste(sig_vars, collapse = ", "), "\n")
    } else {
      cat(" No se encontraron variables con impacto estadistico significativo.\n")
    }
  }
  
  if(!is.null(disp) && !is.na(disp) && disp > 2) {
    cat("\n RECOMENDACION: Debido a la alta sobredispersion, se sugiere \n probar un modelo Binomial Negativa o usar errores estandar robustos.\n")
  }
  
  cat("\n==============================================================\n")
  
  return(invisible(x))  
}


#' @rdname reg_poisson
#' @description Genera una secuencia de 8 graficas de diagnostico para validar el modelo Poisson.
#' @param x Objeto de clase.
#' @param ... Argumentos adicionales.
#' @export
plot.poisson_greenreg <- function(x, ...) {
  # --- 1. Extraccion y Validacion de Datos ---
  modelo <- x$modelo
  if (is.null(x$predicciones)) stop("Error interno: No se encontraron predicciones.")
  df_preds <- x$predicciones
  
  # Tema visual consistente
  mi_tema <- ggplot2::theme_minimal() +
    ggplot2::theme(
      plot.title = ggplot2::element_text(face = "bold", size = 14, color = "#2C3E50"),
      plot.subtitle = ggplot2::element_text(size = 11, color = "#7F8C8D"),
      plot.caption = ggplot2::element_text(hjust = 0, size = 10, face = "italic", color = "#555555", margin = ggplot2::margin(t = 10))
    )
  
  # CORRECCION 1: Usar message() en lugar de cat()
  if (interactive()) {
    message("Generando 8 graficas de diagnostico... (Presiona [Enter] para avanzar)")
  }
  
  # --- GRAFICA 1: CONTEOS OBSERVADOS VS PREDICHOS ---
  
  # Determinamos el limite del eje X (evitando valores extremos que arruinen la visualizacion)
  max_val_obs <- max(df_preds$obs, na.rm = TRUE)
  max_count <- round(stats::quantile(df_preds$obs, 0.99, na.rm = TRUE))
  if (is.na(max_count) || max_count < 1) max_count <- max_val_obs
  
  # A) Frecuencia Real (Observada)
  obs_limpios <- df_preds$obs[!is.na(df_preds$obs)]
  freq_real <- table(factor(obs_limpios, levels = 0:max_count))
  df_counts <- data.frame(Conteo = 0:max_count, Freq = as.numeric(freq_real), Tipo = "Realidad")
  
  # B) Frecuencia Esperada (Modelo Poisson)
  # Calculamos la probabilidad de cada conteo k dado el lambda (pred) de cada fila
  probs_matrix <- sapply(0:max_count, function(k) {
    sum(stats::dpois(k, lambda = df_preds$pred), na.rm = TRUE)
  })
  
  df_expected <- data.frame(Conteo = 0:max_count, Freq = probs_matrix, Tipo = "Modelo (Esperado)")
  
  # Combinamos ambos para ggplot
  df_comparacion <- rbind(df_counts, df_expected)
  
  g1 <- ggplot2::ggplot(df_comparacion, ggplot2::aes(x = factor(Conteo), y = Freq, fill = Tipo)) +
    ggplot2::geom_bar(stat = "identity", position = "dodge", alpha = 0.8) +
    ggplot2::scale_fill_manual(values = c("Realidad" = "#34495E", "Modelo (Esperado)" = "#E67E22")) +
    ggplot2::labs(
      title = "1. Distribucion de Conteos (Real vs Modelo)",
      subtitle = "Predice el modelo la cantidad correcta de eventos?",
      x = "Numero de Eventos (Conteo)",
      y = "Frecuencia Total",
      caption = paste0("INTERPRETACION:\n",
                       " BIEN: Las barras naranjas (Modelo) tienen casi la misma altura que las grises (Realidad).\n",
                       " ALERTA: Si la barra gris del '0' es gigante y la naranja pequena, hay exceso de ceros.\n",
                       " DISPERSION: Si las barras reales son mas anchas que las del modelo, hay sobredispersion.")
    ) +
    mi_tema
  
  print(g1)
  if (interactive()) {readline(prompt = "Grafica 1/8 > ")}
  
  
  # --- GRAFICA 2: VALORES PREDICHOS VS VALORES REALES ---
  
  # Calculamos la correlacion para mostrarla en el subtitulo
  cor_val <- stats::cor(df_preds$obs, df_preds$pred, method = "pearson")
  
  g2 <- ggplot2::ggplot(df_preds, ggplot2::aes(x = pred, y = obs)) +
    # Puntos con ligera transparencia para ver densidad (overplotting)
    ggplot2::geom_point(color = "#2E86C1", alpha = 0.5, size = 2) +
    # Linea de Identidad (Lo que seria un modelo perfecto)
    ggplot2::geom_abline(slope = 1, intercept = 0, linetype = "dashed", color = "red", linewidth = 1) +
    # Suavizado logistico/tendencia para ver el comportamiento real
    ggplot2::geom_smooth(method = "glm", method.args = list(family = "poisson"),
                         color = "#F1C40F", se = TRUE, fill = "#F1C40F", alpha = 0.2) +
    ggplot2::labs(
      title = "2. Precision del Modelo (Predicho vs Real)",
      subtitle = paste0("Correlacion de Pearson: ", round(cor_val, 3)),
      x = "Valor Predicho (Lambda estimado)",
      y = "Valor Real (Conteo observado)",
      caption = paste0("INTERPRETACION:\n",
                       " LINEA ROJA: Es la diagonal de perfeccion (donde Predicho = Real).\n",
                       " LINEA AMARILLA: Es la tendencia real de tus datos.\n",
                       " BIEN: Si los puntos y la linea amarilla siguen de cerca a la roja.\n",
                       " ALERTA: Si los puntos se alejan mucho hacia arriba conforme aumenta X, hay subestimacion.")
    ) +
    mi_tema
  
  print(g2)
  if (interactive()) { readline(prompt = "Grafica 2/8 > ")}
  
  
  # --- GRAFICA 3: IMPORTANCIA DE VARIABLES (IRR) ---
  
  # 1. Preparar los datos desde la tabla de IRR generada en la funcion principal
  df_irr <- as.data.frame(x$irr_table)
  df_irr$Variable <- rownames(df_irr)
  
  # 2. Limpieza: Excluimos el Intercepto para no distorsionar la escala visual
  df_irr <- df_irr[df_irr$Variable != "(Intercept)", ]
  
  # Renombrar columnas para facilitar el manejo en ggplot
  colnames(df_irr) <- c("IRR", "Low", "High", "P_val", "Variable")
  
  # 3. Construccion de la Grafica
  g3 <- ggplot2::ggplot(df_irr, ggplot2::aes(x = reorder(Variable, IRR), y = IRR)) +
    # Linea de referencia en 1 (indica que no hay efecto)
    ggplot2::geom_hline(yintercept = 1, linetype = "dashed", color = "black", linewidth = 0.8) +
    # Barras de error (Intervalos de Confianza al 95%)
    ggplot2::geom_errorbar(ggplot2::aes(ymin = Low, ymax = High), width = 0.2, color = "#7F8C8D") +
    # Puntos del IRR colorizados por significancia
    ggplot2::geom_point(ggplot2::aes(color = P_val < 0.05), size = 4) +
    # Etiquetas con el valor del IRR sobre cada punto
    ggplot2::geom_text(ggplot2::aes(label = round(IRR, 2)), vjust = -1.2, size = 3.5, fontface = "bold") +
    # Invertimos ejes para que los nombres de variables sean legibles (horizontal)
    ggplot2::coord_flip() +
    # Colores semanticos: Verde para significativo, Rojo para no significativo
    ggplot2::scale_color_manual(values = c("TRUE" = "#27AE60", "FALSE" = "#E74C3C"),
                                labels = c("No Significativo", "Significativo (p < 0.05)")) +
    ggplot2::labs(
      title = "3. Importancia de Variables (IRR)",
      subtitle = "Impacto relativo en la tasa de eventos (Incidence Rate Ratio)",
      x = "Variables Predictoras",
      y = "IRR (Razon de Tasas de Incidencia)",
      color = "Estado del Coeficiente:",
      caption = paste0("INTERPRETACION:\n",
                       " VERDE: La variable tiene un impacto real (p < 0.05).\n",
                       " LINEA EN 1: Si el intervalo cruza esta linea, la variable no influye.\n",
                       " DERECHA (>1): La variable INCREMENTA la frecuencia de eventos.\n",
                       " IZQUIERDA (<1): La variable REDUCE la frecuencia de eventos.")
    ) +
    mi_tema +
    ggplot2::theme(legend.position = "top")
  
  print(g3)
  if (interactive()) {readline(prompt = "Grafica 3/8 > ")}
  
  
  # --- GRAFICA 4: VERIFICACION DE SOBREDISPERSION ---
  
  # Preparar datos: La varianza empirica se aproxima con el cuadrado de los residuos
  df_disp <- data.frame(
    predicho = df_preds$pred,
    res_sq = (df_preds$obs - df_preds$pred)^2
  )
  
  g4 <- ggplot2::ggplot(df_disp, ggplot2::aes(x = predicho, y = res_sq)) +
    # Puntos de dispersion
    ggplot2::geom_point(color = "#8E44AD", alpha = 0.4, size = 2) +
    # Linea de Equidispersion (Varianza = Media)
    ggplot2::geom_abline(slope = 1, intercept = 0, linetype = "dashed", color = "black", linewidth = 1) +
    # Tendencia de la varianza real
    ggplot2::geom_smooth(method = "gam", color = "#2ECC71", fill = "#2ECC71", alpha = 0.2) +
    ggplot2::labs(
      title = "4. Verificacion de Sobredispersion",
      subtitle = "Se cumple el supuesto de Media = Varianza?",
      x = "Media Predicha ",
      y = "Varianza Observada (Residuos al cuadrado)",
      caption = paste0("INTERPRETACION:\n",
                       " LINEA NEGRA: Representa la Equidispersion teorica (Poisson perfecta).\n",
                       " LINEA VERDE: Es la relacion real entre media y varianza en tus datos.\n",
                       " BIEN: Si la linea verde sigue de cerca a la negra punteada.\n",
                       " ALERTA: Si la linea verde sube mucho mas rapido que la negra, hay SOBREDISPERSION.")
    ) +
    mi_tema
  
  print(g4)
  if (interactive()) { readline(prompt = "Grafica 4/8 > ")}
  
  
  # --- GRAFICA 5: DATOS INFLUYENTES (DISTANCIA DE COOK) ---
  
  # 1. Calculo de la Distancia de Cook
  cooks_d <- stats::cooks.distance(x$modelo)
  n_obs <- length(cooks_d)
  
  # Umbral critico comun: 4/n
  umbral_cook <- 4 / n_obs
  
  df_cook <- data.frame(
    ID = 1:n_obs,
    Distancia = cooks_d
  )
  
  # Etiquetamos solo los puntos que superan el umbral
  df_cook$Label <- ifelse(df_cook$Distancia > umbral_cook, as.character(df_cook$ID), "")
  
  g5 <- ggplot2::ggplot(df_cook, ggplot2::aes(x = ID, y = Distancia)) +
    # Lineas verticales
    ggplot2::geom_segment(ggplot2::aes(xend = ID, yend = 0), color = "darkgrey", alpha = 0.5) +
    # Puntos destacados
    ggplot2::geom_point(ggplot2::aes(color = Distancia > umbral_cook), size = 2) +
    # Linea de umbral critico
    ggplot2::geom_hline(yintercept = umbral_cook, linetype = "dashed", color = "#E74C3C", linewidth = 0.8)
  
  # CORRECCION 4: Check defensivo de ggrepel
  if (requireNamespace("ggrepel", quietly = TRUE)) {
    g5 <- g5 + ggrepel::geom_text_repel(ggplot2::aes(label = Label), size = 3.5, fontface = "bold")
  } else {
    g5 <- g5 + ggplot2::geom_text(ggplot2::aes(label = Label), vjust = -1, size = 3.5, fontface = "bold")
  }
  
  g5 <- g5 +
    # Colores: Rojo para los que superan el umbral
    ggplot2::scale_color_manual(values = c("FALSE" = "#3498DB", "TRUE" = "#E74C3C")) +
    ggplot2::labs(
      title = "5. Datos Influyentes (Distancia de Cook)",
      subtitle = paste0("Umbral critico (4/n): ", round(umbral_cook, 4)),
      x = "Indice de la Observacion (Fila)",
      y = "Distancia de Cook",
      caption = paste0("INTERPRETACION:\n",
                       " PUNTOS ROJOS: Son observaciones que tienen un impacto excesivo en el modelo.\n",
                       " UMBRAL: Si un punto esta muy por encima de la linea roja, revisa sus datos.\n",
                       " ACCION: Podrian ser errores de dedo o casos excepcionales que distorsionan los IRR.")
    ) +
    mi_tema +
    ggplot2::theme(legend.position = "none")
  
  print(g5)
  if (interactive()) {readline(prompt = "Grafica 5/8 > ")}
  
  
  # --- GRAFICA 6: ANALISIS DE RESIDUOS (DEVIANZA VS AJUSTADOS) ---
  
  # Extraemos residuos de devianza (los estandar para GLM Poisson)
  df_res <- data.frame(
    ajustados = log(df_preds$pred), # Usamos el log(lambda) para linearizar la escala
    residuos = stats::residuals(x$modelo, type = "deviance")
  )
  
  g6 <- ggplot2::ggplot(df_res, ggplot2::aes(x = ajustados, y = residuos)) +
    # Puntos de los residuos
    ggplot2::geom_point(color = "#16A085", alpha = 0.5, size = 2) +
    # Linea central en 0
    ggplot2::geom_hline(yintercept = 0, linetype = "dashed", color = "black", linewidth = 0.8) +
    # Lineas de umbral (+-2)
    ggplot2::geom_hline(yintercept = c(-2, 2), linetype = "dotted", color = "red", alpha = 0.5) +
    # Tendencia suavizada (Loess)
    ggplot2::geom_smooth(method = "loess", color = "#E67E22", fill = "#E67E22", alpha = 0.2, formula = y ~ x) +
    ggplot2::labs(
      title = "6. Analisis de Residuos de Devianza",
      subtitle = "Residuos vs. Valores Ajustados (escala log)",
      x = "Valores Ajustados (Log-Lambda)",
      y = "Residuos de Devianza",
      caption = paste0("INTERPRETACION:\n",
                       " BIEN: Los puntos se reparten al azar arriba y abajo del cero (nube uniforme).\n",
                       " CURVA: Si la linea naranja hace una 'U', te falta una variable cuadratica o interaccion.\n",
                       " EMBUDO: Si la dispersion aumenta hacia la derecha, confirma la Sobredispersion.\n",
                       " OUTLIERS: Puntos fuera de las lineas rojas (+-2) estan muy mal predichos.")
    ) +
    mi_tema
  
  print(g6)
  if (interactive()) {readline(prompt = "Grafica 6/8 > ")}
  
  
  # --- GRAFICA 7: APALANCAMIENTO (LEVERAGE) ---
  
  # 1. Preparar datos de Leverage (h)
  lev <- x$predicciones$leverage
  n_obs <- length(lev)
  p_vars <- length(stats::coef(x$modelo)) # Numero de parametros (k)
  
  # Umbral comun: 2k/n
  umbral_lev <- (2 * p_vars) / n_obs
  
  df_lev <- data.frame(
    ID = 1:n_obs,
    Leverage = lev
  )
  
  # Etiquetar solo puntos con alto apalancamiento
  df_lev$Label <- ifelse(df_lev$Leverage > umbral_lev, as.character(df_lev$ID), "")
  
  g7 <- ggplot2::ggplot(df_lev, ggplot2::aes(x = ID, y = Leverage)) +
    # Lineas verticales para cada punto
    ggplot2::geom_segment(ggplot2::aes(xend = ID, yend = 0), color = "lightgrey", alpha = 0.6) +
    # Puntos con tamano basado en su propio valor de leverage
    ggplot2::geom_point(ggplot2::aes(color = Leverage > umbral_lev, size = Leverage)) +
    # Linea de umbral critico (2k/n)
    ggplot2::geom_hline(yintercept = umbral_lev, linetype = "dashed", color = "#2980B9", linewidth = 0.8)
  
  # CORRECCION 4: Check defensivo de ggrepel
  if (requireNamespace("ggrepel", quietly = TRUE)) {
    g7 <- g7 + ggrepel::geom_text_repel(ggplot2::aes(label = Label), size = 3.5, fontface = "bold")
  } else {
    g7 <- g7 + ggplot2::geom_text(ggplot2::aes(label = Label), vjust = -1, size = 3.5, fontface = "bold")
  }
  
  g7 <- g7 +
    # Escala de colores (Azul para normal, Naranja para alto leverage)
    ggplot2::scale_color_manual(values = c("FALSE" = "#BDC3C7", "TRUE" = "#E67E22")) +
    ggplot2::labs(
      title = "7. Analisis de Apalancamiento (Leverage)",
      subtitle = paste0("Umbral de diseno (2k/n): ", round(umbral_lev, 4)),
      x = "Indice de la Observacion (Fila)",
      y = "Valor de Leverage (h)",
      caption = paste0("INTERPRETACION:\n",
                       " PUNTOS NARANJA: Son observaciones con valores de X muy poco comunes.\n",
                       " UMBRAL AZUL: Marca el limite donde un punto empieza a tener mucho peso.\n",
                       " NOTA: Un alto leverage no es malo por si solo, pero si ese punto \n",
                       "   tambien tiene un residuo alto (Grafica 6), esta danando tu modelo.")
    ) +
    mi_tema +
    ggplot2::theme(legend.position = "none")
  
  print(g7)
  if (interactive()) {readline(prompt = "Grafica 7/8 > ")}
  
  
  # --- GRAFICA 8: AJUSTE DEL MODELO (DENSIDAD REAL VS PREDICHA) ---
  
  # Creamos un dataframe largo para facilitar el mapeo de colores en ggplot
  df_ajuste <- data.frame(
    Valor = c(df_preds$obs, df_preds$pred),
    Fuente = rep(c("Observado (Real)", "Ajustado (Modelo)"), each = nrow(df_preds))
  )
  
  g8 <- ggplot2::ggplot(df_ajuste, ggplot2::aes(x = Valor, fill = Fuente)) +
    # Usamos geom_density con alfa para ver la superposicion
    ggplot2::geom_density(alpha = 0.4, color = "white") +
    # Anadimos lineas de media para cada grupo
    ggplot2::geom_vline(xintercept = mean(df_preds$obs), color = "#2C3E50", linetype = "dashed", linewidth = 1) +
    ggplot2::geom_vline(xintercept = mean(df_preds$pred), color = "#D35400", linetype = "dashed", linewidth = 1) +
    ggplot2::scale_fill_manual(values = c("Observado (Real)" = "#34495E", "Ajustado (Modelo)" = "#E67E22")) +
    ggplot2::labs(
      title = "8. Ajuste Global del Modelo",
      subtitle = "Comparacion de Densidades: Realidad vs. Prediccion",
      x = "Numero de Eventos",
      y = "Densidad (Probabilidad suavizada)",
      caption = paste0("INTERPRETACION:\n",
                       " AJUSTE OPTIMO: Si ambas 'montanas' coinciden en posicion y ancho.\n",
                       " DESPLAZAMIENTO: Si las lineas punteadas (medias) estan lejos, el modelo es sesgado.\n",
                       " FORMA: Si la curva naranja es mucho mas delgada que la gris, confirma la Sobredispersion.\n",
                       " NOTA: La linea naranja representa la 'esperanza' del modelo Poisson.")
    ) +
    mi_tema +
    ggplot2::theme(legend.position = "top")
  
  print(g8)
  
  # CORRECCION 1 y 3: message() en vez de cat(), return(invisible(x))
  if (interactive()) { 
    message("Diagnostico Poisson completado.")
  }
  return(invisible(x))
}

#' Modelo de Media Movil (MA)
#'
#' Esta funcion ajusta un modelo de series de tiempo donde el valor actual \eqn{(Y_t)}
#' se explica mediante una combinacion lineal de los errores (shocks) pasados
#' (\eqn{\epsilon_{t-1}}, \eqn{\epsilon_{t-2}}, ...). Es ideal para modelar fenomenos con
#' efectos de memoria corta o impactos transitorios.
#'
#' @details
#' El flujo interno de la funcion se divide en cinco etapas fundamentales:
#' \enumerate{
#'   \item **Estimacion (MLE):** Utiliza Maxima Verosimilitud para estimar los
#'         parametros \eqn{\theta} (Theta). A diferencia del AR, el MA requiere
#'         algoritmos iterativos ya que los errores no son directamente observables.
#'   \item **Analisis de Invertibilidad:** Calcula las raices inversas del polinomio
#'         caracteristico. Un modelo MA debe ser "Invertible" para que el error
#'         pueda expresarse como una funcion convergente de los datos observados.
#'   \item **Identificacion Estructural:** Evalua la ACF (Autocorrelacion). En un
#'         proceso MA(q), la ACF debe "cortarse" abruptamente despues del lag q.
#'   \item **Auditoria de Residuos:** Verifica que el error remanente sea Ruido Blanco
#'         mediante la prueba de Ljung-Box y la distribucion Gaussiana (Shapiro-Wilk).
#'   \item **Traduccion Didactica:** Genera un reporte que explica la persistencia
#'         de los choques y valida si el modelo es unico y matematicamente solido.
#' }
#'
#' @section Guia de las 9 Graficas de Diagnostico (`plot`):
#' Al ejecutar \code{plot(modelo)}, se despliega una auditoria de "shocks" temporales:
#' \describe{
#'   \item{\strong{1. Inspeccion Visual}}{Muestra la serie bruta con su media historica. El modelo MA supone que la serie siempre es atraida hacia este nivel central.}
#'   \item{\strong{2. Ajuste Temporal}}{Superpone el modelo (azul) sobre la realidad. Permite ver como el modelo "suaviza" los picos de la serie.}
#'   \item{\strong{3. Precision (Real vs Ajustado)}}{Grafica de dispersion para evaluar la correlacion. Los puntos deben seguir la diagonal roja.}
#'   \item{\strong{4. Circulo Unitario}}{Prueba de Invertibilidad. Los rombos azules deben estar dentro del circulo para asegurar que el modelo sea unico.}
#'   \item{\strong{5. Residuos en el Tiempo}}{Evaluacion de aleatoriedad. No debe haber patrones; los errores deben ser impredecibles.}
#'   \item{\strong{6. ACF de Residuos}}{Prueba critica de Ruido Blanco. Si hay barras largas (numeros rojos), el orden \code{q} es insuficiente.}
#'   \item{\strong{7. PACF de Residuos}}{Complemento de la ACF para verificar que no existan dependencias directas remanentes.}
#'   \item{\strong{8. Normalidad (Q-Q Plot)}}{Valida que los choques sigan una campana de Gauss, asegurando la validez de los intervalos de confianza.}
#'   \item{\strong{9. Estacionariedad Movil}}{Confirma que la media y varianza no se desplazan. Vital para asegurar que el modelo MA sea aplicable.}
#' }
#'
#' @param x Vector numerico o serie de tiempo (\code{ts}). No debe contener valores \code{NA}.
#' @param q Orden del modelo (numero de choques pasados a considerar). Por defecto es 1.
#' @param include_mean Logico. Incluir el promedio constante de la serie? Por defecto \code{TRUE}.
#'
#' @return Un objeto de clase \code{"ma_greenreg"}, que consiste en una lista con:
#' \itemize{
#'   \item \code{modelo}: El objeto \code{arima} original (orden 0,0,q).
#'   \item \code{orden_q}: El orden del modelo ajustado.
#'   \item \code{coeficientes_tabla}: Matriz con estimaciones, errores estandar y p-valores.
#'   \item \code{invertibilidad}: Lista con las raices inversas y diagnostico de invertibilidad.
#'   \item \code{diagnosticos}: Resultados de las pruebas de Ljung-Box, Shapiro-Wilk y AIC.
#'   \item \code{data_plot}: Dataframe listo para visualizacion con \code{ggplot2}.
#' }
#' @examples
#' data("datos_anomalia_temperatura")
#' modelo_MA <- modelo_ma(datos_anomalia_temperatura$anomalia_c, q = 1, include_mean = TRUE)
#' modelo_MA
#' plot(modelo_MA)
#'
#' @importFrom stats arima coef residuals fitted Box.test pnorm qnorm shapiro.test acf pacf logLik filter sd
#' @importFrom ggrepel geom_text_repel
#' @import ggplot2
#' @export
modelo_ma <- function(x, q = 1, include_mean = TRUE) {

  # --- 1. Validacion y Ajuste ---
  if (!is.numeric(x)) stop("El argumento 'x' debe ser numerico.")

  # Ajuste usando ARIMA(0,0,q) que equivale a MA(q)
  modelo_base <- stats::arima(x, order = c(0, 0, q), include.mean = include_mean)

  # --- 2. Coeficientes y Estadisticas ---
  coefs <- modelo_base$coef
  se <- sqrt(diag(modelo_base$var.coef))
  z_val <- coefs / se
  p_val <- 2 * (1 - stats::pnorm(abs(z_val)))

  tabla_coefs <- cbind(Estimate = coefs, `Std. Error` = se, `z value` = z_val, `Pr(>|z|)` = p_val)

  # --- 3. Analisis de Invertibilidad (Raices Unitarias) ---
  # La invertibilidad en MA es equivalente a la estabilidad en AR
  # Extraemos solo los coeficientes MA (ignoramos intercepto)
  ma_coefs <- coefs[grepl("^ma", names(coefs))]

  if (length(ma_coefs) > 0) {
    # Polinomio caracteristico: 1 + theta1*z + theta2*z^2 ...
    poly_roots <- polyroot(c(1, ma_coefs))
    inverse_roots <- 1 / poly_roots
    modulos <- Mod(inverse_roots)
    # Regla: Invertible si TODOS los modulos son < 1
    es_invertible <- all(modulos < 1)
  } else {
    inverse_roots <- NULL
    es_invertible <- TRUE
  }

  # --- 4. Diagnosticos de Residuos ---
  residuos <- stats::residuals(modelo_base)

  # Ljung-Box (Ruido Blanco)
  lb_test <- stats::Box.test(residuos, type = "Ljung-Box", lag = q + 5)

  # Shapiro-Wilk (Normalidad)
  if (length(residuos) >= 3 && length(residuos) <= 5000) {
    shapiro_test <- stats::shapiro.test(residuos)
  } else {
    shapiro_test <- list(p.value = NA)
  }

  # --- 5. Datos para Graficas ---
  ajustados <- x - residuos
  sigma <- sqrt(modelo_base$sigma2)
  # Intervalos de confianza 95%
  lower_ci <- ajustados - 1.96 * sigma
  upper_ci <- ajustados + 1.96 * sigma

  df_plot <- data.frame(
    Tiempo = 1:length(x),
    Observado = as.numeric(x),
    Ajustado = as.numeric(ajustados),
    Lower = as.numeric(lower_ci),
    Upper = as.numeric(upper_ci),
    Residuos = as.numeric(residuos)
  )

  # Datos Circulo Unitario
  df_raices <- NULL
  if (!is.null(inverse_roots)) {
    df_raices <- data.frame(Real = Re(inverse_roots), Imaginario = Im(inverse_roots))
  }

  # --- 6. Objeto Final ---
  resultado <- list(
    x_original = x,
    modelo = modelo_base,
    orden_q = q,
    coeficientes_tabla = tabla_coefs,
    invertibilidad = list(es_invertible = es_invertible, raices = inverse_roots),
    diagnosticos = list(lb = lb_test, shapiro = shapiro_test, aic = modelo_base$aic),
    data_plot = df_plot,
    data_raices = df_raices
  )

  class(resultado) <- "ma_greenreg"
  return(resultado)
}

#' @rdname modelo_ma
#' @param x Objeto de clase.
#' @param ... Argumentos adicionales.
#' @export
print.ma_greenreg <- function(x, ...) {
  
  cat("\n==========================================================\n")
  cat("      MODELO DE MEDIA MOVIL MA(", x$orden_q, ") \n", sep = "")
  cat("==========================================================\n\n")
  
  # --- 1. MODELO Y ECUACION ---
  cat("--- 1. MODELO Y ECUACION ---\n")
  theta <- round(x$modelo$coef[grepl("^ma", names(x$modelo$coef))], 3)
  c_int <- round(x$modelo$coef["intercept"], 3)
  
  # Proteccion contra interceptos NA
  mu <- if(is.na(c_int)) 0 else c_int
  
  # Construccion visual de la ecuacion de choques
  eq <- paste0("Y(t) = ", mu, " + E(t)")
  if(length(theta) > 0) {
    for(i in 1:length(theta)) {
      eq <- paste0(eq, " + (", theta[i], " * E(t-", i, "))")
    }
  }
  
  cat("Ecuacion de Medias Moviles Estimada:\n", eq, "\n")
  
  # Proteccion NA para AIC
  aic_val <- x$diagnosticos$aic
  aic_txt <- if(is.null(aic_val) || is.na(aic_val)) "NA" else round(aic_val, 2)
  cat("AIC (Criterio de Akaike):", aic_txt, "\n")
  cat(" NOTA: El modelo MA(", x$orden_q, ") describe como los choques (E) del pasado afectan al presente.\n\n", sep = "")
  
  # --- 2. COEFICIENTES ESTIMADOS ---
  cat("--- 2. COEFICIENTES ESTIMADOS (Parametros Theta) ---\n")
  # Obligatorio usar as.matrix para evitar desajustes
  stats::printCoefmat(as.matrix(x$coeficientes_tabla), digits = 4, signif.stars = TRUE)
  cat("\n GUIA: Los coeficientes Theta miden la persistencia de los choques externos.\n\n")
  
  # --- 3. IDENTIFICACION DEL MODELO (ORDEN q) ---
  cat("--- 3. IDENTIFICACION DEL MODELO (Orden q) ---\n")
  cat(" Orden seleccionado: q =", x$orden_q, "\n")
  cat(" Criterio ACF: La autocorrelacion debe cortarse abruptamente despues del lag", x$orden_q, ".\n")
  cat(" Criterio PACF: Los coeficientes parciales deben decaer exponencialmente hacia cero.\n")
  cat(" NOTA: El modelo MA es el 'espejo' del AR. Aqui la ACF es la que manda para elegir q.\n\n")
  
  # --- 4. VERIFICACION DE SUPUESTOS ---
  cat("--- 4. VERIFICACION DE SUPUESTOS ---\n")
  
  # A) Estacionariedad e Invertibilidad
  cat("A) Estacionariedad e Invertibilidad:\n")
  cat("  Estacionariedad: Un modelo MA puro es SIEMPRE estacionario por definicion.\n")
  
  if (!is.null(x$invertibilidad$raices)) {
    modulos <- Mod(x$invertibilidad$raices)
    cat("  Modulos de raices inversas (Invertibilidad):", paste(round(modulos, 3), collapse = ", "), "\n")
    
    es_inv <- x$invertibilidad$es_invertible
    if(!is.null(es_inv) && !is.na(es_inv) && es_inv) {
      cat("     -> OKEY: El modelo es Invertible (Raices < 1). Los errores son estimables.\n")
    } else {
      cat("     ->  ALERTA: Modelo NO Invertible. Los choques pasados tienen un peso infinito.\n")
    }
  }
  
  # B) Errores ~ Ruido Blanco
  cat("\nB) Diagnostico de Errores (E ~ RB):\n")
  
  # Media Cero (Protegido)
  mean_res <- mean(x$data_plot$Residuos, na.rm = TRUE)
  mean_txt <- if(is.na(mean_res)) "NA" else sprintf("%.4f", mean_res)
  cat(sprintf("  Media de los residuos: %s (Esperado: 0)\n", mean_txt))
  
  # No Autocorrelacion Ljung-Box (Protegido)
  lb_p <- x$diagnosticos$lb$p.value
  lb_txt <- if(is.null(lb_p) || is.na(lb_p)) "NA" else sprintf("%.4f", lb_p)
  cat(sprintf("  No Autocorrelacion (Ljung-Box p-valor): %s\n", lb_txt))
  
  if (!is.null(lb_p) && !is.na(lb_p) && lb_p > 0.05) {
    cat("     -> OKEY: Los residuos son ruido blanco. Toda la senal fue capturada.\n")
  } else {
    cat("     ->  ALERTA: Queda estructura en los residuos. Considera aumentar el orden q.\n")
  }
  
  # Varianza y Normalidad (Protegido)
  sh_p <- x$diagnosticos$shapiro$p.value
  sh_txt <- if(is.null(sh_p) || is.na(sh_p)) "NA" else sprintf("%.4f", sh_p)
  cat(sprintf("  Normalidad (Shapiro p-valor): %s\n", sh_txt))
  cat("     ->", if(!is.null(sh_p) && !is.na(sh_p) && sh_p > 0.05) "OKEY: Errores normales.\n" else " Errores no normales o indeterminados.\n")
  
  # --- 5. RESUMEN DEL MODELO ---
  cat("\n--- 5. RESUMEN DEL MODELO ---\n")
  
  # Validacion logica segura para evitar colapso por TRUE/FALSE faltante
  es_valido <- FALSE
  es_inv <- x$invertibilidad$es_invertible
  
  if (!is.null(es_inv) && !is.na(es_inv) && !is.null(lb_p) && !is.na(lb_p)) {
    if (es_inv && lb_p > 0.05) {
      es_valido <- TRUE
    }
  }
  
  if(es_valido) {
    cat("RESULTADO FINAL: El modelo MA(", x$orden_q, ") es VALIDO y UNICO.\n", sep="")
  } else {
    cat(" RECOMENDACION: Revisa el orden q o la invertibilidad para asegurar un ajuste correcto.\n")
  }
  
  cat("\n Usa plot(modelo) para visualizar la serie y la ACF de los residuos.\n")
  cat("==========================================================\n")
  
  return(invisible(x))
}

#' @rdname modelo_ma
#' @description Genera una secuencia de 9 graficas de diagnostico para validar el modelo MA.
#' @param x Objeto de clase.
#' @param ... Argumentos adicionales.
#' @export
plot.ma_greenreg <- function(x, ...) {
  if (!requireNamespace("ggplot2", quietly = TRUE)) stop("Instala ggplot2.")
  
  df <- x$data_plot
  
  # TEMA ESTANDAR
  mi_tema <- ggplot2::theme_minimal() +
    ggplot2::theme(
      plot.title = ggplot2::element_text(face = "bold", size = 14, color = "#2C3E50"),
      plot.subtitle = ggplot2::element_text(size = 11, color = "#7F8C8D"),
      plot.caption = ggplot2::element_text(hjust = 0, size = 9, face = "italic",
                                           color = "#555555", margin = ggplot2::margin(t = 15),
                                           lineheight = 1.1),
      plot.margin = ggplot2::margin(10, 10, 20, 10),
      axis.title = ggplot2::element_text(face = "bold"),
      legend.position = "top"
    )
  
  # CORRECCION 1: message en lugar de cat
  if (interactive()) {
    message("Generando graficas de diagnostico MA... (Presiona [Enter] para avanzar)")
  }
  
  # --- GRAFICA 1: SERIE ORIGINAL (VALORES VS TIEMPO) ---
  
  # Calculamos la media para la linea de referencia
  media_serie <- mean(df$Observado, na.rm = TRUE)
  
  g1 <- ggplot2::ggplot(df, ggplot2::aes(x = Tiempo, y = Observado)) +
    # Linea de la serie original
    ggplot2::geom_line(color = "#34495E", linewidth = 0.7) +
    # Puntos sutiles
    ggplot2::geom_point(color = "#34495E", alpha = 0.3, size = 1) +
    # Linea de la media historica (Referencia fundamental en MA)
    ggplot2::geom_hline(yintercept = media_serie, linetype = "dashed",
                        color = "#2980B9", linewidth = 0.8) +
    ggplot2::labs(
      title = "1. Comportamiento de la Serie Original",
      subtitle = "Visualizacion de la serie temporal bruta",
      x = "Tiempo (Periodos)",
      y = "Valor Observado",
      caption = paste0("INTERPRETACION:\n",
                       " LINEA AZUL: Es el valor promedio historico.\n",
                       " MEDIA MOVIL: El modelo MA supone que la serie siempre regresa a esta media.\n",
                       " NOTA: Si observas una tendencia marcada (subida o bajada constante),\n",
                       "   un modelo MA puro podria no ser suficiente sin diferenciar primero.")
    ) +
    mi_tema
  
  print(g1)
  if (interactive()) {readline(prompt = "Grafica 1: Inspeccion de la serie pura > ")}
  
  # --- GRAFICA 2: AJUSTE DEL MODELO (Real vs Ajustado) ---
  
  g2 <- ggplot2::ggplot(df, ggplot2::aes(x = Tiempo)) +
    # Intervalo de confianza (Sombra azul)
    ggplot2::geom_ribbon(ggplot2::aes(ymin = Lower, ymax = Upper),
                         fill = "#2980B9", alpha = 0.15) +
    # Serie Observada (Realidad)
    ggplot2::geom_line(ggplot2::aes(y = Observado, color = "Observado"), linewidth = 0.7) +
    # Serie Ajustada (Modelo MA)
    ggplot2::geom_line(ggplot2::aes(y = Ajustado, color = "Modelo MA"), linewidth = 0.8) +
    # Configuracion de colores
    ggplot2::scale_color_manual(values = c("Observado" = "#34495E", "Modelo MA" = "#2980B9")) +
    ggplot2::labs(
      title = paste0("2. Ajuste del Modelo MA(", x$orden_q, ")"),
      subtitle = "Serie Observada vs. Valores Ajustados (Fitted Values)",
      x = "Tiempo (Periodos)",
      y = "Valor",
      color = "Leyenda:",
      caption = paste0("INTERPRETACION:\n",
                       " BIEN: La linea azul debe capturar la tendencia central de la oscura.\n",
                       " SOMBRA AZUL: Margen de error esperado (IC 95%).\n",
                       " NOTA: El modelo MA suele ser mas 'suave' que la serie original,\n",
                       "   ya que filtra el ruido aleatorio para quedarse con el promedio movil.")
    ) +
    mi_tema
  
  print(g2)
  if (interactive()) {readline(prompt = "Grafica 2: Comparacion del modelo contra los datos reales > ")}
  
  # --- GRAFICA 3: PRECISION DEL MODELO (REAL VS AJUSTADO) ---
  
  # Calculamos la correlacion para el reporte visual
  cor_ma <- stats::cor(df$Observado, df$Ajustado, use = "complete.obs")
  
  g3 <- ggplot2::ggplot(df, ggplot2::aes(x = Ajustado, y = Observado)) +
    # Puntos de dispersion (Color azul MA)
    ggplot2::geom_point(color = "#2980B9", alpha = 0.5, size = 2) +
    # Linea de Identidad (El ideal de perfeccion)
    ggplot2::geom_abline(slope = 1, intercept = 0, linetype = "dashed", color = "#E74C3C", linewidth = 1) +
    # Linea de tendencia real con formula explicita para evitar avisos
    ggplot2::geom_smooth(method = "lm", formula = y ~ x, color = "#2C3E50", fill = "#BDC3C7", alpha = 0.2) +
    ggplot2::labs(
      title = "3. Precision del Modelo (Predicho vs Real)",
      subtitle = paste0("Correlacion de Pearson: ", round(cor_ma, 3)),
      x = "Valor Ajustado (Modelo MA)",
      y = "Valor Observado (Realidad)",
      caption = paste0("INTERPRETACION:\n",
                       " LINEA ROJA: Diagonal de 45 (Donde el modelo es perfecto).\n",
                       " BIEN: Los puntos deben agruparse cerca de la linea roja.\n",
                       " NOTA: Si la nube de puntos es muy ancha, el modelo MA tiene poca precision.\n",
                       " CLAVE: Un valor de correlacion cercano a 1 indica un ajuste excelente.")
    ) +
    mi_tema
  
  print(g3)
  if (interactive()) {readline(prompt = "Grafica 3: Analisis de correlacion y precision. > ")}
  
  # --- GRAFICA 4: INVERTIBILIDAD (CIRCULO UNITARIO) ---
  
  if (!is.null(x$data_raices)) {
    # 1. Crear el circulo de referencia
    theta <- seq(0, 2*pi, length.out = 200)
    df_circle <- data.frame(x = cos(theta), y = sin(theta))
    
    # 2. Construccion de la Grafica con Estilo MA (Azul)
    g4 <- ggplot2::ggplot() +
      # Circulo unitario punteado
      ggplot2::geom_path(data = df_circle, ggplot2::aes(x, y),
                         color = "#7F8C8D", linetype = "dashed", linewidth = 0.8) +
      # Ejes cartesianos
      ggplot2::geom_vline(xintercept = 0, color = "#BDC3C7") +
      ggplot2::geom_hline(yintercept = 0, color = "#BDC3C7") +
      # area de Invertibilidad (Verde tenue)
      ggplot2::geom_polygon(data = df_circle, ggplot2::aes(x, y),
                            fill = "#27AE60", alpha = 0.05) +
      # Raices inversas (Puntos azules para MA)
      ggplot2::geom_point(data = x$data_raices,
                          ggplot2::aes(x = Real, y = Imaginario),
                          color = "#2980B9", size = 4, shape = 18) +
      # Ajuste de proporciones
      ggplot2::coord_fixed(xlim = c(-1.2, 1.2), ylim = c(-1.2, 1.2)) +
      ggplot2::labs(
        title = "4. Invertibilidad (Circulo Unitario)",
        subtitle = paste0("Raices Inversas del Polinomio MA(", x$orden_q, ")"),
        x = "Parte Real",
        y = "Parte Imaginaria",
        caption = paste0("INTERPRETACION:\n",
                         " INVERTIBLE: Todas las raices (rombos azules) estan DENTRO del circulo.\n",
                         " ALERTA: Si una raiz toca o sale del circulo, el modelo no es unico.\n",
                         " NOTA: La invertibilidad asegura que los errores del modelo sean estimables\n",
                         "   a partir de los datos observados de forma convergente.")
      ) +
      mi_tema
    
    print(g4)
    
  } else {
    # CORRECCION 1: Usar message()
    message("El modelo MA(0) no tiene raices; es ruido blanco y siempre es invertible.")
  }
  if (interactive()) {readline(prompt = "Grafica 4: Analisis de raices del proceso de media movil > ")}
  
  # --- GRAFICA 5: RESIDUOS EN EL TIEMPO (E vs t) ---
  
  # CORRECCION 2: stats::sd()
  sd_res_ma <- stats::sd(df$Residuos, na.rm = TRUE)
  
  g5 <- ggplot2::ggplot(df, ggplot2::aes(x = Tiempo, y = Residuos)) +
    # Banda de confianza (+-2 desviaciones estandar) en color Morado/Azul
    ggplot2::geom_ribbon(ggplot2::aes(ymin = -2*sd_res_ma, ymax = 2*sd_res_ma),
                         fill = "#8E44AD", alpha = 0.1) +
    # Linea de los residuos
    ggplot2::geom_line(color = "#2C3E50", linewidth = 0.5) +
    # Puntos para identificar errores individuales
    ggplot2::geom_point(color = "#8E44AD", alpha = 0.4, size = 1) +
    # Linea central (Media Cero)
    ggplot2::geom_hline(yintercept = 0, linetype = "dashed", color = "black", linewidth = 0.8) +
    # Umbrales criticos
    ggplot2::geom_hline(yintercept = c(-2*sd_res_ma, 2*sd_res_ma),
                        linetype = "dotted", color = "#E74C3C", alpha = 0.6) +
    ggplot2::labs(
      title = "5. Residuos en el Tiempo",
      subtitle = "Analisis de Aleatoriedad (E ~ Ruido Blanco)",
      x = "Tiempo (Periodos)",
      y = "Error (Residuo)",
      caption = paste0("INTERPRETACION:\n",
                       " BIEN: Los puntos deben oscilar al azar alrededor de la linea negra.\n",
                       " BANDAS ROJAS: El 95% de los errores deben estar dentro de estos limites.\n",
                       " MAL: Si ves ciclos o tendencias, el orden 'q' es insuficiente.\n",
                       " CLAVE: Si la variabilidad aumenta con el tiempo, el modelo es inestable.")
    ) +
    mi_tema
  
  print(g5)
  if (interactive()) {readline(prompt = "Grafica 5: Verificacion de la independencia temporal de los errores. > ")}
  
  # --- GRAFICA 6: AUTOCORRELACION DE RESIDUOS (ACF) ---
  
  # 1. Calculo de la ACF de los residuos (maximo 20 lags)
  acf_res_ma <- stats::acf(df$Residuos, plot = FALSE, lag.max = 20)
  
  # Creamos el dataframe omitiendo el Lag 0 (que siempre es 1)
  df_acf_ma <- data.frame(
    Lag = as.numeric(acf_res_ma$lag[-1]),
    ACF = as.numeric(acf_res_ma$acf[-1])
  )
  
  # Limite de significancia estadistica (95%)
  n_obs_ma <- nrow(df)
  limite_ma <- 1.96 / sqrt(n_obs_ma)
  
  # Logica de etiquetas rojas para alertar sobre fallos en lags especificos
  df_acf_ma$Etiqueta <- ifelse(abs(df_acf_ma$ACF) > limite_ma, as.character(df_acf_ma$Lag), "")
  df_acf_ma$vjust_pos <- ifelse(df_acf_ma$ACF > 0, -0.5, 1.5)
  
  # 2. Construccion de la Grafica
  g6 <- ggplot2::ggplot(df_acf_ma, ggplot2::aes(x = Lag, y = ACF)) +
    # Barras de correlacion (Color oscuro para contraste)
    ggplot2::geom_col(fill = "#34495E", width = 0.4) +
    # Lineas de significancia (Azules para MA)
    ggplot2::geom_hline(yintercept = c(limite_ma, -limite_ma),
                        linetype = "dashed", color = "#2980B9", linewidth = 0.8) +
    # Linea base en cero
    ggplot2::geom_hline(yintercept = 0, color = "black") +
    # Etiquetas de error (Numeros Rojos)
    ggplot2::geom_text(ggplot2::aes(label = Etiqueta, vjust = vjust_pos),
                       color = "#E74C3C", fontface = "bold", size = 3.5) +
    # Escala del eje X
    ggplot2::scale_x_continuous(breaks = 1:20) +
    ggplot2::labs(
      title = "6. Autocorrelacion de Residuos (ACF)",
      subtitle = "Quedo memoria de choques sin capturar?",
      x = "Lag (Retraso temporal)",
      y = "Coeficiente ACF",
      caption = paste0("INTERPRETACION:\n",
                       " BIEN: Si todas las barras estan dentro de las lineas azules.\n",
                       " NUMEROS ROJOS: Indican que el error en ese lag NO es aleatorio.\n",
                       " MAL: Si el Lag 1 es rojo, tu modelo MA(", x$orden_q, ") es insuficiente.\n",
                       " CLAVE: Los residuos deben ser 'Ruido Blanco' (puro azar).")
    ) +
    mi_tema
  
  print(g6)
  if (interactive()) {readline(prompt = "Grafica 6: Diagnostico de independencia de los errores. > ")}
  
  # --- GRAFICA 7: AUTOCORRELACION PARCIAL (PACF) DE RESIDUOS ---
  
  # 1. Calculo de la PACF de los residuos
  pacf_res_ma <- stats::pacf(df$Residuos, plot = FALSE, lag.max = 20)
  
  df_pacf_ma <- data.frame(
    Lag = as.numeric(pacf_res_ma$lag),
    PACF = as.numeric(pacf_res_ma$acf)
  )
  
  # Limite de significancia (Mismo que en ACF)
  n_obs_ma <- nrow(df)
  limite_ma <- 1.96 / sqrt(n_obs_ma)
  
  # Logica de etiquetas rojas para fallas
  df_pacf_ma$Etiqueta <- ifelse(abs(df_pacf_ma$PACF) > limite_ma, as.character(df_pacf_ma$Lag), "")
  df_pacf_ma$vjust_pos <- ifelse(df_pacf_ma$PACF > 0, -0.5, 1.5)
  
  # 2. Construccion de la Grafica con Estilo MA (Azul/Naranja)
  g7 <- ggplot2::ggplot(df_pacf_ma, ggplot2::aes(x = Lag, y = PACF)) +
    # Barras de la PACF (Color Azul MA)
    ggplot2::geom_col(fill = "#2980B9", width = 0.4) +
    # Lineas de significancia (Cambiamos a Naranja para diferenciar de ACF)
    ggplot2::geom_hline(yintercept = c(limite_ma, -limite_ma),
                        linetype = "dashed", color = "#E67E22", linewidth = 0.8) +
    # Linea base
    ggplot2::geom_hline(yintercept = 0, color = "black") +
    # Etiquetas de error (Chivatos en Rojo)
    ggplot2::geom_text(ggplot2::aes(label = Etiqueta, vjust = vjust_pos),
                       color = "#E74C3C", fontface = "bold", size = 3.5) +
    # Escala del eje X
    ggplot2::scale_x_continuous(breaks = 1:20) +
    ggplot2::labs(
      title = "7. Autocorrelacion Parcial (PACF)",
      subtitle = "Identificacion de dependencias directas en residuos",
      x = "Lag (Retraso temporal)",
      y = "Coeficiente PACF",
      caption = paste0("INTERPRETACION:\n",
                       " BIEN: Ninguna barra sobresale de las lineas naranjas.\n",
                       " NUMEROS ROJOS: Indican que ese retraso especifico aun tiene informacion.\n",
                       " NOTA: En un modelo MA bien ajustado, tanto ACF como PACF de residuos \n",
                       "   deberian verse como Ruido Blanco (barras cortas).")
    ) +
    mi_tema
  
  print(g7)
  if (interactive()) {readline(prompt = "Grafica 7: Verificacion de la limpieza estructural del error > ")}
  
  # --- GRAFICA 8: NORMALIDAD DE RESIDUOS (Q-Q PLOT) ---
  
  g8 <- ggplot2::ggplot(df, ggplot2::aes(sample = Residuos)) +
    # Puntos de los cuantiles observados (Color Naranja para MA)
    ggplot2::stat_qq(color = "#E67E22", alpha = 0.6, size = 2) +
    # Linea de referencia teorica (Normalidad perfecta)
    ggplot2::stat_qq_line(color = "#2C3E50", linetype = "dashed", linewidth = 1) +
    ggplot2::labs(
      title = "8. Normalidad de los Errores (Q-Q Plot)",
      subtitle = "Comparacion de Cuantiles de Residuos vs. Normal Teorica",
      x = "Cuantiles Teoricos (Distribucion Normal)",
      y = "Cuantiles Observados (Residuos MA)",
      caption = paste0("INTERPRETACION:\n",
                       " BIEN: Los puntos deben seguir la linea diagonal punteada.\n",
                       " EXTREMOS: Si los puntos se alejan en las puntas, hay valores atipicos.\n",
                       " NOTA: La normalidad valida que el error tipico (Sigma) sea constante.\n",
                       " CLAVE: Si los puntos forman una curva, la serie tiene asimetria.")
    ) +
    mi_tema
  
  print(g8)
  if (interactive()) {readline(prompt = "Grafica 8: Evaluacion de la distribucion gaussiana del error > ")}
  
  # --- GRAFICA 9: DIAGNOSTICO DE ESTACIONARIEDAD (MOVIL) ---
  
  # 1. Definimos la ventana (10% de los datos o minimo 5)
  ventana_ma <- max(5, round(nrow(df) * 0.1))
  
  df_est_ma <- df
  
  # 2. CALCULOS (Es vital forzar as.numeric para ggplot2)
  df_est_ma$Media_Movil <- as.numeric(stats::filter(df$Observado, rep(1/ventana_ma, ventana_ma), sides = 2))
  
  # CORRECCION 2: stats::sd()
  df_est_ma$SD_Movil <- sapply(1:nrow(df), function(i) {
    idx <- max(1, i-ventana_ma):min(nrow(df), i+ventana_ma)
    stats::sd(df$Observado[idx], na.rm = TRUE)
  })
  
  # 3. FILTRADO (Eliminamos los NAs antes de entrar a ggplot)
  df_final_ma <- df_est_ma[!is.na(df_est_ma$Media_Movil), ]
  
  # 4. CONSTRUCCION DE LA GRAFICA
  g9 <- ggplot2::ggplot(df_final_ma, ggplot2::aes(x = Tiempo)) +
    # Serie original completa de fondo (usando df original para no perder datos visuales)
    ggplot2::geom_line(data = df, ggplot2::aes(y = Observado), color = "gray80", linewidth = 0.5) +
    
    # Franja de Varianza Movil (Azul MA)
    ggplot2::geom_ribbon(ggplot2::aes(ymin = Media_Movil - SD_Movil,
                                      ymax = Media_Movil + SD_Movil),
                         fill = "#2980B9", alpha = 0.15) +
    
    # Linea de la Media Movil (Nivel local)
    ggplot2::geom_line(ggplot2::aes(y = Media_Movil), color = "#2980B9", linewidth = 1.2) +
    
    # Linea de la Media Global (Referencia fija en Rojo)
    ggplot2::geom_hline(yintercept = mean(df$Observado, na.rm = TRUE),
                        linetype = "dashed", color = "#E74C3C", linewidth = 0.8) +
    
    ggplot2::labs(
      title = "9. Diagnostico de Estacionariedad",
      subtitle = paste0("Analisis de Media y Varianza Movil (Ventana k = ", ventana_ma, ")"),
      x = "Tiempo (Periodos)",
      y = "Valor de la Serie",
      caption = paste0("INTERPRETACION:\n",
                       " LINEA AZUL: Media local movil. Debe ser horizontal y estable.\n",
                       " LINEA ROJA: Media global (El objetivo del modelo MA).\n",
                       " BIEN: Si la franja azul mantiene su ancho y nivel constante.\n",
                       " NOTA: Se han filtrado los extremos para evitar avisos de valores nulos.")
    ) +
    mi_tema
  
  print(g9)
  
  # CORRECCION 1: message en lugar de cat
  if (interactive()) {
    message("Grafica 9: Verificacion final de estabilidad estadistica completada.")
  }
  
  # CORRECCION 4: Retorno del objeto x original
  return(invisible(x))
}

#' Niveles de agua en presas de Mexico
#'
#' Conjunto de datos que presenta los niveles de almacenamiento de agua
#' registrados en distintas presas y fechas, util para analisis de series de tiempo
#' y modelos ARIMA.
#'
#' @format Un data frame con 1825 observaciones y 2 variables:
#' \describe{
#'   \item{fecha}{Fecha del registro (AAAA-MM-DD)}
#'   \item{nivel_m}{Nivel de almacenamiento de la presa (metros)}
#' }
#'
#' @details Los datos fueron simulados a partir de patrones observados en reportes
#' de CONAGUA, manteniendo coherencia temporal y realismo. Se generaron con fines
#' didacticos para demostrar la aplicacion de modelos AR, MA y ARIMA en \pkg{GREENREG}.
#'
#' @examples
#' data("datos_nivel_presa")
#' ts_data <- datos_nivel_presa$nivel_m
#' modelo_ARIMA <- modelo_arima(ts_data, p = 1, d = 1, q = 1)
#' print(modelo_ARIMA)
#' plot(modelo_ARIMA)
#'
#' @source Elaboracion propia con fines didacticos basada en patrones de datos de CONAGUA.
"datos_nivel_presa"

#' Presencia de roya del cafe en diferentes localidades
#'
#' Base de datos simulada que presenta la presencia/ausencia de roya del cafe
#' junto con variables ambientales y de manejo agronomico relevantes.
#' Es util para ejercicios de regresion logistica.
#'
#' @format Un data frame con 300 observaciones y 4 variables:
#' \describe{
#'   \item{presencia_roya}{Presencia (1) o ausencia (0) de roya del cafe}
#'   \item{humedad_relativa_pct}{Humedad relativa promedio (%)}
#'   \item{altitud_msnm}{Altitud de la finca (m s.n.m.)}
#'   \item{manejo_sombra}{Porcentaje de cobertura de sombra en la plantacion}
#' }
#'
#' @details Los datos fueron generados de forma controlada utilizando parametros
#' similares a los observados por SIAP y estudios agronomicos del INIFAP.
#' Esta base fue creada para ser utilizada en funciones de regresion logistica del paquete \pkg{GREENREG}.
#'
#' @examples
#' data(datos_roya_cafe)
#' modelo_log <- logistico(presencia_roya ~ humedad_relativa_pct + 
#'                         altitud_msnm + manejo_sombra, 
#'                         data = datos_roya_cafe)
#'                         
#'
#' @source Elaboracion propia con fines didacticos basada en informacion de SIAP e INIFAP.
"datos_roya_cafe"

#' Regresion Logistica Binaria (MLG Logit)
#'
#' Esta funcion ajusta un Modelo Lineal Generalizado (GLM) de la familia binomial para
#' predecir la probabilidad de un evento dicotomico (Si/No, 0/1). A diferencia de la
#' regresion lineal, el motor de calculo estima los coeficientes mediante Maxima
#' Verosimilitud (MLE) y ofrece un diagnostico especializado en clasificacion.
#'
#' @details
#' El flujo interno de la funcion se divide en cinco etapas fundamentales:
#' \enumerate{
#'   \item **Ajuste Logit:** Transforma la variable respuesta mediante la funcion de enlace
#'         \eqn{logit(p) = ln(p / (1-p))}, permitiendo que la probabilidad se mantenga entre 0 y 1.
#'   \item **Conversion de Impacto:** Los coeficientes estimados se transforman automaticamente
#'         a **Odds Ratios (OR)** mediante \eqn{e^\beta}. Esto permite concluir cuanto
#'         multiplica cada variable la posibilidad de que ocurra el evento.
#'   \item **Evaluacion de Bondad (McFadden):** Calcula el Pseudo-\eqn{R^2} comparando la
#'         verosimilitud del modelo ajustado contra un modelo nulo (sin predictores).
#'   \item **Metricas de Clasificacion:** Genera la Matriz de Confusion basada en el \code{umbral}
#'         especificado y calcula la Precision Global (Accuracy).
#'   \item **Diagnostico de Influencia:** Evalua si existen observaciones que deforman
#'         las probabilidades mediante el calculo de la Distancia de Cook y el Apalancamiento (Leverage).
#' }
#'
#' @section Guia de las 9 Graficas de Diagnostico (`plot`):
#' Al ejecutar \code{plot(modelo)}, se despliega una secuencia de validacion predictiva:
#' \describe{
#'   \item{\strong{1. Capacidad de Separacion}}{Muestra si el modelo realmente distingue entre grupos. Buscamos "montanas" de densidad que no se traslapen.}
#'   \item{\strong{2. Matriz de Confusion Visual}}{Mapa de calor que resume aciertos (diagonal) y errores (falsos positivos/negativos).}
#'   \item{\strong{3. Odds Ratios (Forest Plot)}}{Visualiza la importancia de las variables. Los puntos a la derecha del 1 son factores de riesgo; a la izquierda, protectores.}
#'   \item{\strong{4. Curva de Calibracion}}{Compara la probabilidad que "dice" el modelo contra la frecuencia real observada. Debe seguir la diagonal.}
#'   \item{\strong{5. Curva ROC (AUC)}}{Mide la potencia global de diagnostico. Un AUC de 1.0 es una clasificacion perfecta; 0.5 es puro azar.}
#'   \item{\strong{6. Distancia de Cook}}{Detecta casos atipicos en la clasificacion que tienen un peso excesivo en los coeficientes.}
#'   \item{\strong{7. Trade-off de Umbrales}}{Permite ver como cambia la Sensibilidad y Especificidad al mover el punto de corte.}
#'   \item{\strong{8. Residuos de Pearson}}{Busca patrones sistematicos en el error. Idealmente, la linea roja debe mantenerse cerca del cero.}
#'   \item{\strong{9. Leverage vs Residuos}}{Identifica observaciones con combinaciones de variables X muy raras que "jalonean" las probabilidades.}
#' }
#'
#' @param formula Una formula de R (ej. \code{presencia_plaga ~ humedad + altitud}).
#' @param data Un \code{data.frame} con las observaciones.
#' @param umbral Punto de corte para clasificar como "Evento (1)" (por defecto 0.5).
#'
#' @return Un objeto de clase \code{"logistico_greenreg"}, que consiste en una lista con los siguientes componentes:
#' \itemize{
#'   \item \code{formula}: La formula de R aplicada al modelo.
#'   \item \code{modelo}: El objeto de clase \code{glm} original con el ajuste binomial.
#'   \item \code{variables}: Una lista que contiene el nombre de la variable respuesta (\code{y}).
#'   \item \code{coeficientes}: Matriz con las estimaciones de los coeficientes, errores estandar, estadisticos z y p-valores.
#'   \item \code{odds_ratios}: Tabla con los Odds Ratios (OR) calculados, sus intervalos de confianza y p-valores asociados.
#'   \item \code{metricas}: Una lista que incluye el Pseudo-R2 de McFadden, el Criterio de Informacion de Akaike (AIC) y la precision global (\emph{Accuracy}).
#'   \item \code{confusion}: Matriz de confusion que cruza los valores reales frente a los predichos segun el umbral especificado.
#'   \item \code{predicciones}: Un \code{data.frame} con los valores observados, probabilidades predichas, clasificacion final y el apalancamiento (\emph{leverage}).
#'   \item \code{diag_leverage}: Lista con informacion diagnostica sobre el apalancamiento y deteccion de puntos extremos.
#'   \item \code{notas}: Lista de mensajes interpretativos sobre la bondad de ajuste y la precision.
#'   \item \code{supuestos}: Un vector de caracteres con las validaciones tecnicas (semaforos) sobre significancia, multicolinealidad e influencia.
#' }
#'
#'
#' @examples
#' data(datos_roya_cafe)
#' modelo_log <- logistico(presencia_roya ~ humedad_relativa_pct + 
#'                         altitud_msnm + manejo_sombra, 
#'                         data = datos_roya_cafe)
#'                         
#' @importFrom stats glm binomial fitted coef confint formula na.omit predict model.frame logLik aggregate pchisq cooks.distance hatvalues rstandard
#' @importFrom ggrepel geom_text_repel
#' @import ggplot2
#' @export
# Asegurate de usar esta version actualizada de tu funcion principal
logistico <- function(formula, data, umbral = 0.5) {

  # --- 1. Preparacion y Ajuste ---
  # Dejamos que glm maneje el punto (.) y los NAs automaticamente
  modelo_base <- stats::glm(formula, data = data, family = stats::binomial(link = "logit"), na.action = stats::na.omit)
  resumen <- summary(modelo_base)

  # Extraemos los datos EXACTOS que uso el modelo (sin NAs)
  data_model <- modelo_base$model

  # Identificar la variable respuesta (siempre es la 1ra columna del model frame)
  var_y_name <- names(data_model)[1]
  y_real <- modelo_base$y # vector 0/1 real usado en el ajuste

  # --- 2. Interpretacion de Coeficientes (Odds Ratios) ---
  coefs_log <- coef(modelo_base)
  coefs_or <- exp(coefs_log)
  cis_or <- exp(stats::confint(modelo_base)) # Intervalos de confianza para OR

  tabla_or <- cbind(OR = coefs_or, `2.5 %` = cis_or[,1], `97.5 %` = cis_or[,2], `P-valor` = resumen$coefficients[,4])

  # --- 3. Metricas de Bondad de Ajuste ---
  # Pseudo R2 de McFadden: 1 - (LogLik_Modelo / LogLik_Nulo)
  ll_modelo <- as.numeric(logLik(modelo_base))

  # Ajustamos modelo nulo sobre LOS MISMOS DATOS exactos (data_model)
  modelo_nulo <- stats::glm(as.formula(paste(var_y_name, "~ 1")), data = data_model, family = binomial)
  ll_nulo <- as.numeric(logLik(modelo_nulo))

  pseudo_r2 <- 1 - (ll_modelo / ll_nulo)

  # Matriz de Confusion y Accuracy
  probs_pred <- fitted(modelo_base)
  clase_pred <- ifelse(probs_pred > umbral, 1, 0)

  tabla_confusion <- table(Real = y_real, Predicho = clase_pred)
  accuracy <- sum(diag(tabla_confusion)) / sum(tabla_confusion)

  # --- 4. Notas Didacticas ---
  nota_r2 <- paste0("Pseudo R2 (McFadden) = ", round(pseudo_r2, 3),
                    ". (Valores entre 0.2 y 0.4 se consideran excelentes en logistica).")

  nota_acc <- paste0("Precision Global (Accuracy): ", round(accuracy * 100, 1),
                     "%. El modelo clasifico correctamente este porcentaje de casos.")

  # Mensaje sobre Odds Ratios significativos
  significativos <- rownames(tabla_or)[tabla_or[, "P-valor"] < 0.05 & rownames(tabla_or) != "(Intercept)"]
  if (length(significativos) > 0) {
    ejemplo_var <- significativos[1]
    or_val <- tabla_or[ejemplo_var, "OR"]
    direccion <- ifelse(or_val > 1, "AUMENTA", "DISMINUYE")
    nota_interp <- paste0("Ejemplo de interpretacion: La variable '", ejemplo_var,
                          "' es significativa. Su OR es ", round(or_val, 2),
                          ", lo que significa que ", direccion, " la probabilidad del evento.")
  } else {
    nota_interp <- "Ninguna variable parece tener un efecto estadisticamente significativo."
  }
  # --- 4.5 Semaforo de Validacion (Diagnostico Tecnico) ---
  # A) Significancia del Modelo (Likelihood Ratio Test)
  p_modelo <- 1 - pchisq(modelo_base$null.deviance - modelo_base$deviance,
                         df = modelo_base$df.null - modelo_base$df.residual)

  nota_signif <- if(p_modelo < 0.05) "OKEY: El modelo es significativamente mejor que el nulo." else " ALERTA: El modelo no aporta informacion relevante (p > 0.05)."

  # B) Multicolinealidad (VIF)
  vif_vals <- if(requireNamespace("car", quietly = TRUE)) car::vif(modelo_base) else NA
  nota_vif <- if(any(vif_vals > 10, na.rm = TRUE)) " ALERTA: Multicolinealidad alta (VIF > 10). Revisa las variables redundantes." else "OK: No hay multicolinealidad severa."

  # C) Influencia (Puntos Cook > Umbral)
  cooksd <- stats::cooks.distance(modelo_base)
  umbral_cook <- 4 / length(cooksd)
  hay_influencia <- any(cooksd > umbral_cook)
  nota_influencia <- if(hay_influencia) " ALERTA: Se detectaron casos influyentes (Distancia de Cook). Revisa los casos atipicos." else "OKEY: No hay puntos influyentes criticos."

  # --- NUEVO: D) Leverage (Apalancamiento) ---
  leverage <- stats::hatvalues(modelo_base) # Calculamos h
  num_vars <- length(coefs_log) # numero de parametros (incluyendo intercepto)
  n_obs <- length(y_real)
  umbral_leverage <- 2 * num_vars / n_obs # Umbral comun: 2*k/n
  hay_leverage_alto <- any(leverage > umbral_leverage)
  nota_leverage <- if(hay_leverage_alto) " ALERTA: Hay observaciones con Leverage alto (puntos atipicos en X). Usa plot() para ver el grafico." else "OKEY: No hay puntos con apalancamiento extremo."

  # --- 5. Construccion del Objeto Final ---
  # Consolidamos TODO en una sola lista
  resultado <- list(
    formula = formula, # <--- Guardamos la formula (necesario para el print seccion 1)
    modelo = modelo_base,
    variables = list(y = var_y_name),
    coeficientes = resumen$coefficients,
    odds_ratios = tabla_or,
    metricas = list(pseudo_r2 = pseudo_r2, aic = modelo_base$aic, accuracy = accuracy),
    confusion = tabla_confusion,
    predicciones = data.frame(
      obs = y_real,
      prob = probs_pred,
      clasif = clase_pred,
      leverage = leverage # <--- Guardamos leverage para el plot
    ),
    diag_leverage = list(umbral = umbral_leverage, hay_alto = hay_leverage_alto), # Info diagnostica
    notas = list(
      r2 = nota_r2,
      acc = nota_acc,
      interp = nota_interp
    ),
    # Aqui unificamos el semaforo que creaste arriba, incluyendo el nuevo de leverage
    supuestos = c(nota_signif, nota_vif, nota_influencia, nota_leverage)
  )

  class(resultado) <- "logistico_greenreg"
  return(resultado)
}



#' @rdname logistico
#' @param x Objeto de clase logistico_greenreg.
#' @param ... Argumentos adicionales.
#' @export
print.logistico_greenreg <- function(x, ...) {
  
  cat("\n==========================================================\n")
  cat("               REGRESION LOGISTICA \n")
  cat("==========================================================\n\n")
  
  # --- 1. MODELO Y ECUACION ---
  cat("--- 1. MODELO Y ECUACION ---\n")
  cat("Formula aplicada:", deparse(x$formula), "\n")
  
  # CORRECCION CRAN: Uso explicito de stats::
  b <- round(stats::coef(x$modelo), 3)
  vars <- names(b)
  ecuacion <- paste0("logit(p) = ", b[1])
  if(length(b) > 1) {
    for(i in 2:length(b)) {
      ecuacion <- paste0(ecuacion, " + (", b[i], " * ", vars[i], ")")
    }
  }
  cat("Ecuacion estimada:\n", ecuacion, "\n\n")
  
  # --- 2. COEFICIENTES ESTIMADOS ---
  cat("--- 2. COEFICIENTES ESTIMADOS ---\n")
  cat("Estimaciones en escala Logit (log-odds):\n")
  stats::printCoefmat(x$coeficientes, digits = 4, signif.stars = TRUE)
  cat("\nInterpretacion en Odds Ratios (OR):\n")
  print(round(x$odds_ratios, 4))
  cat(" GUIA: OR > 1 Aumenta prob. | OR < 1 Disminuye prob.\n\n")
  
  # --- 3. EVALUACION DEL MODELO ---
  cat("--- 3. EVALUACION DEL MODELO ---\n")
  
  # CORRECCION CRAN: Uso explicito de stats::
  chi_cuad <- x$modelo$null.deviance - x$modelo$deviance
  df_chi <- x$modelo$df.null - x$modelo$df.residual
  
  if (!is.na(chi_cuad) && !is.na(df_chi) && df_chi > 0) {
    p_val_lrt <- 1 - stats::pchisq(chi_cuad, df_chi)
    p_val_txt <- if(is.na(p_val_lrt)) "NA" else sprintf("%.4f", p_val_lrt)
    resultado_txt <- if(!is.na(p_val_lrt) && p_val_lrt < 0.05) "Modelo Globalmente Significativo" else "Modelo No Significativo"
  } else {
    chi_cuad <- NA
    df_chi <- NA
    p_val_txt <- "NA"
    resultado_txt <- "No disponible"
  }
  
  cat("Prueba de Razon de Verosimilitud (LRT):\n")
  cat(sprintf(" Chi-cuadrado: %s | GL: %s | p-valor: %s\n", 
              if(is.na(chi_cuad)) "NA" else round(chi_cuad, 3),
              if(is.na(df_chi)) "NA" else df_chi, 
              p_val_txt))
  cat(" Resultado:", resultado_txt, "\n\n")
  
  # --- 4. BONDAD DE AJUSTE ---
  cat("--- 4. BONDAD DE AJUSTE ---\n")
  
  # CORRECCION CRAN: Uso explicito de stats::
  loglik_val <- try(stats::logLik(x$modelo), silent = TRUE)
  loglik_txt <- if(inherits(loglik_val, "try-error") || is.na(loglik_val)) "NA" else round(as.numeric(loglik_val), 3)
  
  cat(" Log-Verosimilitud (LogLik):", loglik_txt, "\n")
  cat(" AIC (Criterio de Akaike): ", round(x$metricas$aic, 3), "\n")
  cat("*", x$notas$r2, "\n")
  cat(" Prueba de Wald: Revisar asteriscos en la seccion 2 (Significancia individual).\n\n")
  
  # --- 5. MATRIZ DE CONFUSION ---
  cat("--- 5. MATRIZ DE CONFUSION (Real vs Predicho) ---\n")
  print(x$confusion)
  cat("\n INTERPRETACION:\n")
  cat("Buscamos una diagonal principal (Real y Predicho coinciden) con valores altos.\n")
  cat("Los valores fuera de la diagonal representan Falsos Positivos y Falsos Negativos.\n")
  cat("*", x$notas$acc, "\n\n")
  
  # --- 6. VERIFICACION DE SUPUESTOS ---
  cat("--- 6. VERIFICACION DE SUPUESTOS ---\n")
  cat(" Linealidad en el Logit: Supuesta (Usar plot(x) para verificar graficos de residuos).\n")
  cat(" Independencia: Observaciones independientes asumidas por diseno.\n")
  
  # Mostrar las alertas de forma limpia y segura
  if (!is.null(x$supuestos)) {
    for (i in 1:length(x$supuestos)) {
      cat(" - ", x$supuestos[i], "\n")
    }
  }
  cat("\n")
  
  # --- 7. RESUMEN DEL MODELO ---
  cat("--- 7. RESUMEN DEL MODELO ---\n")
  cat("Conclusion rapida:", x$notas$interp, "\n")
  
  hay_alerta <- FALSE
  if (!is.null(x$supuestos)) {
    hay_alerta <- any(grepl("ALERTA", x$supuestos))
  }
  
  if(hay_alerta) {
    cat("\n ALERTA: El modelo presenta advertencias tecnicas (Multicolinealidad, Puntos influyentes o Leverage).\n")
    cat(" Se recomienda usar plot(nombre_del_objeto) para ver los graficos de diagnostico.\n")
  }
  
  cat("\n==========================================================\n")
  
  return(invisible(x))
}
#' @rdname logistico
#' @description Genera una secuencia de 9 graficas para validar el modelo de regresion logistica.
#' @param x Objeto de clase logistico_greenreg.
#' @param ... Argumentos adicionales.
#' @export
plot.logistico_greenreg <- function(x, ...) {
  
  df_preds <- x$predicciones 
  modelo   <- x$modelo
  umbral   <- if(!is.null(x$umbral)) x$umbral else 0.5
  
  mi_tema <- ggplot2::theme_minimal() +
    ggplot2::theme(
      plot.title = ggplot2::element_text(face = "bold", size = 14, color = "#2C3E50"),
      plot.subtitle = ggplot2::element_text(size = 11, color = "#7F8C8D"),
      plot.caption = ggplot2::element_text(hjust = 0, size = 9, face = "italic",
                                           color = "#555555", margin = ggplot2::margin(t = 15),
                                           lineheight = 1.1),
      plot.margin = ggplot2::margin(10, 10, 20, 10),
      axis.title = ggplot2::element_text(face = "bold"),
      legend.position = "top"
    )
  
  # CORRECCION 1: Usar message() en lugar de cat()
  if (interactive()) {
    message("Generando graficas de diagnostico... (Presione [Enter] para avanzar)\n")
  }
  
  # --- GRAFICA 1: SEPARACION DE CLASES (DENSIDAD) ---
  g1 <- ggplot2::ggplot(df_preds, ggplot2::aes(x = prob, fill = factor(obs))) +
    ggplot2::geom_density(alpha = 0.6, color = "white") +
    ggplot2::scale_fill_manual(values = c("0" = "#3498DB", "1" = "#E74C3C"), name = "Clase Real") +
    ggplot2::labs(
      title = "1. Capacidad de Separacion (Discriminacion)",
      subtitle = "Distribucion de las probabilidades predichas para cada grupo",
      x = "Probabilidad Predicha por el Modelo (0 a 1)",
      y = "Densidad",
      caption = "INTERPRETACION:\n BIEN: Dos montanas separadas. La azul a la izquierda (cerca de 0) y la roja a la derecha (cerca de 1).\n MAL: Si las montanas estan una encima de la otra (solapadas), el modelo esta confundido."
    ) + mi_tema
  print(g1)
  if (interactive()) {readline(prompt = "Grafica 1/9 > ")}
  
  # --- GRAFICA 2: MATRIZ DE CONFUSION VISUAL ---
  pred_clase <- ifelse(df_preds$prob > umbral, 1, 0)
  tabla <- table(Real = df_preds$obs, Predicho = pred_clase)
  df_matriz <- as.data.frame(tabla)
  
  g2 <- ggplot2::ggplot(df_matriz, ggplot2::aes(x = Predicho, y = Real, fill = Freq)) +
    ggplot2::geom_tile(color = "white") +
    ggplot2::geom_text(ggplot2::aes(label = Freq), size = 8, color = "white", fontface = "bold") +
    ggplot2::scale_fill_gradient(low = "#95A5A6", high = "#2C3E50") +
    ggplot2::labs(
      title = paste("2. Matriz de Confusion (Umbral:", umbral, ")"),
      subtitle = "Conteo de Aciertos y Errores",
      caption = "INTERPRETACION:\n DIAGONAL (0-0 y 1-1): Son los Aciertos correctos.\n CRUZADA (0-1 y 1-0): Son los Errores (Falsos Positivos y Falsos Negativos).\nObjetivo: Que los cuadros oscuros esten en la diagonal."
    ) + mi_tema
  print(g2)
  if (interactive()) {readline(prompt = "Grafica 2/9 > ")}
  
  # --- GRAFICA 3: ODDS RATIOS (FOREST PLOT MEJORADO) ---
  if (requireNamespace("broom", quietly = TRUE)) {
    coefs <- broom::tidy(modelo, conf.int = TRUE, exponentiate = TRUE)
    coefs <- coefs[coefs$term != "(Intercept)", ]
    
    coefs$Tipo <- "No Significativo"
    coefs$Tipo[coefs$p.value < 0.05 & coefs$estimate > 1] <- "Riesgo (Aumenta Prob)"
    coefs$Tipo[coefs$p.value < 0.05 & coefs$estimate < 1] <- "Protector (Baja Prob)"
    
    g3 <- ggplot2::ggplot(coefs, ggplot2::aes(x = estimate, y = reorder(term, estimate), color = Tipo)) +
      ggplot2::geom_vline(xintercept = 1, linetype = "dashed", color = "gray50") +
      ggplot2::geom_pointrange(ggplot2::aes(xmin = conf.low, xmax = conf.high), size = 0.8) +
      ggplot2::scale_color_manual(values = c("Riesgo (Aumenta Prob)" = "#E74C3C",
                                             "Protector (Baja Prob)" = "#27AE60",
                                             "No Significativo" = "#95A5A6")) +
      ggplot2::labs(
        title = "3. Importancia de Variables (Odds Ratios)",
        subtitle = "Impacto de cada variable en la probabilidad del evento",
        x = "Odds Ratio (Escala Logaritmica)", y = "Variables",
        caption = "INTERPRETACION:\n  Derecha del 1: AUMENTA la probabilidad del evento (Riesgo).\n Izquierda del 1: DISMINUYE la probabilidad (Proteccion).\n GRIS (Cruza el 1): La variable NO sirve, estadisticamente no afecta."
      ) + mi_tema
    print(g3)
  } else {
    # CORRECCION 1: Usar message()
    message("Instala el paquete 'broom' para ver la grafica de Odds Ratios: install.packages('broom')")
  }
  if (interactive()) {readline(prompt = "Grafica 3/9 > ")}
  
  # --- GRAFICA 4: CURVA DE CALIBRACION ---
  df_preds$bin <- cut(df_preds$prob, breaks = seq(0, 1, 0.1), include.lowest = TRUE)
  
  calibracion <- aggregate(obs ~ bin, data = df_preds, mean, na.rm=TRUE)
  prob_medias <- aggregate(prob ~ bin, data = df_preds, mean, na.rm=TRUE)
  calibracion$prob_media <- prob_medias$prob
  
  g4 <- ggplot2::ggplot(calibracion, ggplot2::aes(x = prob_media, y = obs)) +
    ggplot2::geom_abline(slope = 1, intercept = 0, linetype = "dashed", color = "gray") +
    ggplot2::geom_line(color = "#8E44AD", linewidth = 1) +
    ggplot2::geom_point(color = "#8E44AD", size = 3) +
    ggplot2::xlim(0,1) + ggplot2::ylim(0,1) +
    ggplot2::labs(
      title = "4. Calibracion del Modelo",
      subtitle = "Dice la verdad el modelo sobre sus probabilidades?",
      x = "Probabilidad que dijo el modelo",
      y = "Porcentaje Real de Eventos",
      caption = "INTERPRETACION:\n BIEN: Los puntos morados siguen la linea gris punteada.\n MAL: Si la curva esta muy por debajo de la linea, el modelo es 'exagerado'."
    ) + mi_tema
  print(g4)
  if (interactive()) {readline(prompt = "Grafica 4/9 > ")}
  
  # --- GRAFICA 5: CURVA ROC ---
  if (requireNamespace("pROC", quietly = TRUE)) {
    roc_obj <- pROC::roc(df_preds$obs, df_preds$prob, quiet = TRUE)
    auc_val <- as.numeric(pROC::auc(roc_obj))
    roc_df <- data.frame(esp = roc_obj$specificities, sens = roc_obj$sensitivities)
    
    g5 <- ggplot2::ggplot(roc_df, ggplot2::aes(x = 1 - esp, y = sens)) +
      ggplot2::geom_line(color = "#D35400", linewidth = 1.2) +
      ggplot2::geom_abline(slope = 1, intercept = 0, linetype = "dashed", color = "gray") +
      ggplot2::labs(
        title = paste0("5. Curva ROC (AUC = ", round(auc_val, 2), ")"),
        subtitle = "Capacidad global de diagnostico",
        x = "Tasa de Falsos Positivos", y = "Sensibilidad (Tasa de Aciertos)",
        caption = "INTERPRETACION:\n BIEN: La curva naranja se acerca a la esquina superior izquierda.\n MAL: Si la curva esta cerca de la diagonal gris, el modelo es puro azar."
      ) + mi_tema
    print(g5)
  } else {
    # CORRECCION 1: Usar message()
    message("Instala el paquete 'pROC' para ver la Curva ROC: install.packages('pROC')")
  }
  if (interactive()) {readline(prompt = "Grafica 5/9 > ")}
  
  # --- GRAFICA 6: VALORES INFLUYENTES (COOK) ---
  cooksd <- stats::cooks.distance(modelo)
  n_datos <- length(cooksd)
  umbral_cook <- 4 / n_datos
  df_cook <- data.frame(Index = 1:n_datos, Cook = cooksd)
  df_cook$Label <- ifelse(df_cook$Cook > umbral_cook, as.character(df_cook$Index), "")
  
  g6 <- ggplot2::ggplot(df_cook, ggplot2::aes(x = Index, y = Cook)) +
    ggplot2::geom_bar(stat = "identity", fill = "#34495E", alpha = 0.8) +
    ggplot2::geom_hline(yintercept = umbral_cook, color = "red", linetype = "dashed") +
    ggplot2::geom_text(ggplot2::aes(label = Label), vjust = -0.5, size = 3) +
    ggplot2::labs(
      title = "6. Datos Influyentes (Distancia de Cook)",
      subtitle = "Deteccion de casos que distorsionan el modelo",
      x = "Numero de Caso", y = "Influencia",
      caption = "INTERPRETACION:\n ALERTA: Las barras altas que cruzan la linea roja son casos atipicos.\nRevisalos en tu base de datos."
    ) + mi_tema
  print(g6)
  if (interactive()) {readline(prompt = "Grafica 6/9 > ")}
  
  # --- GRAFICA 7: BARRIDO DE UMBRALES (Trade-off) ---
  thresholds <- seq(0, 1, length = 100)
  sens <- sapply(thresholds, function(t) mean(df_preds$prob[df_preds$obs == 1] >= t, na.rm=TRUE))
  spec <- sapply(thresholds, function(t) mean(df_preds$prob[df_preds$obs == 0] < t, na.rm=TRUE))
  df_sweep <- data.frame(thresholds, sens, spec)
  
  g7 <- ggplot2::ggplot(df_sweep) +
    ggplot2::geom_line(ggplot2::aes(x = thresholds, y = sens, color = "Sensibilidad"), linewidth = 1) +
    ggplot2::geom_line(ggplot2::aes(x = thresholds, y = spec, color = "Especificidad"), linewidth = 1) +
    ggplot2::geom_vline(xintercept = umbral, linetype = "dashed") +
    ggplot2::scale_color_manual(values = c("Sensibilidad" = "#E74C3C", "Especificidad" = "#2980B9")) +
    ggplot2::labs(
      title = "7. Trade-off: Umbral de Clasificacion",
      subtitle = "Como el umbral afecta los errores",
      x = "Umbral", y = "Tasa", color = "Metrica",
      caption = "INTERPRETACION:\nEl punto de cruce es el umbral 'balanceado'.\nSi necesitas detectar todos los casos, mueve el umbral a la izquierda (mas sensibilidad)."
    ) + mi_tema
  print(g7)
  
  if (interactive()) {readline(prompt = "Grafica 7/9 > ")}
  
  # --- GRAFICA 8: RESIDUOS DE PEARSON ---
  df_preds$pearson_res <- (df_preds$obs - df_preds$prob) / sqrt(df_preds$prob * (1 - df_preds$prob))
  
  g8 <- ggplot2::ggplot(df_preds, ggplot2::aes(x = prob, y = pearson_res)) +
    ggplot2::geom_point(alpha = 0.5, color = "#2C3E50") +
    ggplot2::geom_smooth(method = "loess", color = "#E74C3C", se = FALSE) +
    ggplot2::geom_hline(yintercept = 0, linetype = "dashed") +
    ggplot2::labs(
      title = "8. Analisis de Residuos (Pearson)",
      subtitle = "Existen patrones ocultos en el error?",
      x = "Probabilidad Predicha", y = "Residuo de Pearson",
      caption = "INTERPRETACION:\nSi la linea roja se aleja del 0, el modelo tiene problemas estructurales."
    ) + mi_tema
  print(g8)
  if (interactive()) {readline(prompt = "Grafica 8/9 > ")}
  
  # --- GRAFICA 9: RESIDUOS VS APALANCAMIENTO ---
  lev <- stats::hatvalues(x$modelo)
  res_std <- stats::rstandard(x$modelo) 
  n_datos <- length(lev)
  
  # CORRECCION 2: stats::coef
  p_vars <- length(stats::coef(x$modelo)) 
  umbral_lev <- (2 * p_vars) / n_datos
  
  df_diag <- data.frame(
    index = 1:n_datos,
    leverage = lev,
    resid_std = res_std
  )
  
  df_diag$Label <- ifelse(df_diag$leverage > umbral_lev | abs(df_diag$resid_std) > 2,
                          as.character(df_diag$index), "")
  
  g9 <- ggplot2::ggplot(df_diag, ggplot2::aes(x = leverage, y = resid_std)) +
    ggplot2::geom_point(ggplot2::aes(size = leverage), color = "#C0392B", alpha = 0.6) +
    ggplot2::geom_hline(yintercept = 0, linetype = "dashed", color = "black") +
    ggplot2::geom_hline(yintercept = c(-2, 2), linetype = "dotted", color = "red", alpha = 0.5) +
    ggplot2::geom_vline(xintercept = umbral_lev, linetype = "dashed", color = "blue", alpha = 0.5)
  
  # CORRECCION 4: Proteccion defensiva para ggrepel
  if (requireNamespace("ggrepel", quietly = TRUE)) {
    g9 <- g9 + ggrepel::geom_text_repel(ggplot2::aes(label = Label), size = 3.5, fontface = "bold")
  } else {
    g9 <- g9 + ggplot2::geom_text(ggplot2::aes(label = Label), size = 3.5, vjust = -0.5)
  }
  
  g9 <- g9 +
    ggplot2::labs(
      title = "9. Diagnostico de Influencia (Leverage vs Residuos)",
      subtitle = "Identificando puntos que 'jalonean' el ajuste logistico",
      x = "Apalancamiento (Leverage)",
      y = "Residuos Estandarizados",
      caption = paste0("INTERPRETACION:\n",
                       " EJE X: Puntos a la derecha de la linea azul tienen valores de X muy atipicos.\n",
                       " EJE Y: Puntos fuera de las lineas rojas punteadas son Outliers (errores en clasificacion).\n",
                       " CLAVE: Puntos arriba/abajo a la derecha estan deformando las probabilidades del modelo.")
    ) +
    mi_tema +
    ggplot2::theme(legend.position = "none")
  
  print(g9)
  
  # CORRECCION 1: Usar message()
  if (interactive()) {
    message("Secuencia de graficas completada. Se ha generado la grafica 9 de influencia.\n")
  }
  
  # CORRECCION 3: Retornar el objeto original de forma invisible
  return(invisible(x))
}

#' Regresion Lineal Multiple (RLM)
#'
#' Esta funcion ajusta un modelo de regresion lineal con multiples variables predictoras
#' \eqn{y = \beta_0 + \beta_1 x_1 + \beta_2 x_2 + \dots + \beta_k x_k} mediante Minimos Cuadrados Ordinarios (OLS).
#' Incluye un robusto motor de diagnostico de Multicolinealidad (VIF) y pruebas de supuestos
#' con recomendaciones de accion para el investigador.
#'
#' @details
#' El flujo interno de la funcion se divide en seis etapas:
#' \enumerate{
#'   \item **Validacion de Datos:** Asegura que la formula sea valida y que el dataframe no contenga errores estructurales.
#'   \item **Ajuste (Engine):** Utiliza \code{stats::lm}. En RLM, el foco principal es el **R2 Ajustado**,
#'         que penaliza la inclusion de variables innecesarias que no aportan informacion real.
#'   \item **Deteccion de Multicolinealidad (VIF):** Calcula manualmente el Factor de Inflacion de
#'         la Varianza \eqn{VIF = 1 / (1 - R^2_j)} para cada predictor. Esto identifica si las variables
#'         X estan correlacionadas entre si, lo cual infla los errores estandar y vuelve inestables los coeficientes.
#'   \item **Auditoria de Supuestos:** Ejecuta de forma nativa pruebas de Normalidad (Shapiro),
#'         Homocedasticidad (Breusch-Pagan), Independencia (Durbin-Watson) y Linealidad (Rainbow Test).
#'   \item **Traduccion Automatica:** Evalua los estadisticos y genera "Notas".
#'         y sugerencias de mejora (ej. transformar variables o eliminar predictores redundantes).
#'   \item **Preparacion de Diagnostico:** Construye un dataframe optimizado para \code{ggplot2} que
#'         incluye residuos estandarizados, distancias de Cook y apalancamiento (leverage).
#' }
#'
#' @section Guia de las 10 Graficas de Diagnostico (`plot`):
#' Al ejecutar \code{plot(modelo)}, se despliega una secuencia educativa de 10 visualizaciones:
#' \describe{
#'   \item{\strong{1. Distribucion de Variables (Y, Xi)}}{Muestra histogramas y densidades de todas las variables. Permite detectar sesgos o datos atipicos antes del ajuste.}
#'   \item{\strong{2. Relacion de Variables (Y vs Xi)}}{Scatterplots individuales de la respuesta contra cada predictor con una curva LOESS para verificar la linealidad visualmente.}
#'   \item{\strong{3. Ajuste (Observados vs Ajustados)}}{Muestra la precision global. Los puntos deben seguir la diagonal de 45 grados.}
#'   \item{\strong{4. Q-Q Plot (Normalidad)}}{Valida si los residuos siguen una distribucion normal. Crucial para la validez de los P-valores.}
#'   \item{\strong{5. Homocedasticidad}}{Residuos vs. Predichos. Busca una "nube de estrellas" sin forma de embudo.}
#'   \item{\strong{6. Independencia}}{Residuos vs. Orden de recoleccion. Detecta patrones temporales o espaciales en los errores.}
#'   \item{\strong{7. Prediccion vs Realidad (Identidad)}}{Version simplificada del ajuste para evaluar la correlacion de Pearson.}
#'   \item{\strong{8. Multicolinealidad (VIF)}}{Grafica de barras exclusiva de RLM. Las barras rojas (VIF > 10) indican variables que deben ser eliminadas por redundancia.}
#'   \item{\strong{9. Distancia de Cook}}{Identifica observaciones influyentes. Barras que superan el umbral $4/n$ indican datos que deforman los resultados.}
#'   \item{\strong{10. Leverage vs Residuos}}{Cruza la "rareza" de X (Leverage) con el error en Y (Residuo). Detecta puntos que actuan como palancas sobre la recta.}
#' }
#'
#' @param formula Formula modelo.
#' @param data Dataframe con los datos.
#' @param nivel_confianza Nivel de confianza para intervalos (default 0.95).
#'
#' @return Un objeto de clase \code{"rlm_greenreg"} que contiene:
#' \itemize{
#'   \item \code{modelo}: Objeto \code{lm} original.
#'   \item \code{variables}: Lista con los nombres de Y y los predictores X.
#'   \item \code{coeficientes}: Matriz con estimaciones, p-valores e intervalos de confianza.
#'   \item \code{vif}: Vector con los valores del Factor de Inflacion de la Varianza (VIF).
#'   \item \code{supuestos}: Resultados de los tests de Shapiro-Wilk (Normalidad), Breusch-Pagan 
#'         (Homocedasticidad), Durbin-Watson (Independencia) y Rainbow (Linealidad).
#'   \item \code{notas}: Reporte interpretado con recomendaciones de accion.
#'   \item \code{data_plot}: Dataframe optimizado para visualizaciones.
#' }
#'
#' @examples
#'   data(datos_rendimiento_maiz)
#'   modelo_rlm <- rlm(rendimiento_maiz_ton_ha ~ ., data = datos_rendimiento_maiz)
#'   print(modelo_rlm)
#'   plot(modelo_rlm)
#'
#' @importFrom stats lm anova shapiro.test residuals fitted coef confint sd median cor pf model.matrix cooks.distance hatvalues rstandard
#' @importFrom lmtest bptest dwtest raintest
#' @import ggplot2
#' @export
rlm <- function(formula, data, nivel_confianza = 0.95) {

  # --- 1. Validacion y Ajuste ---
  if (!inherits(formula, "formula")) stop("El argumento 'formula' debe ser una formula (ej. y ~ x1 + x2).")
  if (!is.data.frame(data)) stop("El argumento 'data' debe ser un data.frame.")

  modelo_base <- stats::lm(formula, data = data)
  resumen <- summary(modelo_base)

  # Identificar variables
  var_y <- names(modelo_base$model)[1]
  vars_x <- names(modelo_base$model)[-1] # Todas las predictoras

  # --- 2. Estadisticos Clave ---
  # En RLM, el R2 Ajustado es el rey
  r2_adj <- resumen$adj.r.squared
  sigma <- resumen$sigma
  f_statistic <- resumen$fstatistic

  # P-valor global
  if (!is.null(f_statistic)) {
    p_valor_modelo <- pf(f_statistic[1], f_statistic[2], f_statistic[3], lower.tail = FALSE)
  } else {
    p_valor_modelo <- NA
  }

  coefs <- resumen$coefficients
  intervalos <- stats::confint(modelo_base, level = nivel_confianza)
  tabla_coefs <- cbind(coefs, intervalos)
  tabla_anova <- stats::anova(modelo_base)

  # --- 3. Diagnosticos de Supuestos ---
  residuos <- stats::residuals(modelo_base)

  # A) Normalidad (Shapiro)
  if (length(residuos) >= 3 && length(residuos) <= 5000) {
    p_norm <- stats::shapiro.test(residuos)$p.value
  } else {
    p_norm <- NA
  }

  # B) Homocedasticidad (Breusch-Pagan)
  if (requireNamespace("lmtest", quietly = TRUE)) {
    p_homo <- lmtest::bptest(modelo_base)$p.value
  } else {
    p_homo <- NA
  }

  # C) Independencia (Durbin-Watson)
  if (requireNamespace("lmtest", quietly = TRUE)) {
    dw_res <- lmtest::dwtest(modelo_base)
    stat_dw <- dw_res$statistic
  } else {
    stat_dw <- NA
  }

  # D) MULTICOLINEALIDAD (VIF) - Manual robusto para no depender de 'car'
  # VIF_j = 1 / (1 - R2_j)
  # Calculamos el VIF solo si hay mas de 1 predictor
  vif_vals <- numeric()
  if (length(vars_x) > 1) {
    tryCatch({
      # Matriz de diseno sin intercepto
      X_mat <- stats::model.matrix(modelo_base)[, -1, drop = FALSE]
      if(ncol(X_mat) > 1) {
        for(i in 1:ncol(X_mat)) {
          # Regresion de X_i contra los otros X
          y_aux <- X_mat[, i]
          x_aux <- X_mat[, -i, drop = FALSE]
          r2_aux <- summary(lm(y_aux ~ x_aux))$r.squared
          vif_vals[colnames(X_mat)[i]] <- 1 / (1 - r2_aux)
        }
      }
    }, error = function(e) {
      warning("No se pudo calcular el VIF (posiblemente variables categoricas complejas).")
    })
  }
  # E) LINEALIDAD (Rainbow Test)
  if (requireNamespace("lmtest", quietly = TRUE)) {
    test_lineal <- lmtest::raintest(modelo_base)
    p_lineal <- test_lineal$p.value
  } else {
    p_lineal <- NA
  }
  # --- 4. Notas (Interpretacion) ---

  # R2 Ajustado
  nota_r2 <- paste0("El modelo explica el ", round(r2_adj * 100, 2),
                    "% de la variabilidad (R2 Ajustado). ",
                    ifelse(r2_adj > 0.7, "Ajuste BUENO.",
                           ifelse(r2_adj > 0.4, "Ajuste MODERADO.", "Ajuste BAJO.")))

  # Nota Global
  nota_global <- if (!is.na(p_valor_modelo) && p_valor_modelo < 0.05) {
    "BIEN: El modelo en conjunto es SIGNIFICATIVO (al menos una variable ayuda a predecir)."
  } else {
    "MAL: El modelo en conjunto NO es significativo (ninguna variable parece servir)."
  }

  # Semaforos
  nota_norm <- if (!is.na(p_norm) && p_norm > 0.05) "OKEY: Residuos normales." else " ALERTA: Residuos NO normales."
  nota_homo <- if (!is.na(p_homo) && p_homo > 0.05) "OKEY: Varianza constante." else " ALERTA: Heterocedasticidad detectada."
  nota_indep <- if (!is.na(stat_dw) && stat_dw > 1.5 && stat_dw < 2.5) "OKEY: Independencia." else " ALERTA: Posible autocorrelacion."
  nota_lineal <- if (!is.na(p_lineal) && p_lineal > 0.05) {
    "OKEY: Relacion lineal adecuada."
  } else {
    " ALERTA: Posible No-Linealidad. SUGERENCIA: Prueba transformar variables o agregar terminos cuadraticos."
  }
  # Semaforo VIF
  msg_vif <- "No aplica (1 variable)."
  if (length(vif_vals) > 0) {
    max_vif <- max(vif_vals)
    if (max_vif < 5) {
      msg_vif <- "OKEY: No hay problemas graves de Multicolinealidad (VIF < 5)."
    } else if (max_vif < 10) {
      msg_vif <- " PRECAUCIoN: Hay cierta correlacion entre variables (5 < VIF < 10)."
    } else {
      msg_vif <- "PELIGRO: Multicolinealidad severa (VIF > 10). Tus variables se estorban entre si."
    }
  }

  # --- 5. Objeto Final ---
  resultado <- list(
    modelo = modelo_base,
    variables = list(y = var_y, x = vars_x),
    coeficientes = tabla_coefs,
    anova = tabla_anova,
    vif = vif_vals,
    resumen_estadistico = list(R2_adj = r2_adj, P_valor = p_valor_modelo),
    supuestos = list(norm = p_norm, homo = p_homo, dw = stat_dw, lin = p_lineal),
    notas = list(
      r2 = nota_r2,
      global = nota_global,
      supuestos = c(nota_norm, nota_homo, nota_indep, msg_vif)
    ),
    data_plot = data.frame(
      obs_y = data[[var_y]],
      pred = fitted(modelo_base),
      resid = residuals(modelo_base),
      index = 1:length(residuos)
    )
  )

  class(resultado) <- "rlm_greenreg"
  return(resultado)
}


#' @rdname rlm
#' @param x Objeto de clase rlm_greenreg.
#' @param ... Argumentos adicionales.
#' @export
print.rlm_greenreg <- function(x, ...) {
  
  # 0. Preparacion de la Ecuacion Dinamica
  coefs_vals <- x$coeficientes[, 1]
  nombres_x <- x$variables$x
  
  # Construccion de la cadena: Y = b0 + b1*X1 + b2*X2...
  ecuacion <- paste(round(coefs_vals[1], 4)) # Intercepto
  for(i in 1:length(nombres_x)) {
    val <- round(coefs_vals[i+1], 4)
    signo <- if(val >= 0) " + " else " - "
    ecuacion <- paste0(ecuacion, signo, abs(val), " * ", nombres_x[i])
  }
  
  cat("\n==================================================================\n")
  cat("       REGRESION LINEAL MULTIPLE (RLM)        \n")
  cat("==================================================================\n\n")
  
  # --- 1. MODELO MATEMATICO ESTIMADO ---
  cat("--- 1. MODELO MATEMATICO ESTIMADO ---\n")
  cat(" Variable Dependiente (Y): ", x$variables$y, "\n")
  cat(" Predictores (X):           ", paste(x$variables$x, collapse = ", "), "\n")
  cat(" Ecuacion Estimada:         ", x$variables$y, " =", ecuacion, "\n")
  cat(" Ajuste (R2 Ajustado):      ", round(x$resumen_estadistico$R2_adj * 100, 2), "%\n\n")
  
  # --- 2. COEFICIENTES ESTIMADOS ---
  cat("--- 2. COEFICIENTES ESTIMADOS ---\n")
  # CORRECCION: Se agrega el prefijo stats:: obligatorio para CRAN
  stats::printCoefmat(x$coeficientes[, 1:4], digits = 4, signif.stars = TRUE, has.Pvalue = TRUE)
  
  # Si existe VIF, lo mostramos aqui mismo para contextualizar los coeficientes
  if (length(x$vif) > 0) {
    cat("\nMulticolinealidad (VIF):\n")
    print(round(x$vif, 2))
  }
  cat("\n")
  
  # --- 3. INTERVALOS DE CONFIANZA ---
  cat("--- 3. INTERVALOS DE CONFIANZA ---\n")
  print(round(x$coeficientes[, 5:6], 4))
  cat("\n")
  
  # --- 4. ANALISIS DE VARIANZA (ANOVA) ---
  cat("--- 4. ANALISIS DE VARIANZA (ANOVA) ---\n")
  if (!is.null(x$anova)) {
    # CORRECCION: Se usa printCoefmat para blindar caracteres de significancia de la ANOVA
    stats::printCoefmat(x$anova, digits = 4, pdigits = 4, has.Pvalue = TRUE, na.print = "NA")
  } else {
    cat(" No se encontro la tabla ANOVA en el objeto.\n")
  }
  cat("\n")
  
  # --- 5. VERIFICACION DE SUPUESTOS ---
  cat("--- 5. VERIFICACION DE SUPUESTOS ---\n")
  
  limpiar_txt <- function(txt) {
    if(is.null(txt) || is.na(txt) || length(txt) == 0) return("No disponible")
    res <- strsplit(txt, ": ")[[1]]
    if(length(res) > 1) return(res[2]) else return(res[1])
  }
  nota_lin   <- limpiar_txt(x$notas$supuestos[4]) # Linealidad guardada en la posicion 4
  nota_norm  <- limpiar_txt(x$notas$supuestos[1])
  nota_homo  <- limpiar_txt(x$notas$supuestos[2])
  nota_indep <- limpiar_txt(x$notas$supuestos[3])
  p_lin_txt  <- if(is.na(x$supuestos$lin))  "NA     " else sprintf("%-7.4f", x$supuestos$lin)
  p_norm_txt <- if(is.na(x$supuestos$norm)) "NA     " else sprintf("%-7.4f", x$supuestos$norm)
  p_homo_txt <- if(is.na(x$supuestos$homo)) "NA     " else sprintf("%-7.4f", x$supuestos$homo)
  dw_txt     <- if(is.na(x$supuestos$dw))   "NA      " else sprintf("%-8.2f", x$supuestos$dw)
  
  cat(sprintf(" Linealidad (Rainbow Test):   [p-valor: %s] -> %s\n", p_lin_txt, nota_lin))
  cat(sprintf(" Normalidad (Shapiro-Wilk):   [p-valor: %s] -> %s\n", p_norm_txt, nota_norm))
  cat(sprintf(" Homocedasticidad (B-Pagan):  [p-valor: %s] -> %s\n", p_homo_txt, nota_homo))
  cat(sprintf(" Independencia (D-Watson):    [Estad.: %s] -> %s\n", dw_txt, nota_indep))
  
  # 5. VIF (Solo si hay mas de 1 X)
  if (length(x$variables$x) > 1) {
    cat(sprintf(" Multicolinealidad (VIF):      %s\n",
                limpiar_txt(x$notas$supuestos[length(x$notas$supuestos)])))
  }
  
  # --- 6. RESUMEN DEL MODELO ---
  cat("\n--- 6. RESUMEN DEL MODELO ---\n")
  cat("GLOBAL:    ", x$notas$global, "\n")
  cat("AJUSTE:    ", x$notas$r2, "\n")
  
  # Verificacion rapida de fallos
  hay_error <- any(grepl(" ERROR", x$notas$supuestos))
  cat("VALIDEZ:   ", if(hay_error) "PRECAUCION: Se detectaron fallas en los supuestos fundamentales."
      else "El modelo cumple con los supuestos basicos.", "\n")
  
  cat("\nUse 'plot(modelo)' para ver diagnosticos visuales.\n")
  cat("==================================================================\n\n")
  return(invisible(x))
}

#' @rdname rlm
#' @description Genera una secuencia de 10 graficas para validar el modelo.
#' @param x Objeto de clase rlm_greenreg.
#' @param ... Argumentos adicionales.
#' @importFrom stats cor coef hatvalues rstandard cooks.distance
#' @importFrom ggrepel geom_text_repel
#' @export
plot.rlm_greenreg <- function(x, ...) {
  df <- x$data_plot
  var_y <- x$variables$y

  # Tema base
  mi_tema <- ggplot2::theme_minimal() +
    ggplot2::theme(
      plot.title = ggplot2::element_text(face = "bold", size = 14, color = "#2C3E50"),
      plot.subtitle = ggplot2::element_text(size = 11, color = "#34495E"),
      plot.caption = ggplot2::element_text(hjust = 0, size = 10, face = "italic", color = "#555555", margin = ggplot2::margin(t = 10)),
      axis.title = ggplot2::element_text(face = "bold")
    )

  if (interactive()) {
    message("\nGenerando graficas de diagnostico... (Presione [Enter] para avanzar)\n")
  }
  
  # ==============================================================================
  # --- GRAFICA 1: HISTOGRAMAS (Y, Xi) (DISTRIBUCION) ---
  # ==============================================================================
  datos_raw <- x$modelo$model
  df_dist <- data.frame(
    Valor = as.numeric(unlist(datos_raw)),
    Variable = rep(names(datos_raw), each = nrow(datos_raw))
  )

  g1 <- ggplot2::ggplot(df_dist, ggplot2::aes(x = Valor, fill = Variable)) +
    ggplot2::geom_histogram(ggplot2::aes(y = ggplot2::after_stat(density)),
                            bins = 15, color = "white", alpha = 0.7) +
    ggplot2::geom_density(color = "#E74C3C", linewidth = 1) +
    ggplot2::facet_wrap(~Variable, scales = "free") +
    ggplot2::scale_fill_viridis_d(option = "mako", begin = 0.3, end = 0.8) +
    ggplot2::labs(
      title = "1. Distribucion de Variables (Y, Xi)",
      subtitle = "Como se distribuyen los datos originales antes del ajuste?",
      x = "Valor observado",
      y = "Densidad / Frecuencia",
      caption = paste0("INTERPRETACION:\n",
                       "BIEN: Las variables muestran distribuciones equilibradas (campanas).\n",
                       " ALERTA: Si una variable esta muy sesgada (cola larga), podria afectar los coeficientes.\n",
                       " NOTA: La linea roja ayuda a identificar la forma de la distribucion.")
    ) +
    mi_tema +
    ggplot2::theme(legend.position = "none")
  
  print(g1)
  if (interactive()) {readline(prompt = "Grafica 1/10 (Distribucion Y, Xi) > ")}

  # ==============================================================================
  # --- GRAFICA 2: RELACION DE VARIABLES (Y vs Xi) ---
  # ==============================================================================
  datos_raw <- x$modelo$model
  var_y_name <- names(datos_raw)[1]

  df_relacion <- data.frame()
  for(col in names(datos_raw)[-1]) {
    temp <- data.frame(
      Y = datos_raw[[1]],
      X_val = datos_raw[[col]],
      X_name = col
    )
    df_relacion <- rbind(df_relacion, temp)
  }
  
  g2 <- ggplot2::ggplot(df_relacion, ggplot2::aes(x = X_val, y = Y)) +
    ggplot2::geom_point(color = "#2C3E50", size = 2, alpha = 0.5) +
    ggplot2::geom_smooth(method = "loess", color = "#3498DB", fill = "#3498DB", alpha = 0.2, linewidth = 1) +
    ggplot2::facet_wrap(~X_name, scales = "free_x") +
    ggplot2::labs(
      title = "2. Relacion de Variables (Y frente a Predictoras)",
      subtitle = paste("Visualizando la tendencia individual de cada X respecto a", var_y_name),
      x = "Valor de la Variable Predictora (Xi)",
      y = paste("Variable Respuesta (", var_y_name, ")"),
      caption = paste0("INTERPRETACION:\n",
                       "BIEN: Se observa una tendencia clara (lineal o curva suave) en los puntos.\n",
                       "MAL: Nubes de puntos totalmente dispersas sugieren que esa variable no aporta al modelo.\n",
                       "NOTA: La linea azul ayuda a identificar si la relacion es realmente lineal.")
    ) +
    mi_tema
  
  print(g2)
  if (interactive()) {readline(prompt = "Grafica 2/10 (Relacion Y vs Xi) > ")}

  # ==============================================================================
  # --- GRAFICA 3: AJUSTE (OBSERVADOS VS AJUSTADOS) ---
  # ==============================================================================
  cor_ajuste <- stats::cor(df$obs_y, df$pred)

  g3 <- ggplot2::ggplot(df, ggplot2::aes(x = obs_y, y = pred)) +
    ggplot2::geom_abline(slope = 1, intercept = 0, linetype = "dashed", color = "#E74C3C", linewidth = 1) +
    ggplot2::geom_point(color = "#2C3E50", size = 3, alpha = 0.6) +
    ggplot2::geom_smooth(method = "lm", color = "#3498DB", fill = "#3498DB", alpha = 0.1, linetype = "dotted") +
    ggplot2::labs(
      title = "3. Ajuste: Valores Observados vs. Ajustados",
      subtitle = paste("Correlacion de ajuste (Pearson):", round(cor_ajuste, 4)),
      x = paste("Valores Reales (", x$variables$y, ")"),
      y = "Valores Predichos por el Modelo (Y)",
      caption = paste0("INTERPRETACION:\n",
                       "BIEN: Los puntos se agrupan estrechamente sobre la linea roja diagonal.\n",
                       "MAL: Nube muy dispersa o con forma curvada. El modelo no esta capturando la realidad.\n",
                       "CLAVE: La linea roja representa la perfeccion (Predicho = Real).")
    ) +
    mi_tema

  print(g3)
  if (interactive()) {readline(prompt = "Grafica 3/10 (Observados vs Ajustados) > ")}

  # ==============================================================================
  # --- GRAFICA 4: NORMALIDAD QQ ---
  # ==============================================================================
  g4 <- ggplot2::ggplot(df, ggplot2::aes(sample = resid)) +
    ggplot2::geom_qq(color = "#3498DB") +
    ggplot2::geom_qq_line(color = "#E74C3C", linetype = "dashed") +
    ggplot2::labs(
      title = "4. Grafico Q-Q (Validacion de Normalidad)",
      subtitle = "Los puntos deben seguir la linea diagonal solida.",
      x = "Cuantiles Teoricos",
      y = "Cuantiles de los Residuos",
      caption = "INTERPRETACION:\n BIEN: Los puntos estan pegados a la linea (como hormigas en fila).\n CUIDADO: Si los puntos de los extremos se curvan o alejan mucho,\ntus pruebas de significancia (P-Values) podrian ser poco fiables."
    ) + mi_tema
  
  print(g4) # CORRECCION CRAN: Se ejecuta siempre fuera del bloque condicional
  if (interactive()) {readline(prompt = "Grafica 4/10 (NORMALIDAD) > ")}

  # ==============================================================================
  # --- GRAFICA 5: HOMOCEDASTICIDAD ---
  # ==============================================================================
  g5 <- ggplot2::ggplot(df, ggplot2::aes(x = pred, y = resid)) +
    ggplot2::geom_point(color = "#8E44AD", alpha = 0.7) +
    ggplot2::geom_hline(yintercept = 0, linetype = "dashed") +
    ggplot2::labs(
      title = "5. Homocedasticidad (Varianza Constante)",
      subtitle = "Busca una 'nube aleatoria' (cielo estrellado).",
      x = "Valores Predichos",
      y = "Errores (Residuos)",
      caption = "INTERPRETACION:\n BIEN: Una nube rectangular sin forma definida.\n MAL (Embudo): Si los puntos se abren como un abanico <, la varianza cambia (error grave).\n MAL (Curva): Si ves una forma de 'U', te falta elevar una variable al cuadrado."
    ) + mi_tema
  
  print(g5)
  if (interactive()) {readline(prompt = "Grafica 5/10 (HOMOCEDASTICIDAD) > ")}

  # ==============================================================================
  # --- GRAFICA 6: INDEPENDENCIA ---
  # ==============================================================================
  g6 <- ggplot2::ggplot(df, ggplot2::aes(x = index, y = resid)) +
    ggplot2::geom_line(color = "#95A5A6") +
    ggplot2::geom_point(color = "#D35400") +
    ggplot2::geom_hline(yintercept = 0, linetype = "dashed") +
    ggplot2::labs(
      title = "6. Independencia de los Errores",
      subtitle = "Ves algun patron repetitivo u olas en los datos?",
      caption = "INTERPRETACION:\n BIEN: Ruido puro, picos arriba y abajo sin orden aparente.\n MAL: Si ves olas suaves, zig-zags constantes o grupos, \nsignifica que el pasado afecta al futuro (falta una variable temporal)."
    ) + mi_tema
  
  print(g6)
  if (interactive()) {readline(prompt = "Grafica 6/10 (INDEPENDENCIA) > ")}

  # ==============================================================================
  # --- GRAFICA 7: OBSERVADOS VS PREDICHOS ---
  # ==============================================================================
  g7 <- ggplot2::ggplot(df, ggplot2::aes(x = obs_y, y = pred)) +
    ggplot2::geom_point(color = "#2980B9", alpha = 0.6, size = 2.5) +
    ggplot2::geom_abline(intercept = 0, slope = 1, color = "red", linetype = "dashed", linewidth = 1) +
    ggplot2::labs(
      title = "7. Prediccion vs Realidad (Precision)",
      subtitle = "Lo ideal es que los puntos caigan sobre la linea roja discontinua.",
      x = "Valor Real (Observado)",
      y = "Valor Predicho por el Modelo",
      caption = "INTERPRETACION:\n BIEN: Los puntos forman una linea diagonal estrecha (45 grados).\n MAL: Si ves una nube redonda dispersa, el modelo no esta prediciendo nada util."
    ) + mi_tema
  
  print(g7)
  if (interactive()) {readline(prompt = "Grafica 7/10(OBSERVADOS VS PREDICHOS)  > ")}

  # ==============================================================================
  # --- GRAFICA 8: VIF (MULTICOLINEALIDAD) ---
  # ==============================================================================
  if (length(x$vif) > 0) {
    df_vif <- data.frame(Variable = names(x$vif), VIF = x$vif)
    g8 <- ggplot2::ggplot(df_vif, ggplot2::aes(x = Variable, y = VIF, fill = VIF > 10)) +
      ggplot2::geom_col() +
      ggplot2::geom_hline(yintercept = 10, color = "red", linetype = "dashed") +
      ggplot2::scale_fill_manual(values = c("TRUE" = "#E74C3C", "FALSE" = "#2ECC71"), guide = "none") +
      ggplot2::labs(
        title = "8. Multicolinealidad (Inflacion de Varianza)",
        subtitle = "Variables que dicen lo mismo (Redundancia).",
        y = "Valor VIF",
        x = "Variables Predictoras",
        caption = "INTERPRETACION:\n BIEN (Verde): La variable aporta informacion unica.\n MAL (Rojo): VIF > 10. Esta variable es redundante (es combinacion de otras).\n ACCION: Elimina las variables con barra roja para estabilizar el modelo."
      ) + mi_tema
    
    print(g8)
    if (interactive()) {readline(prompt = "Grafica 8/10 (MULTICOLINEALIDAD) > ")}
  } else {
    if (interactive()) message("Grafica de VIF no generada (solo hay 1 variable predictora).")
  }

  # ==============================================================================
  # --- GRAFICA 9: DISTANCIA DE COOK ---
  # ==============================================================================
  cooksd <- stats::cooks.distance(x$modelo)
  n_datos <- length(cooksd)
  umbral_cook <- 4 / n_datos

  df_cook <- data.frame(Index = 1:n_datos, Cook = cooksd)
  df_cook$Label <- ifelse(df_cook$Cook > umbral_cook, as.character(df_cook$Index), "")

  g9 <- ggplot2::ggplot(df_cook, ggplot2::aes(x = Index, y = Cook)) +
    ggplot2::geom_bar(stat = "identity", fill = "#8E44AD", alpha = 0.7) +
    ggplot2::geom_hline(yintercept = umbral_cook, color = "red", linetype = "dashed") +
    ggplot2::geom_text(ggplot2::aes(label = Label), vjust = -0.5, size = 3) +
    ggplot2::labs(
      title = "9. Valores Influyentes (Distancia de Cook)",
      subtitle = "Algun dato esta forzando al modelo a cambiar?",
      x = "Numero de Observacion (Fila)",
      y = "Influencia (Cook)",
      caption = "INTERPRETACION:\n ALERTA: Las barras que cruzan la linea roja son datos sospechosos.\n ACCION: Revisa esas filas en tu Excel. Son errores de dedo o casos especiales?\nSi son errores, eliminalos y corre el modelo de nuevo."
    ) + mi_tema

  print(g9)
  if (interactive()) {readline(prompt = "Grafica 9/10 (INFLUENCIA) > ")}

  # ==============================================================================
  # --- GRAFICA 10: RESIDUOS VS APALANCAMIENTO ---
  # ==============================================================================
  lev <- stats::hatvalues(x$modelo)
  res_std <- stats::rstandard(x$modelo)
  n_datos <- length(lev)
  p_params <- length(stats::coef(x$modelo))

  umbral_lev <- 2 * (p_params / n_datos)

  df_diag <- data.frame(
    Index = 1:n_datos,
    Leverage = lev,
    ResidStd = res_std
  )

  df_diag$Label <- ifelse(df_diag$Leverage > umbral_lev | abs(df_diag$ResidStd) > 2,
                          as.character(df_diag$Index), "")

  g10 <- ggplot2::ggplot(df_diag, ggplot2::aes(x = Leverage, y = ResidStd)) +
    ggplot2::geom_point(ggplot2::aes(size = Leverage), color = "#C0392B", alpha = 0.6) +
    ggplot2::geom_hline(yintercept = 0, linetype = "dashed", color = "black") +
    ggplot2::geom_hline(yintercept = c(-2, 2), linetype = "dotted", color = "red", alpha = 0.5) +
    ggplot2::geom_vline(xintercept = umbral_lev, linetype = "dashed", color = "blue", alpha = 0.5) +
    
    # CORRECCION: Usamos ggrepel ya que se encuentra declarado en las dependencias de arriba
    ggrepel::geom_text_repel(ggplot2::aes(label = Label), size = 3, fontface = "bold") +
    
    ggplot2::labs(
      title = "10. Diagnostico de Influencia (Leverage vs Residuos)",
      subtitle = "Identificacion de puntos con combinaciones de variables extremas",
      x = "Apalancamiento (Leverage)",
      y = "Residuos Estandarizados",
      caption = paste0("INTERPRETACION:\n",
                       " EJE X: Puntos a la derecha de la linea azul tienen valores de X inusuales.\n",
                       " EJE Y: Puntos fuera de las lineas rojas (2, -2) son Outliers.\n",
                       " PELIGRO: Si un punto esta muy a la derecha Y muy lejos del 0 en Y, es una 'palanca' que deforma el modelo.")
    ) +
    mi_tema +
    ggplot2::theme(legend.position = "none")

  print(g10)
  if (interactive()) {readline(prompt = "Grafica 10/10 (LEVERAGE) > ")}

  if (interactive()) {
    message("\nGraficas de diagnostico completadas.\n")
  }
  
  # CORRECCION CRAN: Devolvemos el objeto original visible/invisible segun el estandar S3
  return(invisible(x))
}

#' Rendimiento de maiz bajo distintos niveles de fertilizacion
#'
#' Conjunto de datos que presenta el rendimiento del maiz en funcion
#' de diferentes variables climaticas y de manejo del cultivo.
#' Este dataset es util para ejemplificar modelos de regresion lineal simple y multiple.
#'
#' @format Un data frame con 250 observaciones y 4 variables:
#' \describe{
#'   \item{rendimiento_maiz_ton_ha}{Rendimiento del cultivo de maiz (ton/ha)}
#'   \item{precipitacion_mm}{Precipitacion registrada durante el ciclo agricola (mm)}
#'   \item{temperatura_c}{Temperatura media (C)}
#'   \item{fertilizante_kg_ha}{Cantidad de fertilizante aplicado (kg/ha)}
#' }
#'
#' @details Los datos fueron generados de forma controlada, tomando como referencia
#' distribuciones y rangos reales reportados por SIAP e INEGI. Esta base se diseno
#' especificamente para probar las funciones de regresion del paquete \pkg{GREENREG}.
#'
#' @examples
#' data(datos_rendimiento_maiz)
#' modelo_rlm <- rlm(rendimiento_maiz_ton_ha ~., data = datos_rendimiento_maiz)
#' modelo_rlm       
#' plot(modelo_rlm)
#'
#' @source Elaboracion propia con fines didacticos tomando referencia de datos del SIAP e INEGI.
"datos_rendimiento_maiz"

#' Anomalias de temperatura media diaria
#'
#' Conjunto de datos que muestra las anomalias (diferencias respecto a la media historica)
#' de la temperatura media diaria registradas a lo largo del tiempo. Este dataset puede
#' utilizarse para ilustrar analisis de series de tiempo y modelos autorregresivos o ARIMA.
#'
#' @format Un data frame con 730 observaciones y 2 variables:
#' \describe{
#'   \item{fecha}{Fecha del registro (AAAA-MM-DD)}
#'   \item{anomalia_c}{Anomalia de temperatura respecto al promedio historico (C)}
#' }
#'
#' @details Los datos fueron construidos de manera simulada a partir de patrones reales
#' observados en bases climaticas del INEGI y de la CONAGUA. La creacion propia de esta
#' base permite garantizar que los valores sean adecuados para fines didacticos y
#' compatibles con las funciones del paquete \pkg{GREENREG}.
#'
#' @examples
#' data("datos_anomalia_temperatura")
#' modelo_AR <- modelo_ar(datos_anomalia_temperatura$anomalia_c, p = 2)
#' modelo_AR
#' plot(modelo_AR)
#' 
#' @source Elaboracion propia con fines didacticos a partir de informacion publica de INEGI y CONAGUA.
"datos_anomalia_temperatura"

# R/globals.R

if(getRversion() >= "2.15.1") {
  utils::globalVariables(c(
    "ACF", "Ajustado", "Conteo", "Cook", "Distancia", "Etiqueta", "Freq", 
    "Fuente", "High", "ID", "IRR", "Imaginario", "Index", "Label", "Lag", 
    "Leverage", "Low", "Lower", "Media_Movil", "Observado", "PACF", "P_val", 
    "Predicho", "Real", "ResidStd", "Residuos", "SD_Movil", "Tiempo", "Tipo", 
    "Upper", "VIF", "Valor", "Variable", "X_val", "Y", "ajustados", 
    "conf.high", "conf.low", "density", "dnorm", "esp", "estimate", 
    "index", "leverage", "obs", "obs_x", "obs_y", "pearson_res", "pred", 
    "predicho", "prob", "prob_media", "res_sq", "resid", "resid_std", 
    "residuos", "term", "vjust_pos", "y"
  ))
}

#' @importFrom stats as.formula density dnorm printCoefmat reorder resid
#' @importFrom utils globalVariables
NULL

#' Modelo Autorregresivo (AR)
#'
#' Esta funcion ajusta un modelo de series de tiempo donde el valor actual \eqn{(Y_t)}
#' se explica mediante una combinacion lineal de sus propios valores pasados
#' \eqn{(Y_{t-1}, Y_{t-2}, \dots)}. Es la herramienta estandar para analizar la
#' "persistencia" o "memoria" de un fenomeno a traves del tiempo.
#'
#' @details
#' El flujo interno de la funcion se divide en cinco etapas diagnosticas:
#' \enumerate{
#'   \item **Estimacion (MLE):** Utiliza Maxima Verosimilitud para encontrar los
#'         coeficientes \eqn{\phi} (Phi) que mejor describen la dependencia temporal.
#'   \item **Analisis de Estabilidad:** Calcula las raices inversas del polinomio
#'         caracteristico. Un modelo AR solo es valido si es "Estacionario", es decir,
#'         si las raices se encuentran dentro del circulo unitario.
#'   \item **Inferencia de Parametros:** Calcula errores estandar y p-valores para
#'         determinar cuantos rezagos (lags) son realmente significativos.
#'   \item **Auditoria de Residuos:** Evalua si el error restante es "Ruido Blanco"
#'         mediante la prueba de Ljung-Box (independencia) y Shapiro-Wilk (normalidad).
#'   \item **Traduccion Didactica:** Genera un reporte que explica si el modelo
#'         "explota" (inestable) o si ha capturado exitosamente la memoria de la serie.
#' }
#'
#' @section Guia de las 9 Graficas de Diagnostico (`plot`):
#' Al ejecutar \code{plot(modelo)}, se despliega una auditoria temporal completa:
#' \describe{
#'   \item{\strong{1. Inspeccion Visual}}{Muestra la serie bruta con su media historica. Ayuda a detectar tendencias que el modelo AR no puede manejar por si solo.}
#'   \item{\strong{2. Ajuste Temporal}}{Superpone el modelo sobre la realidad. La sombra roja indica la incertidumbre (Intervalo de Confianza).}
#'   \item{\strong{3. Residuos en el Tiempo}}{Un "electrocardiograma" del error. No debe presentar ondas, ciclos ni tendencias.}
#'   \item{\strong{4. Circulo Unitario}}{Grafica matematica critica. Si los rombos rojos salen del circulo, el modelo es inestable y no sirve para pronosticar.}
#'   \item{\strong{5. ACF de Residuos}}{Busca correlaciones remanentes. Las barras deben ser pequenas para confirmar que el error es ruido aleatorio.}
#'   \item{\strong{6. PACF de Residuos}}{Ayuda a confirmar si el orden \code{p} seleccionado fue suficiente para limpiar la dependencia directa.}
#'   \item{\strong{7. Normalidad (Q-Q Plot)}}{Valida que los choques aleatorios sigan una distribucion normal, requisito para que los intervalos de confianza sean exactos.}
#'   \item{\strong{8. Precision (Real vs Ajustado)}}{Muestra la correlacion directa entre la prediccion y el dato observado.}
#'   \item{\strong{9. Estacionariedad Movil}}{Analiza si la media y la varianza cambian a lo largo de la serie. Una linea verde horizontal confirma un modelo robusto.}
#' }
#'
#' @param x Vector numerico o serie de tiempo (\code{ts}). No debe contener valores \code{NA}.
#' @param p Orden del modelo (numero de rezagos a considerar). Por defecto es 1.
#' @param nivel_confianza Nivel de significancia para las bandas de error (default 0.95).
#'
#' @return Un objeto de clase \code{"ar_greenreg"} que contiene:
#' \itemize{
#'   \item \code{modelo}: Objeto \code{arima} original.
#'   \item \code{orden_p}: El orden del modelo ajustado.
#'   \item \code{coeficientes}: Matriz con estimaciones, errores estandar y p-valores.
#'   \item \code{estabilidad}: Lista con las raaces inversas y diagnostico de estacionariedad.
#'   \item \code{diagnosticos}: Resultados de las pruebas de Ljung-Box, Shapiro-Wilk y AIC.
#'   \item \code{notas}: Reporte interpretado sobre estabilidad y ruido blanco.
#'   \item \code{data_plot}: Dataframe listo para visualización avanzada.
#' }
#' 
#' @examples
#'
#' data("datos_anomalia_temperatura")
#' modelo_AR <- modelo_ar(datos_anomalia_temperatura$anomalia_c, p = 2)
#' modelo_AR
#' plot(modelo_AR)
#'
#' @importFrom stats arima coef residuals fitted Box.test pnorm qnorm shapiro.test acf pacf filter sd
#' @importFrom ggrepel geom_text_repel
#' @import ggplot2
#' @export
modelo_ar <- function(x, p = 1, nivel_confianza = 0.95) {

  # --- 1. Validacion y Ajuste ---
  if (!is.numeric(x)) stop("El argumento 'x' debe ser numerico o una serie de tiempo (ts).")

  # Ajuste usando ARIMA con q=0, d=0 (es decir, un AR puro)
  modelo_base <- stats::arima(x, order = c(p, 0, 0))

  # --- 2. Coeficientes e Interpretacion ---
  coefs <- coef(modelo_base)
  # Error estandar aproximado (matriz de varianza-covarianza)
  se <- sqrt(diag(modelo_base$var.coef))
  z_val <- coefs / se
  p_val <- 2 * (1 - stats::pnorm(abs(z_val)))

  tabla_coefs <- data.frame(
    Estimacion = coefs,
    Std.Error = se,
    z.value = z_val,
    P.valor = p_val
  )

  # --- 3. Analisis de Estabilidad (Raices) ---
  # El modelo AR es estable si las raices inversas del polinomio estan dentro del circulo unitario
  # Extraemos solo los coeficientes AR (sin intercepto)
  ar_coefs <- coefs[grepl("^ar", names(coefs))]

  if (length(ar_coefs) > 0) {
    poly_roots <- polyroot(c(1, -ar_coefs))
    inverse_roots <- 1 / poly_roots
    modulos <- Mod(inverse_roots)
    es_estable <- all(modulos < 1)
  } else {
    inverse_roots <- NULL
    es_estable <- TRUE # AR(0) es ruido blanco, siempre estable
  }

  # --- 4. Diagnosticos de Residuos ---
  residuos <- stats::residuals(modelo_base)

  # A) Test Ljung-Box (Independencia / Ruido Blanco)
  lb_test <- stats::Box.test(residuos, type = "Ljung-Box", lag = p + 5)
  es_ruido_blanco <- lb_test$p.value > 0.05

  # B) Test Shapiro-Wilk (Normalidad)
  if (length(residuos) >= 3 && length(residuos) <= 5000) {
    shapiro_test <- stats::shapiro.test(residuos)
    es_normal <- shapiro_test$p.value > 0.05
  } else {
    shapiro_test <- list(p.value = NA)
    es_normal <- NA
  }

  # --- 5. Notas ---
  nota_estabilidad <- if (es_estable) {
    " CONCLUSION: El modelo es ESTABLE (Estacionario). Puede usarse para predecir."
  } else {
    " CONCLUSION: El modelo es INESTABLE (No Estacionario). No es confiable a largo plazo."
  }

  nota_lb <- if (es_ruido_blanco) {
    " CONCLUSION: Ruido Blanco (OK). No queda informacion util en los errores."
  } else {
    " CONCLUSION: Existe Autocorrelacion. El modelo no capturo toda la dinamica (Faltan lags)."
  }

  nota_norm <- if (!is.na(es_normal) && es_normal) {
    " CONCLUSION: Los errores son Normales (OK). Los intervalos de confianza son fiables."
  } else {
    " CONCLUSION: Los errores NO son normales. Precaucion con los intervalos."
  }

  # Interpretacion del primer lag
  phi1 <- if(length(ar_coefs) > 0) ar_coefs[1] else 0
  nota_phi <- paste0("Persistencia: El coeficiente AR1 es ", round(phi1, 2), ". ",
                     "Esto indica que tanto del valor de ayer (t-1) se transfiere a hoy.")

  # --- 6. Preparar Datos para Graficas ---
  # Valores ajustados: Real - Residuo
  ajustados <- x - residuos

  # Calculo de Intervalos de Confianza (Shadows)
  sigma <- sqrt(modelo_base$sigma2)
  z_score <- stats::qnorm(1 - (1 - nivel_confianza) / 2)
  lower_ci <- ajustados - z_score * sigma
  upper_ci <- ajustados + z_score * sigma

  df_plot <- data.frame(
    Tiempo = 1:length(x),
    Observado = as.numeric(x),
    Ajustado = as.numeric(ajustados),
    Lower = as.numeric(lower_ci),
    Upper = as.numeric(upper_ci),
    Residuos = as.numeric(residuos)
  )

  # Datos para circulo unitario (si hay raices)
  df_raices <- NULL
  if (!is.null(inverse_roots)) {
    df_raices <- data.frame(
      Real = Re(inverse_roots),
      Imaginario = Im(inverse_roots)
    )
  }

  resultado <- list(
    x_original = x,
    modelo = modelo_base,
    orden_p = p,
    coeficientes = tabla_coefs,
    estabilidad = list(es_estable = es_estable, raices_inv = inverse_roots),
    diagnosticos = list(LjungBox = lb_test, Shapiro = shapiro_test, AIC = modelo_base$aic),
    notas = list(est = nota_estabilidad, lb = nota_lb, norm = nota_norm, phi = nota_phi),
    data_plot = df_plot,
    data_raices = df_raices
  )

  class(resultado) <- "ar_greenreg"
  return(resultado)
}

#' @rdname modelo_ar
#' @param x Objeto de clase.
#' @param ... Argumentos adicionales.
#' @export
print.ar_greenreg <- function(x, ...) {
  
  cat("\n==========================================================\n")
  cat("     MODELO AUTORREGRESIVO AR(", x$orden_p, ") \n", sep = "")
  cat("==========================================================\n\n")
  
  # --- 1. MODELO Y ECUACION ---
  cat("--- 1. MODELO Y ECUACION ---\n")
  phi <- round(x$modelo$coef[grepl("^ar", names(x$modelo$coef))], 3)
  c_int <- round(x$modelo$coef["intercept"], 3)
  
  # Construccion visual de la ecuacion de diferencias
  eq <- paste0("Y(t) = ", if(is.na(c_int)) 0 else c_int)
  if(length(phi) > 0) {
    for(i in 1:length(phi)) {
      eq <- paste0(eq, " + (", phi[i], " * Y(t-", i, "))")
    }
  }
  eq <- paste0(eq, " + E(t)")
  
  cat("Ecuacion de Diferencias Estimada:\n", eq, "\n")
  
  aic_val <- x$diagnosticos$AIC
  aic_txt <- if(is.null(aic_val) || is.na(aic_val)) "NA" else round(aic_val, 2)
  cat("AIC (Criterio de Akaike):", aic_txt, "\n")
  cat(" NOTA: El intercepto representa el valor medio de la serie si es estacionaria.\n\n")
  
  # --- 2. COEFICIENTES ESTIMADOS ---
  cat("--- 2. COEFICIENTES ESTIMADOS (Parametros Phi) ---\n")
  stats::printCoefmat(as.matrix(x$coeficientes), digits = 4, signif.stars = TRUE)
  cat("\n", x$notas$phi, "\n")
  cat(" GUIA: Si |Phi| es cercano a 1, la serie tiene mucha memoria (persistencia).\n\n")
  
  # --- 3. IDENTIFICACION DEL MODELO (ORDEN p) ---
  cat("--- 3. IDENTIFICACION DEL MODELO (Orden p) ---\n")
  cat(" Orden seleccionado: p =", x$orden_p, "\n")
  cat(" Criterio ACF: Las autocorrelaciones decaen exponencialmente o de forma sinoidal.\n")
  cat(" Criterio PACF: Los coeficientes de autocorrelacion parcial se cortan despues del lag", x$orden_p, ".\n")
  cat(" NOTA: Si ves barras significativas en PACF despues de p, podrias necesitar un orden mayor.\n\n")
  
  # --- 4. VERIFICACION DE SUPUESTOS ---
  cat("--- 4. VERIFICACION DE SUPUESTOS ---\n")
  
  # A) Estacionariedad (Estabilidad)
  cat("A) Estacionariedad (Raices Unitarias):\n")
  if (!is.null(x$estabilidad$raices_inv)) {
    modulos <- Mod(x$estabilidad$raices_inv)
    cat("   - Modulos de raices inversas:", paste(round(modulos, 3), collapse = ", "), "\n")
  }
  cat("   -", x$notas$est, "\n")
  
  # B) Errores ~ Ruido Blanco
  cat("\nB) Diagnostico de Errores (E ~ RB):\n")
  
  # Media Cero (Protegido)
  mean_res <- mean(x$data_plot$Residuos, na.rm = TRUE)
  mean_txt <- if(is.na(mean_res)) "NA" else sprintf("%.4f", mean_res)
  cat(sprintf("  Media de los residuos: %s (Esperado: 0)\n", mean_txt))
  
  # No Autocorrelacion (Ljung-Box Protegido)
  lb_pval <- x$diagnosticos$LjungBox$p.value
  lb_txt <- if(is.null(lb_pval) || is.na(lb_pval)) "NA" else sprintf("%.4f", lb_pval)
  cat(sprintf("  No Autocorrelacion (Ljung-Box p-valor): %s\n", lb_txt))
  cat("     ->", x$notas$lb, "\n")
  
  # Varianza Constante y Normalidad (Protegido)
  sh_pval <- x$diagnosticos$Shapiro$p.value
  sh_txt <- if(is.null(sh_pval) || is.na(sh_pval)) "NA" else sprintf("%.4f", sh_pval)
  cat(sprintf("  Normalidad (Shapiro p-valor): %s\n", sh_txt))
  cat("     ->", x$notas$norm, "\n\n")
  
  # --- 5. RESUMEN DEL MODELO ---
  cat("--- 5. RESUMEN DEL MODELO ---\n")
  
  # Validacion logica segura para evitar colapsos por NA
  es_valido <- FALSE
  if (!is.null(x$estabilidad$es_estable) && !is.null(lb_pval) && !is.na(lb_pval)) {
    if (x$estabilidad$es_estable && lb_pval > 0.05) {
      es_valido <- TRUE
    }
  }
  
  if(es_valido) {
    cat("RESULTADO FINAL: El modelo AR(", x$orden_p, ") es VALIDO.\n", sep="")
    cat("   Los residuos son ruido blanco y la serie es estacionaria.\n")
  } else {
    cat(" ADVERTENCIA: El modelo presenta deficiencias tecnicas.\n")
    cat("   Revisa si es necesario diferenciar la serie (d=1) o cambiar el orden p.\n")
  }
  
  cat("\n Usa plot(modelo) para visualizar la serie, ACF y raices unitarias.\n")
  cat("==========================================================\n")
  
  return(invisible(x))
}
#' @rdname modelo_ar
#' @description Genera una secuencia de 9 graficas de diagnostico para validar el modelo AR.
#' @param x Objeto de clase ar_greenreg.
#' @param ... Argumentos adicionales.
#' @export
plot.ar_greenreg <- function(x, ...) {
  
  df <- x$data_plot
  
  mi_tema <- ggplot2::theme_minimal() +
    ggplot2::theme(
      plot.title = ggplot2::element_text(face = "bold", size = 14, color = "#2C3E50"),
      plot.subtitle = ggplot2::element_text(size = 11, color = "#7F8C8D"),
      plot.caption = ggplot2::element_text(hjust = 0, size = 9, face = "italic",
                                           color = "#555555", margin = ggplot2::margin(t = 15),
                                           lineheight = 1.1),
      plot.margin = ggplot2::margin(10, 10, 20, 10),
      axis.title = ggplot2::element_text(face = "bold"),
      legend.position = "top"
    )
  
  # CORRECCION 1: Usar message()
  if (interactive()) {
    message("Generando graficas de diagnostico AR... (Presiona [Enter] para avanzar)\n")
  }
  
  # --- GRAFICA 1: SERIE ORIGINAL (VALORES VS TIEMPO) ---
  
  # CORRECCION 2: Usar stats::sd
  media_serie <- mean(df$Observado, na.rm = TRUE)
  sd_serie <- stats::sd(df$Observado, na.rm = TRUE)
  
  g1 <- ggplot2::ggplot(df, ggplot2::aes(x = Tiempo, y = Observado)) +
    ggplot2::annotate("rect", xmin = -Inf, xmax = Inf,
                      ymin = media_serie - sd_serie, ymax = media_serie + sd_serie,
                      fill = "#34495E", alpha = 0.1) +
    ggplot2::geom_line(color = "#2C3E50", linewidth = 0.7) +
    ggplot2::geom_point(color = "#2C3E50", alpha = 0.3, size = 1) +
    ggplot2::geom_hline(yintercept = media_serie, linetype = "dashed",
                        color = "#E74C3C", linewidth = 0.8) +
    ggplot2::labs(
      title = "1. Comportamiento de la Serie Original",
      subtitle = "Analisis de Trayectoria y Estacionariedad Visual",
      x = "Tiempo (Periodos)",
      y = "Valor Observado",
      caption = paste0("INTERPRETACION:\n",
                       " LINEA ROJA: Es el valor promedio historico de tu serie.\n",
                       " ESTACIONARIA: Si la serie cruza constantemente la linea roja y no tiene tendencia.\n",
                       " TENDENCIA: Si la serie se aleja de la linea roja de forma sostenida (requiere d=1).\n",
                       " NOTA: El area gris representa la zona de fluctuacion normal (+-1 Desviacion Estandar).")
    ) +
    mi_tema
  
  print(g1)
  if (interactive()) {readline(prompt = "Grafica 1: Inspeccion visual de la serie bruta > ")}
  
  
  # --- GRAFICA 2: AJUSTE DEL MODELO ---
  g2 <- ggplot2::ggplot(df, ggplot2::aes(x = Tiempo)) +
    ggplot2::geom_ribbon(ggplot2::aes(ymin = Lower, ymax = Upper),
                         fill = "#E74C3C", alpha = 0.15) +
    ggplot2::geom_line(ggplot2::aes(y = Observado, color = "Observado"), linewidth = 0.8) +
    ggplot2::geom_line(ggplot2::aes(y = Ajustado, color = "Modelo AR"), linewidth = 0.8) +
    ggplot2::scale_color_manual(values = c("Observado" = "#34495E", "Modelo AR" = "#E74C3C")) +
    ggplot2::labs(
      title = paste0("2. Ajuste del Modelo AR(", x$orden_p, ")"),
      subtitle = "Serie Observada vs. Valores Ajustados en el Tiempo",
      x = "Tiempo (Periodos)",
      y = "Valor de la Serie",
      color = "Leyenda:",
      caption = paste0("INTERPRETACION:\n",
                       " BIEN: La linea roja (Modelo) debe seguir la trayectoria de la oscura (Realidad).\n",
                       " SOMBRA ROJA: Representa el intervalo de confianza al 95%.\n",
                       " NOTA: Si el modelo esta muy 'liso' y los datos son muy volatiles, falta orden p.")
    ) +
    mi_tema
  
  print(g2)
  if (interactive()) {readline(prompt = "Grafica 2: Comparacion temporal del ajuste > ")}
  
  
  # --- GRAFICA 3: RESIDUOS EN EL TIEMPO (E vs t) ---
  
  # CORRECCION 2: Usar stats::sd
  sd_res <- stats::sd(df$Residuos, na.rm = TRUE)
  
  g3 <- ggplot2::ggplot(df, ggplot2::aes(x = Tiempo, y = Residuos)) +
    ggplot2::geom_ribbon(ggplot2::aes(ymin = -2*sd_res, ymax = 2*sd_res),
                         fill = "#3498DB", alpha = 0.1) +
    ggplot2::geom_line(color = "#2C3E50", linewidth = 0.5) +
    ggplot2::geom_point(color = "#3498DB", alpha = 0.4, size = 1) +
    ggplot2::geom_hline(yintercept = 0, linetype = "dashed", color = "black", linewidth = 0.8) +
    ggplot2::geom_hline(yintercept = c(-2*sd_res, 2*sd_res),
                        linetype = "dotted", color = "#E74C3C", alpha = 0.6) +
    ggplot2::labs(
      title = "3. Residuos en el Tiempo",
      subtitle = "Buscando Aleatoriedad (E ~ Ruido Blanco)",
      x = "Tiempo (Periodos)",
      y = "Error (Residuo)",
      caption = paste0("INTERPRETACION:\n",
                       " BIEN: Los puntos oscilan al azar alrededor de cero (sin patrones).\n",
                       " BANDAS ROJAS: El 95% de los errores deberian estar dentro de estos limites.\n",
                       " MAL: Si ves una 'U', una tendencia o rachas largas arriba/abajo del cero.\n",
                       " CLAVE: Si hay patrones, el modelo AR(", x$orden_p, ") es insuficiente.")
    ) +
    mi_tema
  
  print(g3)
  if (interactive()) {readline(prompt = "Grafica 3: Analisis de aleatoriedad de los errores > ")}
  
  
  # --- GRAFICA 4: ESTABILIDAD (CIRCULO UNITARIO) ---
  if (!is.null(x$data_raices)) {
    theta <- seq(0, 2*pi, length.out = 200)
    df_circle <- data.frame(x = cos(theta), y = sin(theta))
    
    g4 <- ggplot2::ggplot() +
      ggplot2::geom_path(data = df_circle, ggplot2::aes(x, y),
                         color = "#7F8C8D", linetype = "dashed", linewidth = 0.8) +
      ggplot2::geom_vline(xintercept = 0, color = "#BDC3C7") +
      ggplot2::geom_hline(yintercept = 0, color = "#BDC3C7") +
      ggplot2::geom_polygon(data = df_circle, ggplot2::aes(x, y),
                            fill = "#2ECC71", alpha = 0.05) +
      ggplot2::geom_point(data = x$data_raices,
                          ggplot2::aes(x = Real, y = Imaginario),
                          color = "#E74C3C", size = 4, shape = 18) +
      ggplot2::coord_fixed(xlim = c(-1.2, 1.2), ylim = c(-1.2, 1.2)) +
      ggplot2::labs(
        title = "4. Estabilidad (Circulo Unitario)",
        subtitle = paste0("Raices Inversas del Polinomio AR(", x$orden_p, ")"),
        x = "Parte Real",
        y = "Parte Imaginaria",
        caption = paste0("INTERPRETACION:\n",
                         " ESTABLE: Todas las raices (rombos rojos) estan DENTRO del circulo.\n",
                         " ALERTA: Si una raiz toca el borde, la serie tiene una raiz unitaria (necesita d=1).\n",
                         " INESTABLE: Si una raiz sale del circulo, el modelo no converge y fallara al predecir.")
      ) +
      mi_tema
    
    print(g4)
    
  } else {
    # CORRECCION 1: Usar message() en lugar de cat()
    message(" El modelo AR(0) no tiene raices; es ruido blanco puro y siempre es estable.")
  }
  if (interactive()) {readline(prompt = "Grafica 4: Analisis de raices > ")}
  
  
  # --- GRAFICA 5: AUTOCORRELACION DE RESIDUOS (ACF) ---
  acf_res <- stats::acf(df$Residuos, plot = FALSE, lag.max = 20)
  
  df_acf <- data.frame(
    Lag = as.numeric(acf_res$lag[-1]),
    ACF = as.numeric(acf_res$acf[-1])
  )
  
  n_obs <- nrow(df)
  limite <- 1.96 / sqrt(n_obs)
  
  df_acf$Etiqueta <- ifelse(abs(df_acf$ACF) > limite, as.character(df_acf$Lag), "")
  df_acf$vjust_pos <- ifelse(df_acf$ACF > 0, -0.5, 1.5)
  
  g5 <- ggplot2::ggplot(df_acf, ggplot2::aes(x = Lag, y = ACF)) +
    ggplot2::geom_col(fill = "#34495E", width = 0.4) +
    ggplot2::geom_hline(yintercept = c(limite, -limite),
                        linetype = "dashed", color = "#2980B9", linewidth = 0.8) +
    ggplot2::geom_hline(yintercept = 0, color = "black") +
    ggplot2::geom_text(ggplot2::aes(label = Etiqueta, vjust = vjust_pos),
                       color = "#E74C3C", fontface = "bold", size = 3.5) +
    ggplot2::scale_x_continuous(breaks = 1:20) +
    ggplot2::labs(
      title = "5. Autocorrelacion de Residuos (ACF)",
      subtitle = "Quedo informacion sin capturar en los errores?",
      x = "Lag (Retraso temporal)",
      y = "Coeficiente de Autocorrelacion",
      caption = paste0("INTERPRETACION:\n",
                       " BIEN (RUIDO BLANCO): Todas las barras estan dentro de las lineas azules.\n",
                       " NUMEROS ROJOS: Indican retrasos donde el error sigue un patron.\n",
                       " MAL: Si el Lag 1 es alto, el modelo necesita un orden 'p' mayor.\n",
                       " NOTA: El Lag 0 se omite por ser la correlacion del dato consigo mismo (1.0).")
    ) +
    mi_tema
  
  print(g5)
  if (interactive()) {readline(prompt = "Grafica 5: Evaluacion de la independencia de los errores > ")}
  
  
  # --- GRAFICA 6: AUTOCORRELACION PARCIAL (PACF) DE RESIDUOS ---
  pacf_res <- stats::pacf(df$Residuos, plot = FALSE, lag.max = 20)
  
  df_pacf <- data.frame(
    Lag = as.numeric(pacf_res$lag),
    PACF = as.numeric(pacf_res$acf)
  )
  
  n_obs <- nrow(df)
  limite <- 1.96 / sqrt(n_obs)
  
  df_pacf$Etiqueta <- ifelse(abs(df_pacf$PACF) > limite, as.character(df_pacf$Lag), "")
  df_pacf$vjust_pos <- ifelse(df_pacf$PACF > 0, -0.5, 1.5)
  
  g6 <- ggplot2::ggplot(df_pacf, ggplot2::aes(x = Lag, y = PACF)) +
    ggplot2::geom_col(fill = "#2980B9", width = 0.4) +
    ggplot2::geom_hline(yintercept = c(limite, -limite),
                        linetype = "dashed", color = "#E67E22", linewidth = 0.8) +
    ggplot2::geom_hline(yintercept = 0, color = "black") +
    ggplot2::geom_text(ggplot2::aes(label = Etiqueta, vjust = vjust_pos),
                       color = "#E74C3C", fontface = "bold", size = 3.5) +
    ggplot2::scale_x_continuous(breaks = 1:20) +
    ggplot2::labs(
      title = "6. Autocorrelacion Parcial (PACF)",
      subtitle = "Identificacion de lags residuales directos",
      x = "Lag (Retraso temporal)",
      y = "Coeficiente PACF",
      caption = paste0("INTERPRETACION:\n",
                       " BIEN: Ninguna barra sobresale de las lineas naranjas.\n",
                       " NUMEROS ROJOS: Indican que ese lag especifico aun tiene informacion util.\n",
                       " ACCION: Si el Lag 1 o 2 es rojo, aumenta el orden 'p' en tu funcion modelo_ar().\n",
                       " NOTA: La PACF ayuda a confirmar si el proceso es realmente un AR.")
    ) +
    mi_tema
  
  print(g6)
  if (interactive()) {readline(prompt = "Grafica 6: Verificacion de la estructura del orden p > ")}
  
  
  # --- GRAFICA 7: NORMALIDAD DE RESIDUOS (Q-Q PLOT) ---
  df_limpio <- df[!is.na(df$Residuos), ]
  
  g7 <- ggplot2::ggplot(df_limpio, ggplot2::aes(sample = Residuos)) +
    ggplot2::stat_qq(color = "#8E44AD", alpha = 0.6, size = 2) +
    ggplot2::stat_qq_line(color = "#E74C3C", linetype = "dashed", linewidth = 1) +
    ggplot2::labs(
      title = "7. Normalidad (Q-Q Plot)",
      subtitle = "Comparacion de Cuantiles de Residuos vs. Normal Teorica",
      x = "Cuantiles Teoricos (Normal)",
      y = "Cuantiles Observados (Residuos)",
      caption = paste0("INTERPRETACION:\n",
                       " BIEN: Los puntos morados deben seguir fielmente la linea roja diagonal.\n",
                       " COLAS: Si los extremos se alejan de la linea, hay valores atipicos (outliers).\n",
                       " NOTA: Si los puntos forman una 'S', la distribucion es asimetrica.\n",
                       " CLAVE: La normalidad asegura que tus pronosticos sean estadisticamente confiables.")
    ) +
    mi_tema
  
  print(g7)
  if (interactive()) {readline(prompt = "Grafica 7: Verificacion de la distribucion gaussiana > ")}
  
  
  # --- GRAFICA 8: PRECISION DEL MODELO (REAL VS AJUSTADO) ---
  cor_val <- stats::cor(df$Observado, df$Ajustado, use = "complete.obs")
  
  g8 <- ggplot2::ggplot(df, ggplot2::aes(x = Ajustado, y = Observado)) +
    ggplot2::geom_point(color = "#2980B9", alpha = 0.5, size = 2) +
    ggplot2::geom_abline(slope = 1, intercept = 0, linetype = "dashed", color = "#E74C3C", linewidth = 1) +
    ggplot2::geom_smooth(method = "lm", color = "#2C3E50", fill = "#BDC3C7", alpha = 0.2) +
    ggplot2::labs(
      title = "8. Precision del Modelo (Predicho vs Real)",
      subtitle = paste0("Correlacion de Pearson: ", round(cor_val, 3)),
      x = "Valor Ajustado (Modelo AR)",
      y = "Valor Observado (Realidad)",
      caption = paste0("INTERPRETACION:\n",
                       " LINEA ROJA: Diagonal de perfeccion (Realidad = Modelo).\n",
                       " BIEN: Los puntos deben estar cerca y paralelos a la linea roja.\n",
                       " DISPERSION: Si los puntos se alejan mucho, el modelo tiene poca capacidad predictiva.\n",
                       " NOTA: El modelo AR suele ser conservador y 'suavizar' los extremos.")
    ) +
    mi_tema
  
  print(g8)
  if (interactive()) {readline(prompt = "Grafica 8: Comparacion directa de valores reales y estimados > ")}
  
  
  # --- GRAFICA 9: DIAGNOSTICO DE ESTACIONARIEDAD (MOVIL) ---
  ventana_ar <- max(5, round(nrow(df) * 0.1))
  
  df_est_ar <- df
  df_est_ar$Media_Movil <- as.numeric(stats::filter(df$Observado, rep(1/ventana_ar, ventana_ar), sides = 2))
  
  # CORRECCION 2: Usar stats::sd
  df_est_ar$SD_Movil <- sapply(1:nrow(df), function(i) {
    idx <- max(1, i-ventana_ar):min(nrow(df), i+ventana_ar)
    stats::sd(df$Observado[idx], na.rm = TRUE)
  })
  
  df_final_ar <- df_est_ar[!is.na(df_est_ar$Media_Movil), ]
  
  g9 <- ggplot2::ggplot(df_final_ar, ggplot2::aes(x = Tiempo)) +
    ggplot2::geom_line(data = df, ggplot2::aes(y = Observado), color = "gray80", linewidth = 0.5) +
    ggplot2::geom_ribbon(ggplot2::aes(ymin = Media_Movil - SD_Movil,
                                      ymax = Media_Movil + SD_Movil),
                         fill = "#27AE60", alpha = 0.15) +
    ggplot2::geom_line(ggplot2::aes(y = Media_Movil), color = "#27AE60", linewidth = 1.2) +
    ggplot2::geom_hline(yintercept = mean(df$Observado, na.rm = TRUE),
                        linetype = "dashed", color = "#C0392B", linewidth = 0.8) +
    ggplot2::labs(
      title = "9. Diagnostico de Estacionariedad",
      subtitle = paste0("Analisis de Media y Varianza Movil (Ventana k = ", ventana_ar, ")"),
      x = "Tiempo (Periodos)",
      y = "Valor de la Serie",
      caption = paste0("INTERPRETACION:\n",
                       " LINEA VERDE: Media local movil. Debe ser horizontal y estable.\n",
                       " LINEA ROJA: Media global (Referencia constante).\n",
                       " BIEN: Si la franja verde es estable y no tiene tendencia.\n",
                       " NOTA: Se han filtrado los extremos para un reporte libre de avisos de error.")
    ) +
    mi_tema
  
  print(g9)
  
  # CORRECCION 1: Usar message()
  if (interactive()) {
    message("Secuencia completa de 9 graficas para Series de Tiempo (AR) finalizada.\n")
  }
  
  # CORRECCION 3: Retornar x
  return(invisible(x))
}

#' Asistente Interactivo de Analisis de Datos
#'
#'
#' Esta es la funcion principal y punto de entrada del paquete. Actua como un consultor
#' automatizado que guia al usuario desde la carga de datos hasta la generacion del codigo final,
#' seleccionando la metodologia adecuada segun la naturaleza de la variable objetivo.
#'
#' @section Como funciona este Asistente paso a paso?:
#' \enumerate{
#'   \item **Escaneo y Validacion:**
#'         Analiza la base de datos para asegurar que la variable objetivo existe.
#'         Ademas, realiza un test interno de autocorrelacion para verificar si tiene sentido aplicar modelos de Series de Tiempo (evitando analisis espurios).
#'   \item **Menu Interactivo:**
#'         Despliega opciones en la consola basandose en los datos detectados, preguntando al usuario que enfoque desea (Regresion Clasica, Modelos Generalizados o Series de Tiempo).
#'   \item **Generacion de Estrategias (El "Motor"):**
#'         \itemize{
#'            \item \emph{Para Regresion y GLM:} Crea dos versiones del codigo.
#'            \enumerate{
#'               \item **Modelo Full:** Incluye todas las variables disponibles.
#'               \item **Modelo Optimizado:** Ejecuta internamente un algoritmo "Stepwise" basado en AIC para limpiar el modelo y quedarse solo con los predictores relevantes.
#'            }
#'            \item \emph{Para Series de Tiempo:} Ejecuta una "Busqueda en Rejilla" (Grid Search) probando combinaciones de parametros (p, d, q). Identifica automaticamente si la serie necesita diferenciacion (ARIMA), si es puramente autoregresiva (AR), de media movil (MA) o mixta (ARMA), y genera el codigo para la funcion especifica.
#'         }
#' }
#'
#' @section Guia de la Grafica Exploratoria Generada:
#' Dependiendo de la opcion elegida, la funcion genera una visualizacion preliminar diferente para entender los datos antes de modelar:
#' \describe{
#'   \item{\strong{Escenario 1: Regresion Lineal (Histograma + Gauss)}}{
#'     Muestra un histograma azul de tus datos superpuesto con una curva roja (Campana de Gauss teorica).
#'     \emph{Objetivo:} Evaluar visualmente si la variable respuesta se parece a una distribucion Normal (supuesto clave de la regresion lineal).
#'   }
#'   \item{\strong{Escenario 2: GLM (Barras o Histograma)}}{
#'     Detecta el tipo de variable:
#'     \itemize{
#'       \item *Si es Binaria (0/1):* Grafico de barras naranja. Ayuda a ver si las clases estan desbalanceadas (ej. muchos "0" y pocos "1").
#'       \item *Si es Conteo:* Histograma verde. Muestra la frecuencia de eventos (tipico de Poisson).
#'     }
#'   }
#'   \item{\strong{Escenario 3: Serie de Tiempo (Linea de Tendencia)}}{
#'     Grafica la evolucion de la variable a lo largo del indice temporal.
#'     \emph{Objetivo:} Identificar tendencias (subidas/bajadas a largo plazo) o ciclos repetitivos antes de ajustar el modelo.
#'   }
#' }
#'
#' @param data Data frame o base de datos que contiene las variables.
#' @param variable_objetivo Cadena de texto con el nombre exacto de la columna que deseas analizar (ej. "Ventas").
#' @param verbose Logico. Si es TRUE, despliega los menus. Por defecto es TRUE.
#'
#' @return Devuelve de forma invisible un objeto de clase \code{ggplot2}. 
#' La funcion se utiliza principalmente por su efecto secundario de imprimir 
#' codigo sugerido en la consola y generar visualizaciones.
#'
#' @examples
#'   data(datos_rendimiento_maiz)
#'   analisis_datos(datos_rendimiento_maiz, "rendimiento_maiz_ton_ha")
#'
#' @importFrom stats lm glm step cor shapiro.test arima gaussian binomial poisson acf
#' @importFrom utils menu
#' @import ggplot2
#' @export
analisis_datos <- function(data, variable_objetivo, verbose = TRUE) {

  # --- 1. Validaciones e Informacion de la Base ---

  # Verificamos si la columna objetivo realmente existe en el data frame proporcionado.
  if (!(variable_objetivo %in% names(data))) stop("Error: La variable objetivo no existe en la base de datos.")

  # Extraccion de la variable respuesta (y) y lista de variables predictoras (vars_x).
  y <- data[[variable_objetivo]]
  vars_x <- setdiff(names(data), variable_objetivo)
  n <- nrow(data)

  # Obtenemos el nombre del objeto 'data' tal como lo escribio el usuario (ej. "mi_base")
  # para usarlo textualmente en la generacion automatica del codigo sugerido.
  nombre_data <- deparse(substitute(data))
  # --- 2A. Deteccion Automatica y Recomendacion ---

  # --- CORRECCIoN: Definimos p_externas aqui ---
  p_externas <- length(vars_x)
  # ---------------------------------------------

  # 1. Deteccion de Series de Tiempo
  col_fecha <- names(data)[sapply(data, inherits, what = c("Date", "POSIXct", "POSIXt"))]
  tiene_fecha <- length(col_fecha) == 1
  hay_autocorr <- FALSE

  if (tiene_fecha) {
    # Ordenar por fecha y probar autocorrelacion
    data <- data[order(data[[col_fecha]]), ]
    y_ord <- data[[variable_objetivo]]
    acf_obj <- try(stats::acf(y_ord, plot = FALSE), silent = TRUE)
    if (!inherits(acf_obj, "try-error")) {
      limite_acf <- 2 / sqrt(length(y_ord))
      hay_autocorr <- any(abs(acf_obj$acf[-1]) > limite_acf)
    }
  }
  es_ts <- tiene_fecha && hay_autocorr

  # --- 2. Deteccion Automatica y Menu Interactivo 
  col_fecha <- NULL

  # 1. Buscar si ya existe columna fecha nativa
  cols_nativas <- names(data)[sapply(data, inherits, what = c("Date", "POSIXct", "POSIXt"))]

  if (length(cols_nativas) > 0) {
    col_fecha <- cols_nativas[1]
  } else {
    # 2. Intentar convertir columnas de texto a fecha
    formatos_prueba <- c("%d/%m/%Y", "%Y-%m-%d", "%m/%d/%Y", "%d-%m-%Y", "%Y/%m/%d")
    cols_texto <- names(data)[sapply(data, function(x) is.character(x) || is.factor(x))]

    for (col in cols_texto) {
      vals <- as.character(data[[col]])
      exito <- FALSE
      for (fmt in formatos_prueba) {
        fechas_test <- try(as.Date(vals, format = fmt), silent = TRUE)
        # Criterio: Si no da error y tiene pocos valores nulos (<20%), es fecha valida
        if (!inherits(fechas_test, "try-error") && sum(is.na(fechas_test)) < (0.2 * length(vals))) {
          data[[col]] <- fechas_test # Auto-conversion!
          col_fecha <- col
          exito <- TRUE
          message(sprintf(" Auto-correccion: Columna '%s' detectada como FECHA (formato '%s').\n", col, fmt))
          break
        }
      }
      if (exito) break
    }
  }

  tiene_fecha <- !is.null(col_fecha)
  hay_autocorr <- FALSE

  # 3. Verificar Autocorrelacion (solo si hay fecha)
  if (tiene_fecha) {
    data <- data[order(data[[col_fecha]]), ] # Ordenar cronologicamente
    y_ord <- data[[variable_objetivo]]
    acf_obj <- try(stats::acf(y_ord, plot = FALSE), silent = TRUE)
    if (!inherits(acf_obj, "try-error")) {
      limite_acf <- 2 / sqrt(length(y_ord))
      hay_autocorr <- any(abs(acf_obj$acf[-1]) > limite_acf)
    }
  }
  es_ts <- tiene_fecha && hay_autocorr

  # --- 2B. Logica de Recomendacion ---
  y_temp <- data[[variable_objetivo]]

  es_binaria <- all(y_temp %in% c(0, 1), na.rm=TRUE) || (is.factor(y_temp) && length(levels(y_temp))==2)
  es_conteo <- is.numeric(y_temp) && all(y_temp == floor(y_temp), na.rm=TRUE) && !es_binaria && all(y_temp >= 0, na.rm=TRUE)

  opcion_sugerida <- "1"
  razon_sugerencia <- "Variable continua (Regresion Lineal)."

  if (es_ts) {
    opcion_sugerida <- "3"
    razon_sugerencia <- sprintf("Serie de Tiempo (Fecha detectada: '%s' + Autocorrelacion).", col_fecha)
  } else if (es_binaria) {
    opcion_sugerida <- "2"
    razon_sugerencia <- "Variable Binaria (Logistica)."
  } else if (es_conteo) {
    opcion_sugerida <- "2"
    razon_sugerencia <- "Variable de Conteo (Poisson)."
  }

  # --- 2C. Menu Visual (Protegido para CRAN) ---
  if (interactive() && verbose) {
    cat("\nAsistente de Analisis de Datos\n")
    cat("===========================================\n")
    cat(sprintf("   Base de datos: %s\n", nombre_data))
    cat(sprintf("   Objetivo: '%s'\n", variable_objetivo))
    cat(sprintf("   Observaciones: %d | Predictoras: %d\n\n", n, p_externas))
    
    cat("RECOMENDACION AUTOMATICA:\n")
    cat(sprintf("   -> Sugerencia: Opcion %s\n", opcion_sugerida))
    cat(sprintf("   -> Motivo: %s\n\n", razon_sugerencia))
    
    cat("Opciones Disponibles:\n")
    
    # Marcas visuales para guiar al usuario
    mark1 <- if(opcion_sugerida == "1") "  * (Recomendado)" else ""
    mark2 <- if(opcion_sugerida == "2") "  * (Recomendado)" else ""
    mark3 <- if(opcion_sugerida == "3") "  * (Recomendado)" else ""
    
    cat(sprintf("   1. Regresion Lineal%s\n", mark1))
    cat(sprintf("   2. Modelos GLM (Logistica/Poisson)%s\n", mark2))
    cat(sprintf("   3. Series de Tiempo%s\n", mark3))
  }

  # 1. Manejo de la seleccion
  if (interactive() && verbose) {
    seleccion <- readline(prompt = "Selecciona una opcion (1-3) [Enter para auto]: ")
  } else {
    # En sesiones no interactivas (como los tests de CRAN), 
    # forzamos la sugerencia sin preguntar.
    seleccion <- "" 
  }
  
  # 2. Aplicar la logica de seleccion
  if (seleccion == "") {
    seleccion <- opcion_sugerida
    
    # Usamos message() en lugar de cat() para que sea suprimible
    # Y solo lo mostramos si estamos en una sesion interactiva para no molestar en los tests
    if (interactive() && verbose) {
      message(sprintf("-> Seleccion automatica aplicada: %s", seleccion))
    }
  }

  # --- 3. Procesamiento segun Seleccion ---
  # Inicializacion de variables para almacenar el codigo generado y explicaciones.
  codigo_full <- ""
  codigo_opt <- ""
  explicacion <- ""
  p_graph <- NULL

  # ==============================================================================
  # OPCIoN 1: REGRESIoN LINEAL
  # Estrategia: Ajustar minimos cuadrados ordinarios (RLS/RLM).
  # ==============================================================================
  if (seleccion == "1") {
    message("\n Analizando para Regresion Lineal...\n")

    # A) Codigo Modelo Completo
    # Genera el string del codigo para un modelo que incluye TODAS las variables (formula = y ~ .).
    codigo_full <- paste0(
      sprintf("modelo_rlm_full <- rlm(%s ~ ., data = %s)\n", variable_objetivo, nombre_data),
      "modelo_rlm_full\n",
      "plot(modelo_rlm_full)"
    )

    # B) Codigo Modelo Optimizado (Stepwise AIC)
    # Ejecuta internamente un 'stepwise selection' para encontrar que variables
    # realmente aportan valor al modelo y descartar las redundantes.
    formula_opt <- "."
    vars_seleccionadas <- "Todas"

    if (length(vars_x) > 0) {
      # Ajuste temporal para evaluacion (no se muestra al usuario).
      mod_temp <- stats::lm(as.formula(paste(variable_objetivo, "~ .")), data = data)
      # Seleccion de variables basada en AIC (trace=0 silencia la salida tecnica).
      mod_step <- stats::step(mod_temp, trace = 0)
      vars_opt <- names(coef(mod_step))[-1] # Extraemos nombres de coeficientes (excluyendo intercepto).

      # Si se logro reducir variables, actualizamos la formula optimizada.
      if (length(vars_opt) > 0 && length(vars_opt) < length(vars_x)) {
        formula_opt <- paste(vars_opt, collapse = " + ")
        vars_seleccionadas <- paste(vars_opt, collapse = ", ")
      }
    }

    # Genera el string del codigo para el modelo optimizado.
    codigo_opt <- paste0(
      sprintf("modelo_rlm_opt <- rlm(%s ~ %s, data = %s)\n", variable_objetivo, formula_opt, nombre_data),
      "modelo_rlm_opt\n",
      "plot(modelo_rlm_opt)"
    )

    # C) Grafica Exploratoria con NOTAS
    # Crea un histograma de la variable objetivo y superpone una curva normal teorica
    # para ensenar al usuario sobre el supuesto de normalidad visualmente.
    media_y <- mean(y, na.rm=TRUE); sd_y <- sd(y, na.rm=TRUE)
    p_graph <- ggplot2::ggplot(data, ggplot2::aes(x = .data[[variable_objetivo]])) +
      ggplot2::geom_histogram(ggplot2::aes(y = ggplot2::after_stat(density)), bins = 30, fill = "#3498DB", color = "white", alpha = 0.7) +
      ggplot2::stat_function(fun = dnorm, args = list(mean = media_y, sd = sd_y), color = "red", linewidth = 1.2) +
      ggplot2::labs(
        title = paste("Distribucion de", variable_objetivo),
        subtitle = "Histograma (Datos) vs Campana de Gauss (Teorico)",
        x = variable_objetivo, y = "Densidad",
        caption = "NOTA:\n La linea ROJA representa la normalidad perfecta.\n Si las barras azules se alejan mucho de la forma de campana roja,\n  los supuestos de normalidad podrian no cumplirse (considera transformar variables)."
      ) + ggplot2::theme_minimal()
  }

  # ==============================================================================
  # OPCIoN 2: GLM (Logistica / Poisson)
  # Estrategia: Detectar automaticamente si es binaria (0/1) o conteo (enteros positivos).
  # ==============================================================================
  else if (seleccion == "2") {
    message("\n Analizando para GLM...\n")

    # Deteccion heuristica del tipo de variable respuesta
    es_binaria <- all(y %in% c(0, 1), na.rm=TRUE) || (is.factor(y) && length(levels(y))==2)
    es_conteo <- is.numeric(y) && all(y == floor(y)) && !es_binaria

    # Configuracion de parametros segun el tipo detectado
    if (es_binaria) {
      fun_glm <- "logistico"
      nombre_mod <- "modelo_log"
      familia <- "binomial"
      
      if (interactive() && verbose) {
        cat("   -> Detectada Variable Binaria. Sugerencia: Regresion Logistica.\n")
      }
      
      nota_grafica <- paste("NOTA:\n Revisa el balance entre las barras (0 y 1).\n",
                            "Si una barra es diminuta comparada con la otra (Desbalance),\n",
                            "el modelo tendra dificultades para predecir la clase minoritaria.")
      
    } else if (es_conteo) {
      fun_glm <- "reg_poisson"
      nombre_mod <- "modelo_pois"
      familia <- "poisson"
      
      if (interactive() && verbose) {
        cat("   -> Detectada Variable de Conteo. Sugerencia: Regresion Poisson.\n")
      }
      
      nota_grafica <- paste("NOTA:\n En Poisson, es comun ver barras altas a la izquierda (valores bajos)\n",
                            "que descienden hacia la derecha. Esto es normal y esperado.")
      
    } else {
      # Fallback a gaussiana si no es clara la deteccion
      fun_glm <- "glm_gen"
      nombre_mod <- "modelo_glm"
      familia <- "gaussian"
      
      if (interactive() && verbose) {
        cat("Tipo ambiguo. Usando configuracion generica.\n")
      }
      
      nota_grafica <- "NOTA: Distribucion de la variable respuesta."
    }
    # A) Codigo Modelo Completo
    codigo_full <- paste0(
      sprintf("%s_full <- %s(%s ~ ., data = %s)\n", nombre_mod, fun_glm, variable_objetivo, nombre_data),
      sprintf("%s_full\n", nombre_mod),
      sprintf("plot(%s_full)", nombre_mod)
    )

    # B) Codigo Modelo Optimizado
    # Intenta realizar una seleccion de variables stepwise para GLM.
    formula_opt <- "."
    if (length(vars_x) > 0) {
      try({
        mod_temp <- stats::glm(as.formula(paste(variable_objetivo, "~.")), data = data, family = familia)
        mod_step <- stats::step(mod_temp, trace = 0)
        vars_opt <- names(coef(mod_step))[-1]
        if (length(vars_opt) > 0 && length(vars_opt) < length(vars_x)) {
          formula_opt <- paste(vars_opt, collapse = " + ")
        }
      }, silent = TRUE)
    }

    codigo_opt <- paste0(
      sprintf("%s_opt <- %s(%s ~ %s, data = %s)\n", nombre_mod, fun_glm, variable_objetivo, formula_opt, nombre_data),
      sprintf("%s_opt\n", nombre_mod),
      sprintf("plot(%s_opt)", nombre_mod)
    )

    # C) Grafica
    # Grafico de barras para binaria o histograma para conteo/continuo
    p_graph <- ggplot2::ggplot(data, ggplot2::aes(x = .data[[variable_objetivo]])) +
      (if(es_binaria) ggplot2::geom_bar(fill = "orange", alpha=0.8) else ggplot2::geom_histogram(fill="#2ECC71", bins=20, color="white")) +
      ggplot2::labs(
        title = "Distribucion de la Variable Respuesta",
        caption = nota_grafica
      ) + ggplot2::theme_minimal()
  }

  # ==============================================================================
  # OPCIoN 3: SERIES DE TIEMPO (Sin Alertas)
  # Estrategia: Grid Search (fuerza bruta controlada) para hallar (p, d, q) optimos por AIC.
  # ==============================================================================
  else if (seleccion == "3") {
    message(" Analisis de Serie de Tiempo (AR / MA / ARMA / ARIMA)...")

    if (p_externas > 0) {
      message(" Los modelos AR/MA/ARMA/ARIMA son univariados.")
    }


    mejor_aic <- Inf
    mejor_orden <- c(0, 0, 0) # Orden inicial (p, d, q)

    # Grid Search con silenciador de warnings (suppressWarnings)
    # Se iteran combinaciones de p (AR), d (Diferencias) y q (MA).
    for (d_test in 0:1) {
      for (p_test in 0:3) {
        for (q_test in 0:3) {
          if (p_test==0 && q_test==0 && d_test==0) next # Saltar modelo nulo
          try({
            # Ajuste de prueba. suppressWarnings() es clave aqui para evitar que el usuario
            # se asuste con alertas de "posible no convergencia" durante la busqueda automatica.
            mod <- suppressWarnings(stats::arima(y, order = c(p_test, d_test, q_test), method="CSS-ML"))

            # Guardamos el modelo si tiene el menor AIC encontrado hasta ahora
            if (!is.null(mod$aic) && mod$aic < mejor_aic) {
              mejor_aic <- mod$aic
              mejor_orden <- c(p_test, d_test, q_test)
            }
          }, silent = TRUE)
        }
      }
    }

    p_opt <- mejor_orden[1]; d_opt <- mejor_orden[2]; q_opt <- mejor_orden[3]

    # Seleccion de Funcion de la paqueteria segun el orden optimo encontrado
    if (d_opt > 0) {
      # Si d > 0, requiere diferenciacion -> ARIMA
      funcion_chapi <- "modelo_arima"; nombre_modelo <- "modelo_ARIMAchapi"
      args <- sprintf("p = %d, d = %d, q = %d", p_opt, d_opt, q_opt)
    } else {
      # Si d = 0, es estacionaria (teoricamente). Verificamos componentes AR y MA.
      if (p_opt > 0 && q_opt == 0) {
        funcion_chapi <- "modelo_ar"; nombre_modelo <- "modelo_AR"; args <- sprintf("p = %d", p_opt)
      } else if (p_opt == 0 && q_opt > 0) {
        funcion_chapi <- "modelo_ma"; nombre_modelo <- "modelo_MA"; args <- sprintf("q = %d, include_mean = TRUE", q_opt)
      } else {
        funcion_chapi <- "modelo_arma"; nombre_modelo <- "modelo_ARMA"; args <- sprintf("p = %d, q = %d, include_mean = TRUE", p_opt, q_opt)
      }
    }

    explicacion <- sprintf("AIC optimo encontrado con orden (%d, %d, %d).", p_opt, d_opt, q_opt)

    # Codigo unico: En series de tiempo univariadas no seleccionamos variables,
    # sino que seleccionamos el orden del modelo, por eso solo hay un bloque de codigo.
    codigo_salida <- paste0(
      sprintf("ts_data <- %s$%s\n", nombre_data, variable_objetivo),
      sprintf("%s <- %s(ts_data, %s)\n", nombre_modelo, funcion_chapi, args),
      sprintf("%s\n", nombre_modelo),
      sprintf("plot(%s)", nombre_modelo)
    )

    # Grafica de Serie de Tiempo con Notas
    p_graph <- ggplot2::ggplot(data, ggplot2::aes(x = 1:n, y = .data[[variable_objetivo]])) +
      ggplot2::geom_line(color = "#8E44AD") +
      ggplot2::labs(
        title = paste("Evolucion Temporal:", variable_objetivo),
        x = "Tiempo / indice", y = "Valor",
        caption = "NOTA:\n Observa la grafica: Los datos suben o bajan con el tiempo (Tendencia)?\n Se repiten patrones regularmente (Ciclos)?\n El modelo sugerido abajo ha sido seleccionado para manejar estas caracteristicas."
      ) + ggplot2::theme_minimal()

    codigo_full <- codigo_salida # Para unificar la salida final
    codigo_opt <- "" # No aplica segunda opcion en TS univariado
  } else {
    stop("Opcion invalida.")
  }

  # --- 4. Salida Final ---
  if (interactive() && verbose) {
    cat("\n ANALISIS COMPLETADO.\n")
    
    if (explicacion != "") {
      cat(paste("NOTA:", explicacion, "\n"))
    }
    cat("--------------------------------------------------\n")
    
    # Imprimir Opcion 1 (Codigo sugerido para Series de Tiempo)
    if (seleccion == "3") {
      cat(" Codigo Sugerido (Mejor Ajuste AIC):\n")
      cat("--------------------------------------------------\n")
      cat(codigo_full, "\n")
      cat("--------------------------------------------------\n")
    } else {
      # Opciones para Regresion y GLM
      cat(" OPCION A: Modelo con TODAS las variables\n")
      cat("--------------------------------------------------\n")
      cat(codigo_full, "\n")
      cat("--------------------------------------------------\n\n")
      
      cat(" OPCION B: Modelo OPTIMIZADO (Variables Seleccionadas)\n")
      cat("   (Elimina variables que aportan ruido o son redundantes)\n")
      cat("--------------------------------------------------\n")
      cat(codigo_opt, "\n")
      cat("--------------------------------------------------\n")
    }
    if (!is.null(p_graph)) print(p_graph) 
  }
  
  return(invisible(p_graph))
}

#' Registros de tala ilegal en distintas regiones
#'
#' Conjunto de datos que contiene el numero de eventos de tala ilegal observados
#' junto con variables explicativas relacionadas con la densidad poblacional,
#' superficie forestal y esfuerzo de vigilancia. Ideal para ejemplificar la aplicacion
#' de modelos de regresion Poisson.
#'
#' @format Un data frame con 200 observaciones y 4 variables:
#' \describe{
#'   \item{numero_talas}{Numero de eventos de tala ilegal observados}
#'   \item{densidad_poblacion}{Densidad poblacional de la region (hab/km2)}
#'   \item{superficie_forestal_ha}{Superficie forestal en hectareas}
#'   \item{dias_vigilancia}{Numero de dias de vigilancia reportados}
#' }
#'
#' @details Los datos fueron generados de manera simulada con base en valores reportados
#' por CONAFOR y SEMARNAT. Esto permite realizar ejercicios reproducibles y seguros
#' sin exponer datos sensibles o confidenciales.
#'
#' @examples
#' data(datos_tala_ilegal_poisson)
#' modelo_poisson <- reg_poisson(numero_talas ~ densidad_poblacion + 
#'                               superficie_forestal_ha + dias_vigilancia, 
#'                               data = datos_tala_ilegal_poisson)
#'                               
#'
#' @source Elaboracion propia con fines didacticos tomando referencia de CONAFOR e INEGI.
"datos_tala_ilegal_poisson"

#' Regresion Lineal Simple (RLS)
#'
#' Esta funcion ajusta un modelo de regresion lineal \eqn{y = \beta_0 + \beta_1 x} mediante
#' Minimos Cuadrados Ordinarios (OLS, por sus siglas en ingles 'Ordinary Least Squares'). 
#' A diferencia de \code{lm()} estandar, esta funcion devuelve un objeto enriquecido con 
#' interpretaciones en lenguaje natural, verificaciones automaticas de supuestos y 
#' sugerencias para el investigador.
#'
#' @details
#' El flujo interno de la funcion se divide en cinco etapas criticas:
#' \enumerate{
#'   \item **Preparacion:** Valida que los datos sean coherentes y extrae las variables de la formula.
#'   \item **Estimacion:** Calcula los coeficientes, el error estandar residual \eqn{(\sigma)} y la significancia global (F-test).
#'   \item **Inferencia:** Genera intervalos de confianza para los parametros \eqn{(\beta)} al nivel especificado.
#'   \item **Auditoria de Supuestos:** Ejecuta pruebas de Shapiro-Wilk (Normalidad), Breusch-Pagan (Homocedasticidad)
#'         y Durbin-Watson (Independencia).
#'   \item **Traduccion:** Convierte los estadisticos abstractos en "Notas" de texto que recomiendan acciones
#'         especificas (ej. aplicar logaritmos o revisar datos atipicos).
#' }
#'
#' @section Guia de las 9 Graficas de Diagnostico (\code{plot}):
#' Al ejecutar \code{plot(modelo)}, se despliega una secuencia educativa:
#' \describe{
#'   \item{\strong{1. Histogramas Exploratorios}}{Compara la distribucion de X e Y. Util para detectar sesgos iniciales.}
#'   \item{\strong{2. Relacion Original (Scatterplot)}}{Muestra la nube de puntos pura para confirmar linealidad.}
#'   \item{\strong{3. Recta de Regresion}}{Visualiza el ajuste final con bandas de confianza.}
#'   \item{\strong{4. Q-Q Plot (Normalidad)}}{Grafica los cuantiles de los residuos contra una normal teorica.}
#'   \item{\strong{5. Homocedasticidad}}{Residuos vs. Predichos para verificar varianza constante.}
#'   \item{\strong{6. Independencia}}{Residuos vs. Orden de datos para detectar autocorrelacion.}
#'   \item{\strong{7. Precision (Predicho vs Real)}}{Muestra la correlacion directa entre valores reales y estimados.}
#'   \item{\strong{8. Distancia de Cook}}{Identifica observaciones influyentes o atipicos peligrosos.}
#'   \item{\strong{9. Leverage vs Residuos}}{Diferencia entre errores en Y (Outliers) y valores extremos en X (Apalancamiento).}
#' }
#'
#' @param formula Una formula de R estandar (ej. \code{rendimiento ~ fertilizante}).
#' @param data El data.frame que contiene las variables.
#' @param nivel_confianza Nivel de confianza para los intervalos (por defecto 0.95).
#'
#' @return Un objeto de clase \code{"rls_greenreg"} que es una lista con:
#' \itemize{
#'   \item \code{modelo}: El objeto \code{lm} original.
#'   \item \code{variables}: Nombres de X e Y.
#'   \item \code{coeficientes}: Matriz con Estimaciones, Errores e Intervalos.
#'   \item \code{supuestos}: Resultados de los tests de Shapiro, BP y DW.
#'   \item \code{notas}: Diccionario de interpretaciones y sugerencias de mejora.
#'   \item \code{data_plot}: Dataframe con residuos e indices de influencia.
#' }
#'
#' @examples
#'   data(datos_rendimiento_maiz)
#'   modelo <- rls(rendimiento_maiz_ton_ha ~ precipitacion_mm, data = datos_rendimiento_maiz)
#'   print(modelo)
#'   plot(modelo)
#'
#' @importFrom stats lm anova shapiro.test residuals fitted coef confint sd median cor pf cooks.distance hatvalues rstandard
#' @importFrom lmtest bptest dwtest
#' @import ggplot2
#' @export
  rls <- function(formula, data, nivel_confianza = 0.95) {
    
    if (!inherits(formula, "formula")) stop("El argumento 'formula' debe ser una formula (ej. y ~ x).")
    if (!is.data.frame(data)) stop("El argumento 'data' debe ser un data.frame.")
    
    modelo_base <- stats::lm(formula, data = data)
    resumen <- summary(modelo_base)
    
    var_y <- names(modelo_base$model)[1]
    var_x <- names(modelo_base$model)[2]
    
    r2 <- resumen$r.squared
    r2_adj <- resumen$adj.r.squared
    sigma <- resumen$sigma
    f_statistic <- resumen$fstatistic
    
    if (!is.null(f_statistic)) {
      p_valor_modelo <- stats::pf(f_statistic[1], f_statistic[2], f_statistic[3], lower.tail = FALSE)
    } else {
      p_valor_modelo <- NA
    }
    
    coefs <- resumen$coefficients
    intervalos <- stats::confint(modelo_base, level = nivel_confianza)
    tabla_coefs <- cbind(coefs, intervalos)
    tabla_anova <- stats::anova(modelo_base)
    
    residuos <- stats::residuals(modelo_base)
    
    # A) Normalidad (Shapiro-Wilk)
    if (length(residuos) >= 3 && length(residuos) <= 5000) {
      test_shapiro <- stats::shapiro.test(residuos)
      p_norm <- test_shapiro$p.value
    } else {
      p_norm <- NA
    }
    
    # B) Homocedasticidad (Breusch-Pagan)
    p_homo <- NA
    if (requireNamespace("lmtest", quietly = TRUE)) {
      test_bp <- lmtest::bptest(modelo_base)
      p_homo <- test_bp$p.value
    }
    
    # C) Independencia (Durbin-Watson)
    stat_dw <- NA; p_indep <- NA
    if (requireNamespace("lmtest", quietly = TRUE)) {
      test_dw <- lmtest::dwtest(modelo_base)
      stat_dw <- test_dw$statistic
      p_indep <- test_dw$p.value
    }
    
    # --- Generacion de Notas ---
    nota_r2 <- paste0("El modelo explica el ", round(r2 * 100, 2),
                      "% de la variabilidad de '", var_y, "'. ",
                      ifelse(r2 > 0.7, "Es un ajuste ALTO (Bueno).",
                             ifelse(r2 > 0.4, "Es un ajuste MODERADO.", "Es un ajuste BAJO.")))
    
    beta1 <- coefs[2, 1]
    pval_beta1 <- coefs[2, 4]
    nota_beta1 <- if (pval_beta1 < 0.05) {
      paste0("La relacion es SIGNIFICATIVA. Por cada aumento de 1 unidad en '", var_x,
             "', la variable '", var_y, "' cambia en promedio ", round(beta1, 4), " unidades.")
    } else {
      paste0("La variable '", var_x, "' NO tiene un efecto estadisticamente significativo. ",
             "SUGERENCIA: Intenta recolectar mas datos o revisa si la relacion es curva.")
    }
    
    nota_norm <- if (!is.na(p_norm) && p_norm > 0.05) {
      "OKEY: Los residuos parecen normales (p > 0.05)."
    } else {
      "ALERTA: Residuos NO normales. ACCION: Prueba transformar Y o usa modelos no parametricos."
    }
    
    nota_homo <- if (!is.na(p_homo) && p_homo > 0.05) {
      "OKEY: Varianza constante (Homocedasticidad)."
    } else {
      "ALERTA: Heterocedasticidad detectada. ACCION: Se recomienda usar errores estandar robustos."
    }
    
    nota_indep <- if (!is.na(stat_dw) && stat_dw > 1.5 && stat_dw < 2.5) {
      "OKEY: Residuos independientes (DW cercano a 2)."
    } else {
      "ALERTA: Posible autocorrelacion. ACCION: Revisa si los datos son una serie temporal."
    }
    
    fallo_critico <- (!is.na(p_norm) && p_norm <= 0.05) || (!is.na(p_homo) && p_homo <= 0.05) || (!is.na(stat_dw) && (stat_dw <= 1.5 | stat_dw >= 2.5))
    nota_experto <- if(fallo_critico) "RECOMENDACION FINAL: Fallas en supuestos. Consulta a un analista." else "El modelo cumple con los supuestos basicos."
    
    resultado <- list(
      modelo = modelo_base,
      variables = list(y = var_y, x = var_x),
      info_datos = list(n = length(residuos), media_y = mean(data[[var_y]], na.rm=TRUE), sd_y = stats::sd(data[[var_y]], na.rm=TRUE)),
      anova = tabla_anova,
      coeficientes = tabla_coefs,
      resumen_estadistico = list(R2 = r2, R2_adj = r2_adj, Sigma = sigma, F_stat = f_statistic),
      supuestos = list(shapiro_p = p_norm, bp_p = p_homo, dw_stat = stat_dw),
      notas = list(r2 = nota_r2, impacto = nota_beta1, supuestos = c(nota_norm, nota_homo, nota_indep), alerta_experto = nota_experto),
      data_plot = data.frame(obs_y = data[[var_y]], obs_x = data[[var_x]], pred = stats::fitted(modelo_base), resid = residuos, index = 1:length(residuos))
    )
    
    class(resultado) <- "rls_greenreg"
    return(resultado)
  }

  #' @rdname rls
  #' @param x Objeto de clase rls_greenreg.
  #' @param ... Argumentos adicionales.
  #' @export
  print.rls_greenreg <- function(x, ...) {
    b0 <- round(x$coeficientes[1, 1], 4)
    b1 <- round(x$coeficientes[2, 1], 4)
    signo_txt <- if(b1 >= 0) " + " else " - "
    
    cat("\n==================================================================\n")
    cat("       REGRESION LINEAL SIMPLE        \n")
    cat("==================================================================\n\n")
    
    cat("\n--- 1. MODELO MATEMATICO ESTIMADO ---\n")
    cat(" Variable Dependiente (Y): ", x$variables$y, "\n")
    cat(" Variable Independiente (X):", x$variables$x, "\n")
    cat(" Ecuacion de la Recta:   ", x$variables$y, " =", b0, signo_txt, abs(b1), "*", x$variables$x, "\n")
    cat(" Bondad de Ajuste (R2):   ", round(x$resumen_estadistico$R2 * 100, 2), "%\n\n")
    
    cat("\n--- 2. COEFICIENTES ESTIMADOS ---\n")
    stats::printCoefmat(x$coeficientes[, 1:4], digits = 4, signif.stars = TRUE)
    
    # --- NUEVA SECCIÓN: TABLA ANOVA INCORPORADA ---
    cat("\n--- 3. TABLA DE ANALISIS DE VARIANZA (ANOVA) ---\n")
    if (!is.null(x$anova)) {
      # Usamos printCoefmat para controlar los digitos y evitar caracteres raros de las estrellas en ANOVA
      stats::printCoefmat(x$anova, digits = 4, pdigits = 4, has.Pvalue = TRUE, na.print = "NA")
    } else {
      cat(" No se encontro la tabla ANOVA en el objeto.\n")
    }
    
    cat("\n--- 4. VERIFICACION DE SUPUESTOS ---\n")
    
    # Correccion para p-valor de Shapiro (Evita caidas por NA)
    p_shapiro_txt <- if(is.na(x$supuestos$shapiro_p)) "NA     " else sprintf("%-7.4f", x$supuestos$shapiro_p)
    cat(sprintf(" Normalidad (Shapiro-Wilk):   [p-valor: %s] -> %s\n", p_shapiro_txt, x$notas$supuestos[1]))
    
    # Correccion para p-valor de Breusch-Pagan (Evita caidas por NA)
    p_bp_txt <- if(is.na(x$supuestos$bp_p)) "NA     " else sprintf("%-7.4f", x$supuestos$bp_p)
    cat(sprintf(" Homocedasticidad (B-Pagan):  [p-valor: %s] -> %s\n", p_bp_txt, x$notas$supuestos[2]))
    
    # Correccion para estadistico Durbin-Watson (Evita caidas por NA)
    dw_txt <- if(is.na(x$supuestos$dw_stat)) "NA      " else sprintf("%-8.2f", x$supuestos$dw_stat)
    cat(sprintf(" Independencia (D-Watson):    [Estad.: %s] -> %s\n", dw_txt, x$notas$supuestos[3]))
    
    cat("\n--- 5. RESUMEN DEL MODELO ---\n")
    cat("IMPACTO:  ", x$notas$impacto, "\n")
    cat("VALIDEZ:  ", x$notas$alerta_experto, "\n")
    
    return(invisible(x))
  }
  
  #' @rdname rls
  #' @description Genera una secuencia de 9 graficas para validar el modelo.
  #' @param x Objeto de clase rls_greenreg.
  #' @param ... Argumentos adicionales.
  #' @export
  plot.rls_greenreg <- function(x, ...) {
    
    df <- x$data_plot
    var_y <- x$variables$y
    var_x <- x$variables$x
    
    # Tema visual consistente
    mi_tema <- ggplot2::theme_minimal() +
      ggplot2::theme(
        plot.title = ggplot2::element_text(face = "bold", size = 14, color = "#2C3E50"),
        plot.subtitle = ggplot2::element_text(size = 11, color = "#34495E"),
        plot.caption = ggplot2::element_text(hjust = 0, size = 10, face = "italic", color = "#555555", margin = ggplot2::margin(t = 10)),
        axis.title = ggplot2::element_text(face = "bold")
      )
    
    # CORRECCION CRAN: Usamos message() en lugar de cat()
    if (interactive()) {
      message("\nGenerando graficas de diagnostico...")
      message("Presione [Enter] en la consola para avanzar entre las 9 visualizaciones.\n")
    }
    
    # ==============================================================================
    # --- GRAFICA 1: HISTOGRAMAS EXPLORATORIOS (X, Y) ---
    # ==============================================================================
    df_long <- data.frame(
      Valor = c(df$obs_x, df$obs_y),
      Variable = c(rep(paste("Variable X:", var_x), nrow(df)),
                   rep(paste("Variable Y:", var_y), nrow(df)))
    )
    
    g1 <- ggplot2::ggplot(df_long, ggplot2::aes(x = Valor, fill = Variable)) +
      ggplot2::geom_histogram(bins = 15, color = "white", alpha = 0.8, show.legend = FALSE) +
      ggplot2::facet_wrap(~Variable, scales = "free") + 
      ggplot2::scale_fill_manual(values = c("#3498DB", "#16A085")) +
      ggplot2::labs(
        title = "1. Exploracion Inicial de Datos (Histogramas)",
        subtitle = "Como se distribuyen las variables originales antes del modelo?",
        x = "Valor de la Variable",
        y = "Frecuencia Absoluta",
        caption = paste0("INTERPRETACION:\n",
                         "BIEN: Las barras forman una figura aproximadamente simetrica (campana).\n",
                         " ALERTA: Si un histograma tiene una 'cola' muy larga hacia la derecha o izquierda,\n",
                         "   la variable esta sesgada. Esto puede afectar la precision de la recta de regresion.")
      ) +
      mi_tema
    print(g1)
    if (interactive()) {readline(prompt = "Grafica 1/9 (Histogramas X,Y) > Presione [Enter] para continuar...")}
    
    # ==============================================================================
    # --- GRAFICA 2: RELACION ORIGINAL (SCATTERPLOT) ---
    # ==============================================================================
    g2 <- ggplot2::ggplot(df, ggplot2::aes(x = obs_x, y = obs_y)) +
      ggplot2::geom_point(color = "#2C3E50", size = 3, alpha = 0.6) +
      ggplot2::labs(
        title = "2. Relacion Original (Scatterplot)",
        subtitle = paste("Explorando la correlacion visual entre", var_x, "y", var_y),
        x = var_x,
        y = var_y,
        caption = paste0("INTERPRETACION:\n",
                         "BIEN: Los puntos forman una hilera o elipse clara (ascendente o descendente).\n",
                         "MAL: Los puntos parecen una nube circular o ruidosa. Indica que no hay relacion lineal.")
      ) +
      mi_tema
    
    print(g2)
    if (interactive()) {readline(prompt = "Grafica 2/9 (Relacion Original) > ")}
    
    # ==============================================================================
    # --- GRAFICA 3: RECTA DE REGRESION (AJUSTE) ---
    # ==============================================================================
    g3 <- ggplot2::ggplot(df, ggplot2::aes(x = obs_x, y = obs_y)) +
      ggplot2::geom_point(color = "#2C3E50", size = 3, alpha = 0.4) +
      ggplot2::geom_smooth(method = "lm", color = "#E74C3C", fill = "#E74C3C", alpha = 0.2, linewidth = 1.2) +
      ggplot2::labs(
        title = "3. Ajuste del Modelo (Recta de Regresion)",
        subtitle = paste("Modelo Estimado:", var_y, "~", var_x),
        x = var_x,
        y = var_y,
        caption = paste0("INTERPRETACION:\n",
                         "BIEN: La linea roja cruza por el centro de la nube de puntos.\n",
                         "NOTA: La banda sombreada es el error de estimacion. Si es muy ancha,\n",
                         "   el modelo tiene mucha incertidumbre en esa zona.")
      ) +
      mi_tema
    
    print(g3)
    if (interactive()) {readline(prompt = "Grafica 3/9 (Ajuste del Modelo) > ")}
    
    # ==============================================================================
    # --- GRAFICA 4: NORMALIDAD (QQ-PLOT) ---
    # ==============================================================================
    g4 <- ggplot2::ggplot(df, ggplot2::aes(sample = resid)) +
      ggplot2::geom_qq(color = "#3498DB", size = 2) +
      ggplot2::geom_qq_line(color = "#E74C3C", linewidth = 1.2, linetype = "dashed") +
      ggplot2::labs(
        title = "4. Validacion de Normalidad (Q-Q Plot)",
        subtitle = "Comparacion contra una distribucion normal ideal",
        x = "Cuantiles Teoricos", y = "Cuantiles Observados",
        caption = "INTERPRETACION:\nBIEN: Los puntos azules estan pegados a la linea roja discontinua.\n MAL: Si los puntos en los extremos se curvan o alejan mucho de la linea."
      ) + mi_tema
    print(g4)
    if (interactive()) {readline(prompt = "Grafica 4/9 (Normalidad) > ")}
    
    # ==============================================================================
    # --- GRAFICA 5: HOMOCEDASTICIDAD ---
    # ==============================================================================
    g5 <- ggplot2::ggplot(df, ggplot2::aes(x = pred, y = resid)) +
      ggplot2::geom_point(color = "#8E44AD", size = 2.5, alpha = 0.7) +
      ggplot2::geom_hline(yintercept = 0, linetype = "dashed", color = "black") +
      ggplot2::labs(
        title = "5. Homocedasticidad (Varianza Constante)",
        subtitle = "El error es igual para valores chicos y grandes?",
        x = "Valores Predichos", y = "Residuos (Errores)",
        caption = "INTERPRETACION:\n BIEN: Nube dispersa sin forma (como cielo estrellado).\n MAL: Forma de 'Embudo' (<) o abanico. Significa que el error crece con el valor."
      ) + mi_tema
    print(g5)
    if (interactive()) {readline(prompt = "Grafica 5/9 Homocedasticidad > ")}
    
    # ==============================================================================
    # --- GRAFICA 6: INDEPENDENCIA ---
    # ==============================================================================
    g6 <- ggplot2::ggplot(df, ggplot2::aes(x = index, y = resid)) +
      ggplot2::geom_line(color = "#95A5A6", alpha = 0.5) +
      ggplot2::geom_point(color = "#D35400", size = 2) +
      ggplot2::geom_hline(yintercept = 0, linetype = "dashed") +
      ggplot2::labs(
        title = "6. Independencia (Residuos vs Orden)",
        subtitle = "Detectando patrones en el tiempo de recoleccion",
        x = "Orden de Recoleccion", y = "Residuos",
        caption = "INTERPRETACION:\n BIEN: Ruido aleatorio (picos arriba y abajo sin orden).\n MAL: Si ves ondas suaves o tendencias, los datos no son independientes."
      ) + mi_tema
    print(g6)
    if (interactive()) { readline(prompt = "Grafica 6/9 Independencia > ")}
    
    # ==============================================================================
    # --- GRAFICA 7: PREDICCION VS REALIDAD ---
    # ==============================================================================
    correlacion <- stats::cor(df$obs_y, df$pred)
    g7 <- ggplot2::ggplot(df, ggplot2::aes(x = obs_y, y = pred)) +
      ggplot2::geom_abline(slope = 1, intercept = 0, linetype = "dashed", color = "gray50", linewidth = 1) +
      ggplot2::geom_point(color = "#27AE60", size = 3, alpha = 0.7) +
      ggplot2::labs(
        title = "7. Precision: Prediccion vs Realidad",
        subtitle = paste("Correlacion:", round(correlacion, 3)),
        x = "Valor Real", y = "Valor Predicho",
        caption = "INTERPRETACION:\n BIEN: Los puntos caen sobre la linea gris punteada (Identidad).\n MAL: Nube dispersa y redonda (el modelo no predice bien)."
      ) + mi_tema
    print(g7)
    if (interactive()) {readline(prompt = "Grafica 7/9 > ")}
    
    # ==============================================================================
    # --- GRAFICA 8: DISTANCIA DE COOK ---
    # ==============================================================================
    cooksd <- stats::cooks.distance(x$modelo)
    n_datos <- length(cooksd)
    umbral_cook <- 4 / n_datos
    df_cook <- data.frame(Index = 1:n_datos, Cook = cooksd)
    df_cook$Label <- ifelse(df_cook$Cook > umbral_cook, as.character(df_cook$Index), "")
    
    g8 <- ggplot2::ggplot(df_cook, ggplot2::aes(x = Index, y = Cook)) +
      ggplot2::geom_bar(stat = "identity", fill = "#E67E22", alpha = 0.8) +
      ggplot2::geom_hline(yintercept = umbral_cook, color = "red", linetype = "dashed") +
      ggplot2::geom_text(ggplot2::aes(label = Label), vjust = -0.5, size = 3) +
      ggplot2::labs(
        title = "8. Valores Influyentes (Distancia de Cook)",
        subtitle = "Deteccion de datos atipicos peligrosos",
        x = "Numero de Observacion", y = "Influencia",
        caption = "INTERPRETACION:\n ALERTA: Las barras que cruzan la linea roja son datos sospechosos.\n ACCION: Revisa esas filas en tu Excel, podrian ser errores de captura."
      ) + mi_tema
    print(g8)
    if (interactive()) {readline(prompt = "Grafica 8/9 Influyentes > ")}
    
    # ==============================================================================
    # --- GRAFICA 9: RESIDUOS VS APALANCAMIENTO ---
    # ==============================================================================
    lev <- stats::hatvalues(x$modelo)
    res_std <- stats::rstandard(x$modelo)
    n_datos <- length(lev)
    umbral_lev <- 4 / n_datos
    
    df_diag <- data.frame(
      index = 1:n_datos,
      leverage = lev,
      resid_std = res_std
    )
    
    df_diag$Label <- ifelse(df_diag$leverage > umbral_lev | abs(df_diag$resid_std) > 2,
                            as.character(df_diag$index), "")
    
    g9 <- ggplot2::ggplot(df_diag, ggplot2::aes(x = leverage, y = resid_std)) +
      ggplot2::geom_point(ggplot2::aes(size = leverage), color = "#C0392B", alpha = 0.6) +
      ggplot2::geom_hline(yintercept = 0, linetype = "dashed", color = "black") +
      ggplot2::geom_hline(yintercept = c(-2, 2), linetype = "dotted", color = "red", alpha = 0.5) +
      ggplot2::geom_vline(xintercept = umbral_lev, linetype = "dashed", color = "blue", alpha = 0.5)
    
    # Verificacion defensiva del paquete ggrepel
    if (requireNamespace("ggrepel", quietly = TRUE)) {
      g9 <- g9 + ggrepel::geom_text_repel(ggplot2::aes(label = Label), size = 3.5, fontface = "bold")
    } else {
      g9 <- g9 + ggplot2::geom_text(ggplot2::aes(label = Label), size = 3.5, vjust = -0.5)
    }
    
    g9 <- g9 + 
      ggplot2::labs(
        title = "9. Diagnostico de Influencia (Leverage vs Residuos)",
        subtitle = "Identificando puntos que 'jalonean' la recta",
        x = "Apalancamiento (Leverage)",
        y = "Residuos Estandarizados",
        caption = paste0("INTERPRETACION:\n",
                         " EJE X: Puntos a la derecha de la linea azul tienen valores de X muy raros.\n",
                         " EJE Y: Puntos fuera de las lineas rojas punteadas son Outliers (errores en Y).\n",
                         " CLAVE: Si un punto esta muy a la derecha Y muy arriba/abajo, esta deformando tu modelo.")
      ) +
      mi_tema +
      ggplot2::theme(legend.position = "none")
    
    print(g9)
    
    # CORRECCION CRAN: Usamos message()
    if (interactive()) {
      message("Secuencia de graficas completada.\n")
    }
  }

#' Modelo Integrado de Media Movil Autorregresivo (ARIMA)
#'
#' Esta funcion ajusta el modelo mas versatil para series de tiempo univariadas:
#' el ARIMA(p, d, q). Combina la memoria de largo plazo (AR), la estabilizacion
#' mediante diferenciacion (I) para eliminar tendencias, y la respuesta a
#' choques aleatorios de corto plazo (MA).
#'
#' @details
#' El flujo interno de la funcion ejecuta una auditoria de cinco fases:
#' \enumerate{
#'   \item \strong{Diferenciacion (Fase I):} Si \eqn{d > 0}, el modelo resta los valores
#'         consecutivos para eliminar la tendencia y lograr que la serie sea
#'         "Estacionaria" (promedio constante).
#'   \item \strong{Estimacion ARMA:} Sobre la serie ya estabilizada, estima los
#'         parametros \eqn{\phi} (AR) y \eqn{\theta} (MA) mediante Maxima Verosimilitud.
#'   \item \strong{Doble Validacion de Estructura:} Analiza las raices del polinomio
#'         caracteristico para asegurar que la parte AR sea \strong{Estable} y la
#'         parte MA sea \strong{Invertible}.
#'   \item \strong{Auditoria de Residuos:} Verifica que el error final sea "Ruido Blanco"
#'         mediante las pruebas de Ljung-Box (independencia) y Shapiro-Wilk (normalidad).
#'   \item \strong{Diagnostico Dinamico:} Evalua si la media y la varianza se mantienen
#'         estables a lo largo del tiempo despues de la integracion.
#' }
#'
#' @section Guia de las 10 Graficas de Diagnostico:
#' Al ejecutar \code{plot(modelo)}, se despliega el diagnostico mas completo del paquete:
#' \itemize{
#'   \item \strong{1. Serie Original:} Visualiza la serie bruta. Fundamental para identificar la tendencia inicial que justifica el valor de \eqn{d}.
#'   \item \strong{2. Ajuste Integrado:} Superpone el modelo (purpura) sobre la realidad, incluyendo bandas de confianza del 95\%.
#'   \item \strong{3. Serie Diferenciada:} Muestra la serie despues de aplicar \eqn{d}. Si el modelo es correcto, esta serie debe verse plana y sin tendencia.
#'   \item \strong{4. Circulo Unitario:} Grafica de convergencia matematica. Los puntos rojos (AR) y azules (MA) deben estar dentro del circulo gris.
#'   \item \strong{5. Residuos en el Tiempo:} Busca patrones de error. Los puntos deben formar una nube aleatoria sin ciclos visibles.
#'   \item \strong{6. ACF de Residuos:} Prueba de independencia. Las barras deben estar dentro de los limites para confirmar el Ruido Blanco.
#'   \item \strong{7. PACF de Residuos:} Ayuda a identificar si el orden autorregresivo \eqn{p} es suficiente para explicar la inercia.
#'   \item \strong{8. Normalidad (Q-Q Plot):} Valida que los errores sigan una distribucion gaussiana para asegurar pronosticos confiables.
#'   \item \strong{9. Precision (Real vs Ajustado):} Evaluacion de la correlacion de Pearson entre los datos y la estimacion.
#'   \item \strong{10. Estacionariedad Final:} Muestra la media y varianza movil. Una franja purpura estable certifica un modelo robusto.
#' }
#'
#' @param x Vector numerico o serie de tiempo (\code{ts}).
#' @param p Orden Autorregresivo (memoria del pasado).
#' @param d Orden de Integracion (numero de diferenciaciones).
#' @param q Orden de Media Movil (memoria de los errores).
#' @param include_mean Logico. Incluir constante? (Suele ser \code{FALSE} si \eqn{d > 0}).
#'
#' @return Un objeto de clase \code{"arima_greenreg"} que contiene:
#' \itemize{
#'   \item \code{modelo}: El objeto \code{arima} original de \code{stats}.
#'   \item \code{ordenes}: Vector con los valores de p, d y q.
#'   \item \code{coeficientes_tabla}: Matriz con estimaciones, errores estandar y p-valores.
#'   \item \code{estatus}: Lista con el estado de estabilidad e invertibilidad.
#'   \item \code{diagnosticos}: Resultados de Ljung-Box, Shapiro-Wilk y AIC.
#'   \item \code{data_plot}: Dataframe listo para visualización con \code{ggplot2}.
#'   \item \code{data_diff}: Dataframe de la serie diferenciada (si \eqn{d > 0}).
#' }
#'
#' @examples
#' data("datos_nivel_presa")
#' ts_data <- datos_nivel_presa$nivel_m
#' modelo_ARIMA <- modelo_arima(ts_data, p = 1, d = 1, q = 1)
#' print(modelo_ARIMA)
#' plot(modelo_ARIMA)
#' 
#'
#' @seealso \code{\link[stats]{arima}}
#'
#' @importFrom stats arima coef residuals fitted Box.test pnorm qnorm shapiro.test acf pacf filter sd
#' @import ggplot2
#' @export
modelo_arima <- function(x, p = 1, d = 1, q = 1, include_mean = TRUE) {

  # --- 1. Validacion y Ajuste ---
  if (!is.numeric(x)) stop("El argumento 'x' debe ser numerico.")

  # Ajuste ARIMA(p, d, q)
  modelo_base <- stats::arima(x, order = c(p, d, q), include.mean = include_mean)

  # --- 2. Coeficientes y Estadisticas ---
  coefs <- modelo_base$coef

  # Si no hay coeficientes (ej: ARIMA(0,1,0)), manejamos el caso
  if (length(coefs) > 0) {
    se <- sqrt(diag(modelo_base$var.coef))
    z_val <- coefs / se
    p_val <- 2 * (1 - stats::pnorm(abs(z_val)))
    tabla_coefs <- cbind(Estimate = coefs, `Std. Error` = se, `z value` = z_val, `Pr(>|z|)` = p_val)
  } else {
    tabla_coefs <- matrix(NA, nrow=0, ncol=4)
  }

  # --- 3. Analisis de Raices (Estabilidad e Invertibilidad) ---

  # A) Raices AR (Estabilidad)
  ar_coefs <- coefs[grepl("^ar", names(coefs))]
  raices_ar_inv <- NULL
  es_estable <- TRUE
  if (length(ar_coefs) > 0) {
    raices_ar_inv <- 1 / polyroot(c(1, -ar_coefs))
    es_estable <- all(Mod(raices_ar_inv) < 1)
  }

  # B) Raices MA (Invertibilidad)
  ma_coefs <- coefs[grepl("^ma", names(coefs))]
  raices_ma_inv <- NULL
  es_invertible <- TRUE
  if (length(ma_coefs) > 0) {
    raices_ma_inv <- 1 / polyroot(c(1, ma_coefs))
    es_invertible <- all(Mod(raices_ma_inv) < 1)
  }

  # --- 4. Diagnosticos de Residuos ---
  residuos <- stats::residuals(modelo_base)

  # Ljung-Box (Ruido Blanco)
  lb_test <- stats::Box.test(residuos, type = "Ljung-Box", lag = p + q + 5)

  # Shapiro-Wilk (Normalidad)
  if (length(residuos) >= 3 && length(residuos) <= 5000) {
    shapiro_test <- stats::shapiro.test(residuos)
  } else {
    shapiro_test <- list(p.value = NA)
  }

  # --- 5. Datos para Graficas ---
  ajustados <- x - residuos # En ARIMA esto devuelve la serie original ajustada
  sigma <- sqrt(modelo_base$sigma2)
  # Intervalos 95%
  lower_ci <- ajustados - 1.96 * sigma
  upper_ci <- ajustados + 1.96 * sigma

  df_plot <- data.frame(
    Tiempo = 1:length(x),
    Observado = as.numeric(x),
    Ajustado = as.numeric(ajustados),
    Lower = as.numeric(lower_ci),
    Upper = as.numeric(upper_ci),
    Residuos = as.numeric(residuos)
  )

  # Datos de la serie diferenciada (para ver si d fue efectivo)
  df_diff <- NULL
  if (d > 0) {
    serie_diff <- diff(x, differences = d)
    df_diff <- data.frame(Tiempo = 1:length(serie_diff), Valor = as.numeric(serie_diff))
  }

  # Datos combinados para circulo unitario
  df_raices <- data.frame(Real = numeric(), Imaginario = numeric(), Tipo = character())
  if (!is.null(raices_ar_inv)) {
    df_raices <- rbind(df_raices, data.frame(Real = Re(raices_ar_inv), Imaginario = Im(raices_ar_inv), Tipo = "AR (Estabilidad)"))
  }
  if (!is.null(raices_ma_inv)) {
    df_raices <- rbind(df_raices, data.frame(Real = Re(raices_ma_inv), Imaginario = Im(raices_ma_inv), Tipo = "MA (Invertibilidad)"))
  }

  # --- 6. Objeto Final ---
  resultado <- list(
    modelo = modelo_base,
    ordenes = c(p=p, d=d, q=q),
    coeficientes_tabla = tabla_coefs,
    estatus = list(estable = es_estable, invertible = es_invertible),
    diagnosticos = list(lb = lb_test, shapiro = shapiro_test, aic = modelo_base$aic),
    data_plot = df_plot,
    data_diff = df_diff,
    data_raices = df_raices
  )

  class(resultado) <- "arima_greenreg"
  return(resultado)
}

#' @rdname modelo_arima
#' @param x Objeto de clase.
#' @param ... Argumentos adicionales.
#' @export
print.arima_greenreg <- function(x, ...) {
  
  p <- x$ordenes['p']
  d <- x$ordenes['d']
  q <- x$ordenes['q']
  
  cat("\n==========================================================\n")
  cat(sprintf("MODELO INTEGRADO ARIMA(%d, %d, %d) \n", p, d, q))
  cat("==========================================================\n\n")
  
  # --- 1. MODELO Y ECUACION ---
  cat("--- 1. MODELO Y ECUACION ---\n")
  
  # Componentes para la descripcion
  parte_ar <- if(p > 0) paste0("AR(", p, ")") else ""
  parte_i  <- if(d > 0) paste0("I(", d, ")") else ""
  parte_ma <- if(q > 0) paste0("MA(", q, ")") else ""
  
  # USAMOS Filter para limpiar los espacios vacios
  estructura <- Filter(function(z) nchar(z) > 0, c(parte_ar, parte_i, parte_ma))
  
  cat("Estructura:", paste(estructura, collapse = " + "), "\n")
  
  if(d > 0) {
    cat(sprintf("Ecuacion con d = %d: Y(t) = f(Pasado, Choques)\n", d))
    cat(" NOTA: El modelo trabaja sobre la serie diferenciada para eliminar la tendencia.\n")
  } else {
    cat("Ecuacion: Y(t) = f(Pasado, Choques)\n")
  }
  
  aic_val <- x$diagnosticos$aic
  aic_txt <- if(is.null(aic_val) || is.na(aic_val)) "NA" else round(aic_val, 2)
  cat("AIC (Criterio de Akaike):", aic_txt, "\n\n")
  
  # --- 2. COEFICIENTES ESTIMADOS ---
  cat("--- 2. COEFICIENTES ESTIMADOS ---\n")
  
  if (!is.null(x$coeficientes_tabla) && nrow(x$coeficientes_tabla) > 0) {
    stats::printCoefmat(as.matrix(x$coeficientes_tabla), digits = 4, signif.stars = TRUE)
  } else {
    cat("  (Modelo sin coeficientes estimados: Caminata aleatoria pura)\n")
  }
  cat("\n GUIA: Los coeficientes explican la relacion entre los datos ya diferenciados.\n\n")
  
  # --- 3. IDENTIFICACION DEL MODELO (p, d, q) ---
  cat("--- 3. IDENTIFICACION DEL MODELO ---\n")
  cat(sprintf(" p (Auto-Regresivo): %d periodos de memoria propia.\n", p))
  cat(sprintf(" d (Integracion):    %d diferenciacion(es) para lograr estacionariedad.\n", d))
  cat(sprintf(" q (Media Movil):    %d choques pasados que afectan el presente.\n", q))
  cat(" NOTA: Un modelo ARIMA es exitoso si 'd' estabiliza la media y 'p,q' eliminan la autocorrelacion.\n\n")
  
  # --- 4. VERIFICACION DE SUPUESTOS ---
  cat("--- 4. VERIFICACION DE SUPUESTOS ---\n")
  
  # A) Estacionariedad y Estructura
  cat("A) Estacionariedad y Estabilidad:\n")
  if(d > 0) cat(sprintf("   Estacionariedad: Lograda mediante %d diferenciacion(es).\n", d))
  
  if(p > 0) {
    est <- x$estatus$estable
    status_ar <- if(!is.null(est) && !is.na(est) && est) "ESTABLE" else "INESTABLE o Indeterminado"
    cat(sprintf("  Parte AR: %s (Raices Rojas dentro del circulo).\n", status_ar))
  }
  
  if(q > 0) {
    inv <- x$estatus$invertible
    status_ma <- if(!is.null(inv) && !is.na(inv) && inv) "INVERTIBLE" else "NO INVERTIBLE o Indeterminado"
    cat(sprintf("  Parte MA: %s (Rombos Azules dentro del circulo).\n", status_ma))
  }
  
  # B) Residuos ~ Ruido Blanco (E)
  cat("\nB) Diagnostico de Residuos (E ~ Ruido Blanco):\n")
  
  # Media Cero
  m_res <- mean(x$data_plot$Residuos, na.rm = TRUE)
  m_res_txt <- if(is.na(m_res)) "NA" else sprintf("%.4f", m_res)
  cat(sprintf("  Media de residuos: %s (Esperado: 0)\n", m_res_txt))
  
  # No Autocorrelacion
  lb_p <- x$diagnosticos$lb$p.value
  lb_txt <- if(is.null(lb_p) || is.na(lb_p)) "NA" else sprintf("%.4f", lb_p)
  cat(sprintf("  Independencia (Ljung-Box p-valor): %s\n", lb_txt))
  
  if (!is.null(lb_p) && !is.na(lb_p) && lb_p > 0.05) {
    cat("     -> OKEY: Los residuos son independientes (Ruido Blanco).\n")
  } else {
    cat("     ->  ALERTA: El modelo no capturo toda la informacion disponible.\n")
  }
  
  # Normalidad y Varianza
  sh_p <- x$diagnosticos$shapiro$p.value
  sh_txt <- if(is.null(sh_p) || is.na(sh_p)) "N/D" else sprintf("%.4f", sh_p)
  cat(sprintf("  Normalidad (Shapiro p-valor): %s\n", sh_txt))
  
  # --- 5. RESUMEN DEL MODELO ---
  cat("\n--- 5. RESUMEN DEL MODELO ---\n")
  
  # Logica de validacion blindada contra NAs
  ar_ok <- if (p == 0) TRUE else (!is.null(x$estatus$estable) && !is.na(x$estatus$estable) && x$estatus$estable)
  ma_ok <- if (q == 0) TRUE else (!is.null(x$estatus$invertible) && !is.na(x$estatus$invertible) && x$estatus$invertible)
  lb_ok <- (!is.null(lb_p) && !is.na(lb_p) && lb_p > 0.05)
  
  if(ar_ok && ma_ok && lb_ok) {
    cat(" MODELO VALIDO: El ARIMA ha sido correctamente identificado y ajustado.\n")
  } else {
    cat(" REVISION NECESARIA: El modelo presenta problemas de estabilidad o residuos sucios.\n")
  }
  
  cat("\n Usa plot(modelo) para ver las 9 graficas, incluyendo la Serie Diferenciada.\n")
  cat("==========================================================\n")
  
  return(invisible(x))
}

#' @rdname modelo_arima 
#' @description Genera un panel visual para evaluar el ajuste del modelo.
#' @param x Un objeto de clase "arima_greenreg".
#' @param ... Argumentos adicionales.
#' @export
plot.arima_greenreg <- function(x, ...) {
  
  # --- 0. EXTRACCION DE PARAMETROS ---
  df <- x$data_plot
  p  <- x$ordenes['p']
  d  <- x$ordenes['d']
  q  <- x$ordenes['q']
  
  # TEMA
  mi_tema <- ggplot2::theme_minimal() +
    ggplot2::theme(
      plot.title = ggplot2::element_text(face = "bold", size = 14, color = "#2C3E50"),
      plot.subtitle = ggplot2::element_text(size = 11, color = "#7F8C8D"),
      plot.caption = ggplot2::element_text(hjust = 0, size = 9, face = "italic",
                                           color = "#555555", margin = ggplot2::margin(t = 15),
                                           lineheight = 1.1),
      plot.margin = ggplot2::margin(10, 10, 20, 10),
      axis.title = ggplot2::element_text(face = "bold"),
      legend.position = "top"
    )
  
  # CORRECCION 1: Usar message()
  if (interactive()) {
    message("Iniciando diagnostico ARIMA... (10 graficas totales)")
  }
  
  # --- GRAFICA 1: SERIE ORIGINAL (VALORES VS TIEMPO) ---
  
  media_orig <- mean(df$Observado, na.rm = TRUE)
  
  g1 <- ggplot2::ggplot(df, ggplot2::aes(x = Tiempo, y = Observado)) +
    # Linea de la serie original
    ggplot2::geom_line(color = "#34495E", linewidth = 0.7) +
    # Puntos sutiles
    ggplot2::geom_point(color = "#34495E", alpha = 0.3, size = 1) +
    # Linea de la media (Referencia)
    ggplot2::geom_hline(yintercept = media_orig, linetype = "dashed",
                        color = "#8E44AD", linewidth = 0.8) +
    ggplot2::labs(
      title = "1. Comportamiento de la Serie Original",
      subtitle = "Visualizacion de la serie temporal bruta (sin diferenciar)",
      x = "Tiempo (Periodos)",
      y = "Valor Observado",
      caption = paste0("INTERPRETACION:\n",
                       " LINEA PURPURA: Promedio historico general.\n",
                       " ANALISIS: Si la serie sube o baja constantemente (tendencia),\n",
                       "   el valor de d debe ser mayor a 0 para estabilizarla.\n",
                       " NOTA: El ARIMA usara d=", d, " para corregir lo que vemos aqui.")
    ) +
    mi_tema
  
  print(g1)
  if (interactive()) {readline(prompt = "Grafica 1: Inspeccion de la serie original. > ")}
  
  # --- GRAFICA 2: AJUSTE DEL MODELO ARIMA (Real vs Ajustado) ---
  
  g2 <- ggplot2::ggplot(df, ggplot2::aes(x = Tiempo)) +
    # Sombra de Confianza (IC 95%) en Purpura
    ggplot2::geom_ribbon(ggplot2::aes(ymin = Lower, ymax = Upper),
                         fill = "#8E44AD", alpha = 0.15) +
    # Serie Observada (Realidad)
    ggplot2::geom_line(ggplot2::aes(y = Observado, color = "Observado"), linewidth = 0.7) +
    # Serie Ajustada (Modelo ARIMA)
    ggplot2::geom_line(ggplot2::aes(y = Ajustado, color = "Modelo ARIMA"), linewidth = 0.8) +
    # Escala de colores
    ggplot2::scale_color_manual(values = c("Observado" = "#34495E", "Modelo ARIMA" = "#8E44AD")) +
    ggplot2::labs(
      title = paste0("2. Ajuste del Modelo ARIMA(", p, ",", d, ",", q, ")"),
      subtitle = "Serie Observada vs. Valores Ajustados (Fitted)",
      x = "Tiempo (Periodos)",
      y = "Valor",
      color = "Referencia:",
      caption = paste0("INTERPRETACION:\n",
                       " BIEN: La linea purpura debe seguir la trayectoria de la oscura.\n",
                       " SOMBRA: Rango donde el modelo espera que caigan los datos reales.\n",
                       " NOTA: El ARIMA integra la diferenciacion (d) para seguir tendencias\n",
                       "   y los componentes (p,q) para ajustar las oscilaciones.")
    ) +
    mi_tema
  
  print(g2)
  if (interactive()) {readline(prompt = "Grafica 2: Evaluacion del ajuste integrado. > ")}
  
  # --- GRAFICA 3: SERIE DIFERENCIADA (ESTACIONARIEDAD LOGRADA) ---
  
  if (!is.null(x$data_diff)) {
    # Usamos el dataframe de la serie diferenciada calculado en la funcion principal
    df_diff <- x$data_diff
    media_diff <- mean(df_diff$Valor, na.rm = TRUE)
    
    g3 <- ggplot2::ggplot(df_diff, ggplot2::aes(x = Tiempo, y = Valor)) +
      # Linea de la serie transformada (Diferenciada)
      ggplot2::geom_line(color = "#8E44AD", linewidth = 0.6) +
      # Linea de referencia en Cero
      ggplot2::geom_hline(yintercept = 0, linetype = "solid", color = "black", alpha = 0.5) +
      # Linea de la media de la diferencia
      ggplot2::geom_hline(yintercept = media_diff, linetype = "dashed",
                          color = "#E74C3C", linewidth = 0.8) +
      ggplot2::labs(
        title = paste0("3. Serie Diferenciada (d = ", d, ")"),
        subtitle = "Transformacion para lograr Estacionariedad",
        x = "Tiempo (Periodos tras diferencia)",
        y = paste0("^", d, " Valor"),
        caption = paste0("INTERPRETACION:\n",
                         " BIEN: La serie debe oscilar aleatoriamente alrededor de la linea roja.\n",
                         " LINEA ROJA: Media de los cambios. Si es cercana a cero, no hay deriva.\n",
                         " NOTA: Esta es la serie que los componentes AR(", p, ") y MA(", q, ")\n",
                         "   estan intentando modelar realmente.")
      ) +
      mi_tema
    
    print(g3)
    
  } else {
    # Caso donde d = 0 (Modelo es un ARMA simple)
    # CORRECCION 1: message en lugar de cat
    if (interactive()) {
      message(" d = 0: No hay serie diferenciada para mostrar. La serie original ya es estacionaria.")
    }
  }
  if (interactive()) {readline(prompt = "Grafica 3: Verificacion del efecto de la diferenciacion. > ")}
  
  
  # --- GRAFICA 4: ESTRUCTURA MATEMATICA (CIRCULO UNITARIO) ---
  
  # CORRECCION 4: Validacion defensiva de data_raices
  if (!is.null(x$data_raices) && nrow(x$data_raices) > 0) {
    # 1. Definicion del circulo de referencia
    theta_circ <- seq(0, 2*pi, length.out = 200)
    df_circle_arima <- data.frame(x = cos(theta_circ), y = sin(theta_circ))
    
    # 2. Construccion del grafico dual de raices
    g4 <- ggplot2::ggplot() +
      # Circulo unitario (Limite de estabilidad e invertibilidad)
      ggplot2::geom_path(data = df_circle_arima, ggplot2::aes(x, y),
                         color = "#7F8C8D", linetype = "dashed", linewidth = 0.8) +
      # area sombreada interna (Zona de validez)
      ggplot2::geom_polygon(data = df_circle_arima, ggplot2::aes(x, y),
                            fill = "#8E44AD", alpha = 0.05) +
      # Ejes de coordenadas
      ggplot2::geom_vline(xintercept = 0, color = "gray85") +
      ggplot2::geom_hline(yintercept = 0, color = "gray85") +
      # Raices AR (Rojas) y MA (Azules)
      ggplot2::geom_point(data = x$data_raices,
                          ggplot2::aes(x = Real, y = Imaginario, color = Tipo, shape = Tipo),
                          size = 4, stroke = 1.5) +
      # Colores y formas diferenciadas
      ggplot2::scale_color_manual(values = c("AR (Estabilidad)" = "#E74C3C",
                                             "MA (Invertibilidad)" = "#2980B9")) +
      ggplot2::scale_shape_manual(values = c("AR (Estabilidad)" = 16,
                                             "MA (Invertibilidad)" = 18)) +
      # Proporcion fija para que el circulo no parezca un ovalo
      ggplot2::coord_fixed(xlim = c(-1.1, 1.1), ylim = c(-1.1, 1.1)) +
      ggplot2::labs(
        title = "4. Estructura Matematica (Circulo Unitario)",
        subtitle = paste0("Raices del Polinomio ARIMA(", p, ",", d, ",", q, ")"),
        x = "Parte Real",
        y = "Parte Imaginaria",
        color = "Componente:", shape = "Componente:",
        caption = paste0("INTERPRETACION:\n",
                         " PUNTOS ROJOS (AR): Estabilidad del sistema dinamico.\n",
                         " ROMBOS AZULES (MA): Invertibilidad (unicidad del modelo).\n",
                         " CRITICO: Si una raiz toca el borde gris, el modelo es casi\n",
                         "   no-estacionario (posiblemente d requiere ser mayor).")
      ) +
      mi_tema + ggplot2::theme(legend.position = "right")
    
    print(g4)
    
  } else {
    # CORRECCION 1: message en lugar de cat
    if (interactive()) {
      message(paste0(" ARIMA(", p, ",", d, ",", q, ") sin componentes AR/MA para graficar raices."))
    }
  }
  if (interactive()) {readline(prompt = "Grafica 4: Analisis de la convergencia del modelo. > ")}
  
  # --- GRAFICA 5: RESIDUOS EN EL TIEMPO (E vs t) ---
  
  # CORRECCION 2: stats::sd
  sd_res_arima <- stats::sd(df$Residuos, na.rm = TRUE)
  
  g5 <- ggplot2::ggplot(df, ggplot2::aes(x = Tiempo, y = Residuos)) +
    # Banda de confianza (+-2 SD) en Purpura tenue
    ggplot2::geom_ribbon(ggplot2::aes(ymin = -2*sd_res_arima, ymax = 2*sd_res_arima),
                         fill = "#8E44AD", alpha = 0.1) +
    # Linea de los residuos (errores)
    ggplot2::geom_line(color = "#2C3E50", linewidth = 0.5) +
    # Puntos para detectar shocks o outliers
    ggplot2::geom_point(color = "#8E44AD", alpha = 0.4, size = 1) +
    # Linea central de referencia (Media Cero)
    ggplot2::geom_hline(yintercept = 0, linetype = "dashed", color = "black", linewidth = 0.8) +
    # Lineas criticas de control (+-2 SD)
    ggplot2::geom_hline(yintercept = c(-2*sd_res_arima, 2*sd_res_arima),
                        linetype = "dotted", color = "#E74C3C", alpha = 0.7) +
    ggplot2::labs(
      title = "5. Residuos en el Tiempo (E vs t)",
      subtitle = paste0("Analisis de Independencia para ARIMA(", p, ",", d, ",", q, ")"),
      x = "Tiempo (Periodos)",
      y = "Error Residual",
      caption = paste0("INTERPRETACION:\n",
                       " BIEN: Los puntos deben ser una nube aleatoria sin patron aparente.\n",
                       " ROJO: Puntos fuera de las lineas punteadas son 'shocks' imprevistos.\n",
                       " NOTA: Si ves que la nube se ensancha o se estrecha (Varianza no constante),\n",
                       "   el supuesto de Homocedasticidad se esta violando.")
    ) +
    mi_tema
  
  print(g5)
  if (interactive()) {readline(prompt = "Grafica 5: Inspeccion visual de los errores del modelo. > ")}
  
  # --- GRAFICA 6: AUTOCORRELACION DE RESIDUOS (ACF) ---
  
  # 1. Calculo de la ACF (Lags 1 a 20)
  acf_res_arima <- stats::acf(df$Residuos, plot = FALSE, lag.max = 20)
  df_acf_arima <- data.frame(
    Lag = as.numeric(acf_res_arima$lag[-1]),
    ACF = as.numeric(acf_res_arima$acf[-1])
  )
  
  # Limite de significancia estadistica (95%)
  limite_arima <- 1.96 / sqrt(nrow(df))
  
  # Logica de deteccion: Resaltar fallos en Rojo
  df_acf_arima$Etiqueta <- ifelse(abs(df_acf_arima$ACF) > limite_arima, as.character(df_acf_arima$Lag), "")
  df_acf_arima$vjust_pos <- ifelse(df_acf_arima$ACF > 0, -0.5, 1.5)
  
  # 2. Construccion de la Grafica (Identidad Purpura ARIMA)
  g6 <- ggplot2::ggplot(df_acf_arima, ggplot2::aes(x = Lag, y = ACF)) +
    # Barras de correlacion (Gris oscuro para sobriedad tecnica)
    ggplot2::geom_col(fill = "#34495E", width = 0.5) +
    # Lineas de significancia (Purpura ARIMA)
    ggplot2::geom_hline(yintercept = c(limite_arima, -limite_arima),
                        linetype = "dashed", color = "#8E44AD", linewidth = 0.8) +
    # Linea base
    ggplot2::geom_hline(yintercept = 0, color = "black") +
    # Alertas visuales (Numeros de Lag que fallan)
    ggplot2::geom_text(ggplot2::aes(label = Etiqueta, vjust = vjust_pos),
                       color = "#E74C3C", fontface = "bold", size = 3.5) +
    ggplot2::scale_x_continuous(breaks = 1:20) +
    ggplot2::labs(
      title = "6. Autocorrelacion de Residuos (ACF)",
      subtitle = "Se ha extraido toda la dependencia temporal?",
      x = "Lag (Retraso)",
      y = "Coeficiente ACF",
      caption = paste0("INTERPRETACION:\n",
                       " BIEN: Si todas las barras estan dentro de las lineas purpuras.\n",
                       " ROJO: El lag indicado aun tiene informacion no capturada por el modelo.\n",
                       " NOTA: El ARIMA(", p, ",", d, ",", q, ") busca que el error presente\n",
                       "   sea totalmente independiente de los errores pasados.")
    ) +
    mi_tema
  
  print(g6)
  if (interactive()) {readline(prompt = "Grafica 6: Validacion de independencia estadistica. > ")}
  
  # --- GRAFICA 7: AUTOCORRELACION PARCIAL DE RESIDUOS (PACF) ---
  
  # 1. Calculo de la PACF (Lags 1 a 20)
  pacf_res_arima <- stats::pacf(df$Residuos, plot = FALSE, lag.max = 20)
  df_pacf_arima <- data.frame(
    Lag = as.numeric(pacf_res_arima$lag),
    PACF = as.numeric(pacf_res_arima$acf)
  )
  
  # Limite de significancia (95%)
  limite_arima <- 1.96 / sqrt(nrow(df))
  
  # Logica de deteccion: Alertas en Rojo
  df_pacf_arima$Etiqueta <- ifelse(abs(df_pacf_arima$PACF) > limite_arima, as.character(df_pacf_arima$Lag), "")
  df_pacf_arima$vjust_pos <- ifelse(df_pacf_arima$PACF > 0, -0.5, 1.5)
  
  # 2. Construccion de la Grafica
  g7 <- ggplot2::ggplot(df_pacf_arima, ggplot2::aes(x = Lag, y = PACF)) +
    # Barras de la PACF (Color Purpura ARIMA)
    ggplot2::geom_col(fill = "#8E44AD", width = 0.5) +
    # Lineas de significancia (Naranja para precaucion tecnica)
    ggplot2::geom_hline(yintercept = c(limite_arima, -limite_arima),
                        linetype = "dashed", color = "#E67E22", linewidth = 0.8) +
    # Linea base
    ggplot2::geom_hline(yintercept = 0, color = "black") +
    # Alertas visuales
    ggplot2::geom_text(ggplot2::aes(label = Etiqueta, vjust = vjust_pos),
                       color = "#E74C3C", fontface = "bold", size = 3.5) +
    ggplot2::scale_x_continuous(breaks = 1:20) +
    ggplot2::labs(
      title = "7. Autocorrelacion Parcial de Residuos (PACF)",
      subtitle = "Identificacion de dependencias directas residuales",
      x = "Lag (Retraso)",
      y = "Coeficiente PACF",
      caption = paste0("INTERPRETACION:\n",
                       " BIEN: Todas las barras dentro de las lineas naranjas.\n",
                       " NUMEROS ROJOS: Indican que el componente AR(", p, ") no fue suficiente.\n",
                       " NOTA: El PACF limpia el efecto de los lags intermedios para ver la\n",
                       "   conexion pura entre el presente y el pasado del error.")
    ) +
    mi_tema
  
  print(g7)
  if (interactive()) {readline(prompt = "Grafica 7: Diagnostico final de la estructura del error. > ")}
  
  # --- GRAFICA 8: NORMALIDAD DE RESIDUOS (Q-Q PLOT) ---
  
  g8 <- ggplot2::ggplot(df, ggplot2::aes(sample = Residuos)) +
    # Puntos de cuantiles observados (Purpura ARIMA)
    ggplot2::stat_qq(color = "#8E44AD", alpha = 0.6, size = 2) +
    # Linea de referencia teorica (Distribucion Normal)
    ggplot2::stat_qq_line(color = "#2C3E50", linetype = "dashed", linewidth = 1) +
    ggplot2::labs(
      title = "8. Normalidad de los Errores (Q-Q Plot)",
      subtitle = "Distribucion de Residuos vs. Cuantiles Teoricos Normales",
      x = "Cuantiles Teoricos (Normal)",
      y = "Cuantiles Observados (Residuos ARIMA)",
      caption = paste0("INTERPRETACION:\n",
                       " BIEN: Los puntos deben estar alineados sobre la diagonal punteada.\n",
                       " COLAS: Si los extremos se alejan, hay valores extremos no capturados.\n",
                       " NOTA: La normalidad permite que las inferencias estadisticas sean exactas.\n",
                       " CLAVE: Si hay mucha curvatura, el modelo ARIMA puede estar sesgado.")
    ) +
    mi_tema
  
  print(g8)
  if (interactive()) {readline(prompt = "Grafica 8: Verificacion de la distribucion de los errores. > ")}
  
  
  # --- GRAFICA 9: PRECISION DEL MODELO (REAL VS AJUSTADO) ---
  
  # Calculamos la correlacion de Pearson para el reporte de precision
  cor_arima <- stats::cor(df$Observado, df$Ajustado, use = "complete.obs")
  
  g9 <- ggplot2::ggplot(df, ggplot2::aes(x = Ajustado, y = Observado)) +
    # Puntos de dispersion (Color Purpura ARIMA)
    ggplot2::geom_point(color = "#8E44AD", alpha = 0.5, size = 2) +
    # Linea de Identidad (Ideal: Real = Predicho)
    ggplot2::geom_abline(slope = 1, intercept = 0, linetype = "dashed", color = "#E74C3C", linewidth = 1) +
    # Linea de tendencia ajustada (Regresion de validacion)
    ggplot2::geom_smooth(method = "lm", formula = y ~ x, color = "#2C3E50", fill = "#BDC3C7", alpha = 0.2) +
    ggplot2::labs(
      title = "9. Precision del Modelo (Predicho vs Real)",
      subtitle = paste0("Correlacion de Pearson: ", round(cor_arima, 3)),
      x = "Valor Ajustado (Modelo ARIMA)",
      y = "Valor Observado (Realidad)",
      caption = paste0("INTERPRETACION:\n",
                       " LINEA ROJA: El escenario ideal de prediccion perfecta.\n",
                       " BIEN: Los puntos purpuras deben seguir la trayectoria de la linea roja.\n",
                       " DISPERSION: Indica la incertidumbre del modelo ante choques aleatorios.\n",
                       " NOTA: El ARIMA integra la tendencia para mejorar esta alineacion.")
    ) +
    mi_tema
  
  print(g9)
  if (interactive()) {readline(prompt = "Grafica 9: Evaluacion de la capacidad de ajuste integrada. > ")}
  
  # --- GRAFICA 10: DIAGNOSTICO DE ESTACIONARIEDAD FINAL ---
  
  # 1. Definir ventana (10% de los datos o minimo 5)
  ventana_arima <- max(5, round(nrow(df) * 0.1))
  
  # 2. Calculos sobre la Serie Original (para ver la estabilidad del ajuste)
  df_est_arima <- df
  df_est_arima$Media_Movil <- as.numeric(stats::filter(df$Observado, rep(1/ventana_arima, ventana_arima), sides = 2))
  
  # CORRECCION 2: stats::sd
  df_est_arima$SD_Movil <- sapply(1:nrow(df), function(i) {
    idx <- max(1, i-ventana_arima):min(nrow(df), i+ventana_arima)
    stats::sd(df$Observado[idx], na.rm = TRUE)
  })
  
  # 3. FILTRO (Eliminamos NAs de los extremos para un reporte limpio)
  df_final_arima <- df_est_arima[!is.na(df_est_arima$Media_Movil), ]
  
  # 4. CONSTRUCCION DE LA GRAFICA (Estilo Purpura ARIMA)
  g10 <- ggplot2::ggplot(df_final_arima, ggplot2::aes(x = Tiempo)) +
    # Serie original completa de fondo (Atenuada)
    ggplot2::geom_line(data = df, ggplot2::aes(y = Observado), color = "gray85", linewidth = 0.5) +
    
    # Franja de Varianza Movil (Purpura ARIMA)
    ggplot2::geom_ribbon(ggplot2::aes(ymin = Media_Movil - SD_Movil,
                                      ymax = Media_Movil + SD_Movil),
                         fill = "#8E44AD", alpha = 0.15) +
    
    # Linea de la Media Movil (Nivel local)
    ggplot2::geom_line(ggplot2::aes(y = Media_Movil), color = "#8E44AD", linewidth = 1.2) +
    
    # Linea de la Media Global (Referencia fija en Rojo)
    ggplot2::geom_hline(yintercept = mean(df$Observado, na.rm = TRUE),
                        linetype = "dashed", color = "#E74C3C", linewidth = 0.8) +
    
    ggplot2::labs(
      title = "10. Diagnostico de Estacionariedad Final",
      subtitle = paste0("Analisis Dinamico de Media y Varianza (Ventana k = ", ventana_arima, ")"),
      x = "Tiempo (Periodos)",
      y = "Valor de la Serie",
      caption = paste0("INTERPRETACION:\n",
                       " LINEA PURPURA: Media local. En un ARIMA con d=", d, ", debe seguir la tendencia.\n",
                       " LINEA ROJA: Promedio global de la serie original.\n",
                       " BIEN: Si la franja purpura tiene un ancho constante (Varianza estable).\n",
                       " NOTA: Si el ancho de la franja crece, el modelo ARIMA requiere transformacion Log.")
    ) +
    mi_tema
  
  print(g10)
  
  # CORRECCION 1: Usar message() en lugar de cat()
  if (interactive()) {
    message("Diagnostico ARIMA finalizado con exito (10/10 graficas).")
  }
  
  # CORRECCION 3: Retornar el objeto original (x)
  return(invisible(x))
}

#' Modelo Autorregresivo de Media Movil (ARMA)
#'
#' Esta funcion ajusta un modelo hibrido ARMA(p, q) que combina la dependencia
#' de los valores pasados (AutoRegresivo) con la influencia de los errores
#' pasados (Media Movil). Es la herramienta estandar para modelar series de
#' tiempo estacionarias con estructuras de dependencia complejas.
#'
#' @details
#' El flujo interno de la funcion realiza una auditoria estructural en cinco pasos:
#' \enumerate{
#'   \item **Estimacion Conjunta:** Utiliza Maxima Verosimilitud (MLE) para encontrar
#'         simultaneamente los coeficientes \eqn{\phi} (AR) y \eqn{\theta} (MA).
#'   \item **Estabilidad (Parte AR):** Verifica que el componente autorregresivo no sea
#'         explosivo. Las raices inversas del polinomio AR deben estar dentro del circulo.
#'   \item **Invertibilidad (Parte MA):** Verifica que el componente de media movil sea
#'         univoco. Las raices inversas del polinomio MA deben estar dentro del circulo.
#'   \item **Diagnostico de Residuos:** Evalua si el error resultante es Ruido Blanco
#'         mediante el test de Ljung-Box, asegurando que no quede senal sin capturar.
#'   \item **Criterio de Informacion:** Proporciona el AIC para facilitar la comparacion
#'         entre diferentes combinaciones de (p, q), buscando el equilibrio entre
#'         complejidad y precision.
#' }
#'
#' @section Guia de las 9 Graficas de Diagnostico (`plot`):
#' Al ejecutar \code{plot(modelo)}, se despliega una secuencia de validacion mixta:
#' \describe{
#'   \item{\strong{1. Inspeccion de Serie}}{Muestra la serie bruta con su media. Fundamental para confirmar visualmente la estacionariedad.}
#'   \item{\strong{2. Ajuste Mixto}}{Superpone el modelo (verde) sobre la realidad. Permite evaluar como interactuan la inercia y los shocks.}
#'   \item{\strong{3. Residuos en el Tiempo}}{Evaluacion de la aleatoriedad del error. Los puntos deben formar una nube sin patrones.}
#'   \item{\strong{4. Circulo Unitario Dual}}{Grafica critica exclusiva de ARMA. Muestra simultaneamente la Estabilidad (Puntos Rojos) e Invertibilidad (Rombos Azules).}
#'   \item{\strong{5. ACF de Residuos}}{Busca autocorrelaciones remanentes. Las barras deben ser minimas para confirmar que el modelo es optimo.}
#'   \item{\strong{6. PACF de Residuos}}{Ayuda a identificar si el orden autorregresivo (p) es suficiente.}
#'   \item{\strong{7. Normalidad (Q-Q Plot)}}{Valida que los errores sigan una campana de Gauss, requisito para la validez de los intervalos de confianza.}
#'   \item{\strong{8. Precision (Real vs Ajustado)}}{Muestra la correlacion directa. Un ARMA exitoso suele tener una alineacion estrecha sobre la diagonal.}
#'   \item{\strong{9. Estacionariedad Movil}}{Prueba final de estabilidad. Una franja verde horizontal confirma que el modelo no se desplaza en el tiempo.}
#' }
#'
#' @param x Vector numerico o serie de tiempo (\code{ts}).
#' @param p Orden Autorregresivo (pasados de la variable).
#' @param q Orden de Media Movil (pasados del error).
#' @param include_mean Logico. Incluir constante/intercepto? Por defecto \code{TRUE}.
#'
#' @return Un objeto de clase \code{"arma_greenreg"} que contiene:
#' \itemize{
#'   \item \code{modelo}: Objeto \code{arima} original (orden p, 0, q).
#'   \item \code{ordenes}: Vector con los valores de p y q.
#'   \item \code{coeficientes_tabla}: Matriz con estimaciones, errores estandar y p-valores.
#'   \item \code{estatus}: Lista con el estado de estabilidad e invertibilidad.
#'   \item \code{diagnosticos}: Resultados de Ljung-Box, Shapiro-Wilk y AIC.
#'   \item \code{data_plot}: Dataframe listo para visualizacion con \code{ggplot2}.
#' }
#'
#' @examples
#' data("datos_anomalia_temperatura")
#' ts_data <- datos_anomalia_temperatura$anomalia_c
#' modelo <- modelo_arma(ts_data, p = 1, q = 1)
#' modelo
#' plot(modelo)
#'
#' @importFrom stats arima coef residuals fitted Box.test pnorm qnorm shapiro.test acf pacf filter sd
#' @import ggplot2
#' @export
modelo_arma <- function(x, p = 1, q = 1, include_mean = TRUE) {

  # --- 1. Validacion y Ajuste ---
  if (!is.numeric(x)) stop("El argumento 'x' debe ser numerico.")

  # Ajuste ARIMA(p, 0, q)
  modelo_base <- stats::arima(x, order = c(p, 0, q), include.mean = include_mean)

  # --- 2. Coeficientes y Estadisticas ---
  coefs <- modelo_base$coef
  se <- sqrt(diag(modelo_base$var.coef))
  z_val <- coefs / se
  p_val <- 2 * (1 - stats::pnorm(abs(z_val)))

  tabla_coefs <- cbind(Estimate = coefs, `Std. Error` = se, `z value` = z_val, `Pr(>|z|)` = p_val)

  # --- 3. Analisis de Raices (Doble Verificacion) ---

  # A) Estabilidad (Parte AR) - Raices en ROJO
  ar_coefs <- coefs[grepl("^ar", names(coefs))]
  raices_ar_inv <- NULL
  es_estable <- TRUE

  if (length(ar_coefs) > 0) {
    poly_roots_ar <- polyroot(c(1, -ar_coefs))
    raices_ar_inv <- 1 / poly_roots_ar
    es_estable <- all(Mod(raices_ar_inv) < 1)
  }

  # B) Invertibilidad (Parte MA) - Raices en AZUL
  ma_coefs <- coefs[grepl("^ma", names(coefs))]
  raices_ma_inv <- NULL
  es_invertible <- TRUE

  if (length(ma_coefs) > 0) {
    poly_roots_ma <- polyroot(c(1, ma_coefs))
    raices_ma_inv <- 1 / poly_roots_ma
    es_invertible <- all(Mod(raices_ma_inv) < 1)
  }

  # --- 4. Diagnosticos de Residuos ---
  residuos <- stats::residuals(modelo_base)

  # Ljung-Box (Ruido Blanco)
  lb_test <- stats::Box.test(residuos, type = "Ljung-Box", lag = p + q + 5)

  # Shapiro-Wilk (Normalidad)
  if (length(residuos) >= 3 && length(residuos) <= 5000) {
    shapiro_test <- stats::shapiro.test(residuos)
  } else {
    shapiro_test <- list(p.value = NA)
  }

  # --- 5. Datos para Graficas ---
  ajustados <- x - residuos
  sigma <- sqrt(modelo_base$sigma2)
  # Intervalos 95%
  lower_ci <- ajustados - 1.96 * sigma
  upper_ci <- ajustados + 1.96 * sigma

  df_plot <- data.frame(
    Tiempo = 1:length(x),
    Observado = as.numeric(x),
    Ajustado = as.numeric(ajustados),
    Lower = as.numeric(lower_ci),
    Upper = as.numeric(upper_ci),
    Residuos = as.numeric(residuos)
  )

  # Datos combinados para circulo unitario
  df_raices <- data.frame(Real = numeric(), Imaginario = numeric(), Tipo = character())
  if (!is.null(raices_ar_inv)) {
    df_raices <- rbind(df_raices, data.frame(Real = Re(raices_ar_inv), Imaginario = Im(raices_ar_inv), Tipo = "AR (Estabilidad)"))
  }
  if (!is.null(raices_ma_inv)) {
    df_raices <- rbind(df_raices, data.frame(Real = Re(raices_ma_inv), Imaginario = Im(raices_ma_inv), Tipo = "MA (Invertibilidad)"))
  }

  # --- 6. Objeto Final ---
  resultado <- list(
    modelo = modelo_base,
    ordenes = c(p = p, q = q),
    coeficientes_tabla = tabla_coefs,
    estatus = list(estable = es_estable, invertible = es_invertible),
    diagnosticos = list(lb = lb_test, shapiro = shapiro_test, aic = modelo_base$aic),
    data_plot = df_plot,
    data_raices = df_raices
  )

  class(resultado) <- "arma_greenreg"
  return(resultado)
}

#' @rdname modelo_arma
#' @param x Objeto de clase.
#' @param ... Argumentos adicionales.
#' @export
print.arma_greenreg <- function(x, ...) {
  
  # Uso de sep="" para que los numeros p y q queden pegados a las comas
  cat("\n==========================================================\n")
  cat(" Modelo Mixto ARMA(", x$ordenes['p'], ",", x$ordenes['q'], ") \n", sep="")
  cat("==========================================================\n\n")
  
  cat("--- 1. COEFICIENTES ESTIMADOS ---\n")
  stats::printCoefmat(as.matrix(x$coeficientes_tabla), digits = 4, signif.stars = TRUE)
  
  # Proteccion contra NA en AIC
  aic_val <- x$diagnosticos$aic
  aic_txt <- if(is.null(aic_val) || is.na(aic_val)) "NA" else round(aic_val, 2)
  cat("\nAIC:", aic_txt, "(Menor es mejor)\n\n")
  
  cat("--- 2. VERIFICACION ESTRUCTURAL (Raices Unitarias) ---\n")
  
  # Check AR (Protegido contra NA)
  if (x$ordenes['p'] > 0) {
    est <- x$estatus$estable
    if (!is.null(est) && !is.na(est) && est) {
      cat(" Parte AR: ESTABLE (Raices dentro del circulo).\n")
    } else { 
      cat(" Parte AR: INESTABLE (Explosivo o Indeterminado).\n")
    }
  } else { 
    cat(" Parte AR: No aplica (p=0).\n") 
  }
  
  # Check MA (Protegido contra NA)
  if (x$ordenes['q'] > 0) {
    inv <- x$estatus$invertible
    if (!is.null(inv) && !is.na(inv) && inv) {
      cat(" Parte MA: INVERTIBLE (Modelo unico).\n")
    } else { 
      cat(" Parte MA: NO INVERTIBLE (Problemas de identificacion).\n")
    }
  } else { 
    cat(" Parte MA: No aplica (q=0).\n") 
  }
  cat("\n")
  
  cat("--- 3. DIAGNOSTICO DE RESIDUOS ---\n")
  
  # Proteccion contra NA en Ljung-Box y sprintf
  lb_p <- x$diagnosticos$lb$p.value
  lb_txt <- if(is.null(lb_p) || is.na(lb_p)) "NA" else sprintf("%.4f", lb_p)
  
  cat(sprintf(" Independencia (Ljung-Box): p-valor = %s\n", lb_txt))
  
  # Evaluacion logica blindada
  if (!is.null(lb_p) && !is.na(lb_p) && lb_p > 0.05) {
    cat(" Ruido Blanco: El modelo capturo la dinamica temporal.\n")
  } else {
    cat(" Autocorrelacion: El modelo ARMA actual es insuficiente o indeterminado.\n")
  }
  
  cat("\n Usa plot(modelo) para graficas de diagnostico del modelo ARMA.\n")
  cat("==========================================================\n")
  
  # Retorno obligatorio S3
  return(invisible(x))
}

#' @rdname modelo_arma
#' @description Genera una secuencia de 9 graficas de diagnostico para validar el modelo ARMA.
#' @param x Objeto de clase.
#' @param ... Argumentos adicionales.
#' @export
plot.arma_greenreg <- function(x, ...) {
  
  df <- x$data_plot
  p  <- x$ordenes['p'] # Extraemos p del objeto x
  q  <- x$ordenes['q'] # Extraemos q del objeto x
  
  # TEMA ESTANDAR
  mi_tema <- ggplot2::theme_minimal() +
    ggplot2::theme(
      plot.title = ggplot2::element_text(face = "bold", size = 14, color = "#2C3E50"),
      plot.subtitle = ggplot2::element_text(size = 11, color = "#7F8C8D"),
      plot.caption = ggplot2::element_text(hjust = 0, size = 9, face = "italic",
                                           color = "#555555", margin = ggplot2::margin(t = 15),
                                           lineheight = 1.1),
      plot.margin = ggplot2::margin(10, 10, 20, 10),
      axis.title = ggplot2::element_text(face = "bold"),
      legend.position = "top"
    )
  
  # CORRECCION: Usar message
  if (interactive()) {
    message("Generando graficas ARMA... (Presiona [Enter] para avanzar)")
  }
  
  # --- GRAFICA 1: SERIE ORIGINAL (VALORES VS TIEMPO) ---
  
  media_global <- mean(df$Observado, na.rm = TRUE)
  
  g1 <- ggplot2::ggplot(df, ggplot2::aes(x = Tiempo, y = Observado)) +
    ggplot2::geom_line(color = "#34495E", linewidth = 0.7) +
    ggplot2::geom_point(color = "#34495E", alpha = 0.3, size = 1) +
    ggplot2::geom_hline(yintercept = media_global, linetype = "dashed",
                        color = "#16A085", linewidth = 0.8) +
    ggplot2::labs(
      title = "1. Comportamiento de la Serie Original",
      subtitle = "Visualizacion de la serie temporal antes del ajuste mixto",
      x = "Tiempo (Periodos)",
      y = "Valor Observado",
      caption = paste0("INTERPRETACION:\n",
                       " LINEA VERDE: Promedio historico de la serie.\n",
                       " OBJETIVO: El modelo ARMA asume que la serie fluctua alrededor\n",
                       "   de este nivel constante (Estacionariedad en media).\n",
                       " NOTA: Si ves una 'escalera' o tendencia, el modelo ARMA(p,q)\n",
                       "   podria requerir una diferencia previa (Modelo ARIMA).")
    ) +
    mi_tema
  
  print(g1)
  if (interactive()) {readline(prompt = "Grafica 1: Inspeccion de los datos brutos > ")}
  
  
  # --- GRAFICA 2: AJUSTE DEL MODELO ARMA (Real vs Ajustado) ---
  
  g2 <- ggplot2::ggplot(df, ggplot2::aes(x = Tiempo)) +
    ggplot2::geom_ribbon(ggplot2::aes(ymin = Lower, ymax = Upper),
                         fill = "#16A085", alpha = 0.15) +
    ggplot2::geom_line(ggplot2::aes(y = Observado, color = "Observado"), linewidth = 0.7) +
    ggplot2::geom_line(ggplot2::aes(y = Ajustado, color = "Modelo ARMA"), linewidth = 0.8) +
    ggplot2::scale_color_manual(values = c("Observado" = "#34495E", "Modelo ARMA" = "#16A085")) +
    ggplot2::labs(
      title = paste0("2. Ajuste del Modelo ARMA(", p, ",", q, ")"),
      subtitle = "Comparacion de valores reales vs. estimacion mixta",
      x = "Tiempo (Periodos)",
      y = "Valor",
      color = "Referencia:",
      caption = paste0("INTERPRETACION:\n",
                       " BIEN: La linea verde debe 'abrazar' los movimientos de la oscura.\n",
                       " SOMBRA: Zona de confianza. El 95% de los datos deben caer aqui.\n",
                       " NOTA: El ARMA es mas potente que el AR o MA por separado,\n",
                       "   logrando un equilibrio entre tendencia y choques aleatorios.")
    ) +
    mi_tema
  
  print(g2)
  if (interactive()) {readline(prompt = "Grafica 2: Evaluacion visual del ajuste mixto > ")}
  
  
  # --- GRAFICA 3: RESIDUOS (ERRORES) VS TIEMPO ---
  
  # CORRECCION: stats::sd
  sd_res_arma <- stats::sd(df$Residuos, na.rm = TRUE)
  
  g3 <- ggplot2::ggplot(df, ggplot2::aes(x = Tiempo, y = Residuos)) +
    ggplot2::geom_ribbon(ggplot2::aes(ymin = -2*sd_res_arma, ymax = 2*sd_res_arma),
                         fill = "#16A085", alpha = 0.1) +
    ggplot2::geom_line(color = "#2C3E50", linewidth = 0.5) +
    ggplot2::geom_point(color = "#16A085", alpha = 0.4, size = 1) +
    ggplot2::geom_hline(yintercept = 0, linetype = "dashed", color = "black", linewidth = 0.8) +
    ggplot2::geom_hline(yintercept = c(-2*sd_res_arma, 2*sd_res_arma),
                        linetype = "dotted", color = "#E74C3C", alpha = 0.7) +
    ggplot2::labs(
      title = "3. Analisis de Residuos (E vs t)",
      subtitle = "Verificacion de Aleatoriedad y Media Cero",
      x = "Tiempo (Periodos)",
      y = "Residuo (Error)",
      caption = paste0("INTERPRETACION:\n",
                       " BIEN: Los errores deben ser una 'nube' aleatoria centrada en cero.\n",
                       " ROJO: Puntos fuera de estas lineas son shocks no explicados (Outliers).\n",
                       " MAL: Si ves ciclos, 'olas' o tendencias, el modelo no es suficiente.\n",
                       " NOTA: El ARMA busca que el error de hoy no dependa del error de ayer.")
    ) +
    mi_tema
  
  print(g3)
  if (interactive()) {readline(prompt = "Grafica 3: Inspeccion de la calidad del error residual > ")}
  
  
  # --- GRAFICA 4: ESTRUCTURA (CIRCULO UNITARIO AR Y MA) ---
  
  # CORRECCION defensiva: validar is.null
  if (!is.null(x$data_raices) && nrow(x$data_raices) > 0) {
    theta_circ <- seq(0, 2*pi, length.out = 200)
    df_circle_arma <- data.frame(x = cos(theta_circ), y = sin(theta_circ))
    
    g4 <- ggplot2::ggplot() +
      ggplot2::geom_path(data = df_circle_arma, ggplot2::aes(x, y),
                         color = "#7F8C8D", linetype = "dashed", linewidth = 0.8) +
      ggplot2::geom_polygon(data = df_circle_arma, ggplot2::aes(x, y),
                            fill = "#BDC3C7", alpha = 0.1) +
      ggplot2::geom_vline(xintercept = 0, color = "gray80") +
      ggplot2::geom_hline(yintercept = 0, color = "gray80") +
      ggplot2::geom_point(data = x$data_raices,
                          ggplot2::aes(x = Real, y = Imaginario, color = Tipo, shape = Tipo),
                          size = 4, stroke = 1.2) +
      ggplot2::scale_color_manual(values = c("AR (Estabilidad)" = "#E74C3C",
                                             "MA (Invertibilidad)" = "#2980B9")) +
      ggplot2::scale_shape_manual(values = c("AR (Estabilidad)" = 16,
                                             "MA (Invertibilidad)" = 18)) +
      ggplot2::coord_fixed(xlim = c(-1.2, 1.2), ylim = c(-1.2, 1.2)) +
      ggplot2::labs(
        title = "4. Estructura de Raices (Circulo Unitario)",
        subtitle = paste0("Diagnostico Dual: ARMA(", p, ",", q, ")"),
        x = "Parte Real",
        y = "Parte Imaginaria",
        color = "Componente:", shape = "Componente:",
        caption = paste0("INTERPRETACION:\n",
                         " PUNTOS ROJOS (AR): Si estan dentro, el modelo es ESTABLE.\n",
                         " ROMBOS AZULES (MA): Si estan dentro, el modelo es INVERTIBLE.\n",
                         " CRITICO: Si algun punto toca o sale del circulo gris,\n",
                         "   los resultados estadisticos NO son confiables.")
      ) +
      mi_tema + ggplot2::theme(legend.position = "right")
    
    print(g4)
    
  } else {
    # CORRECCION: Usar message
    message("El modelo ARMA(0,0) no tiene raices; es ruido blanco.")
  }
  if (interactive()) {readline(prompt = "Grafica 4: Verificacion de estabilidad e invertibilidad mixta > ")}
  
  
  # --- GRAFICA 5: AUTOCORRELACION DE RESIDUOS (ACF) ---
  
  acf_res_arma <- stats::acf(df$Residuos, plot = FALSE, lag.max = 20)
  df_acf_arma <- data.frame(
    Lag = as.numeric(acf_res_arma$lag[-1]),
    ACF = as.numeric(acf_res_arma$acf[-1])
  )
  
  limite_arma <- 1.96 / sqrt(nrow(df))
  
  df_acf_arma$Etiqueta <- ifelse(abs(df_acf_arma$ACF) > limite_arma, as.character(df_acf_arma$Lag), "")
  df_acf_arma$vjust_pos <- ifelse(df_acf_arma$ACF > 0, -0.5, 1.5)
  
  g5 <- ggplot2::ggplot(df_acf_arma, ggplot2::aes(x = Lag, y = ACF)) +
    ggplot2::geom_col(fill = "#34495E", width = 0.4) +
    ggplot2::geom_hline(yintercept = c(limite_arma, -limite_arma),
                        linetype = "dashed", color = "#16A085", linewidth = 0.8) +
    ggplot2::geom_hline(yintercept = 0, color = "black") +
    ggplot2::geom_text(ggplot2::aes(label = Etiqueta, vjust = vjust_pos),
                       color = "#E74C3C", fontface = "bold", size = 3.5) +
    ggplot2::scale_x_continuous(breaks = 1:20) +
    ggplot2::labs(
      title = "5. Autocorrelacion de Residuos (ACF)",
      subtitle = "Quedo estructura temporal en el error?",
      x = "Lag (Retraso)",
      y = "Coeficiente ACF",
      caption = paste0("INTERPRETACION:\n",
                       " BIEN: Si todas las barras estan 'atrapadas' en las lineas verdes.\n",
                       " NUMEROS ROJOS: Indican que lags NO son ruido blanco.\n",
                       " NOTA: El ARMA busca 'limpiar' la ACF. Si el Lag 1 o 2 es rojo,\n",
                       "   el modelo mixto aun no es optimo.")
    ) +
    mi_tema
  
  print(g5)
  if (interactive()) {readline(prompt = "Grafica 5: Diagnostico de independencia de los errores mixtos > ")}
  
  
  # --- GRAFICA 6: AUTOCORRELACION PARCIAL (PACF) DE RESIDUOS ---
  
  pacf_res_arma <- stats::pacf(df$Residuos, plot = FALSE, lag.max = 20)
  
  df_pacf_arma <- data.frame(
    Lag = as.numeric(pacf_res_arma$lag),
    PACF = as.numeric(pacf_res_arma$acf)
  )
  
  limite_arma <- 1.96 / sqrt(nrow(df))
  
  df_pacf_arma$Etiqueta <- ifelse(abs(df_pacf_arma$PACF) > limite_arma, as.character(df_pacf_arma$Lag), "")
  df_pacf_arma$vjust_pos <- ifelse(df_pacf_arma$PACF > 0, -0.5, 1.5)
  
  g6 <- ggplot2::ggplot(df_pacf_arma, ggplot2::aes(x = Lag, y = PACF)) +
    ggplot2::geom_col(fill = "#16A085", width = 0.4) +
    ggplot2::geom_hline(yintercept = c(limite_arma, -limite_arma),
                        linetype = "dashed", color = "#E67E22", linewidth = 0.8) +
    ggplot2::geom_hline(yintercept = 0, color = "black") +
    ggplot2::geom_text(ggplot2::aes(label = Etiqueta, vjust = vjust_pos),
                       color = "#E74C3C", fontface = "bold", size = 3.5) +
    ggplot2::scale_x_continuous(breaks = 1:20) +
    ggplot2::labs(
      title = "6. Autocorrelacion Parcial (PACF)",
      subtitle = "Identificacion de dependencias directas en errores mixtos",
      x = "Lag (Retraso)",
      y = "Coeficiente PACF",
      caption = paste0("INTERPRETACION:\n",
                       " BIEN: Ninguna barra sobresale de las lineas naranjas.\n",
                       " NUMEROS ROJOS: Indican que ese lag especifico aun tiene 'senal'.\n",
                       " NOTA: Si el Lag 1 es significativo, el componente AR(", p, ") \n",
                       "   no esta capturando bien la inercia inmediata.")
    ) +
    mi_tema
  
  print(g6)
  if (interactive()) {readline(prompt = "Grafica 6: Auditoria final de la estructura del error > ")}
  
  
  # --- GRAFICA 7: NORMALIDAD DE RESIDUOS (Q-Q PLOT) ---
  
  g7 <- ggplot2::ggplot(df, ggplot2::aes(sample = Residuos)) +
    ggplot2::stat_qq(color = "#16A085", alpha = 0.6, size = 2) +
    ggplot2::stat_qq_line(color = "#2C3E50", linetype = "dashed", linewidth = 1) +
    ggplot2::labs(
      title = "7. Normalidad de los Errores (Q-Q Plot)",
      subtitle = "Comparacion de Cuantiles de Residuos Mixtos vs. Normal Teorica",
      x = "Cuantiles Teoricos (Normal)",
      y = "Cuantiles Observados (Residuos ARMA)",
      caption = paste0("INTERPRETACION:\n",
                       " BIEN: Los puntos deben seguir la linea diagonal punteada.\n",
                       " COLAS: Si los extremos se curvan, hay eventos extremos (outliers).\n",
                       " NOTA: La normalidad asegura que el error tipico sea consistente.\n",
                       " CLAVE: Si los puntos se alejan mucho, los p-valores del modelo son dudosos.")
    ) +
    mi_tema
  
  print(g7)
  if (interactive()) {readline(prompt = "Grafica 7: Verificacion de la distribucion gaussiana del error > ")}
  
  
  # --- GRAFICA 8: PRECISION DEL MODELO (REAL VS AJUSTADO) ---
  
  cor_arma <- stats::cor(df$Observado, df$Ajustado, use = "complete.obs")
  
  g8 <- ggplot2::ggplot(df, ggplot2::aes(x = Ajustado, y = Observado)) +
    ggplot2::geom_point(color = "#16A085", alpha = 0.5, size = 2) +
    ggplot2::geom_abline(slope = 1, intercept = 0, linetype = "dashed", color = "#E74C3C", linewidth = 1) +
    ggplot2::geom_smooth(method = "lm", formula = y ~ x, color = "#2C3E50", fill = "#BDC3C7", alpha = 0.2) +
    ggplot2::labs(
      title = "8. Precision del Modelo (Predicho vs Real)",
      subtitle = paste0("Correlacion de Pearson: ", round(cor_arma, 3)),
      x = "Valor Ajustado (Modelo ARMA)",
      y = "Valor Observado (Realidad)",
      caption = paste0("INTERPRETACION:\n",
                       " LINEA ROJA: El ideal donde el modelo predice exactamente la realidad.\n",
                       " BIEN: Los puntos verdes deben seguir la trayectoria de la linea roja.\n",
                       " DISPERSION: Si los puntos estan muy lejos de la linea, el modelo es pobre.\n",
                       " CLAVE: Un ARMA exitoso debe tener una correlacion superior a 0.7.")
    ) +
    mi_tema
  
  print(g8)
  if (interactive()) {readline(prompt = "Grafica 8: Analisis de correlacion y capacidad predictiva > ")}
  
  
  # --- GRAFICA 9: DIAGNOSTICO DE ESTACIONARIEDAD (MOVIL) ---
  
  ventana_arma <- max(5, round(nrow(df) * 0.1))
  
  df_est_arma <- df
  
  df_est_arma$Media_Movil <- as.numeric(stats::filter(df$Observado, rep(1/ventana_arma, ventana_arma), sides = 2))
  
  # CORRECCION: stats::sd
  df_est_arma$SD_Movil <- sapply(1:nrow(df), function(i) {
    idx <- max(1, i-ventana_arma):min(nrow(df), i+ventana_arma)
    stats::sd(df$Observado[idx], na.rm = TRUE)
  })
  
  df_final_grafica <- df_est_arma[!is.na(df_est_arma$Media_Movil), ]
  
  g9 <- ggplot2::ggplot(df_final_grafica, ggplot2::aes(x = Tiempo)) +
    ggplot2::geom_line(data = df, ggplot2::aes(y = Observado), color = "gray85", linewidth = 0.5) +
    ggplot2::geom_ribbon(ggplot2::aes(ymin = Media_Movil - SD_Movil,
                                      ymax = Media_Movil + SD_Movil),
                         fill = "#16A085", alpha = 0.15) +
    ggplot2::geom_line(ggplot2::aes(y = Media_Movil), color = "#16A085", linewidth = 1.2) +
    ggplot2::geom_hline(yintercept = mean(df$Observado, na.rm = TRUE),
                        linetype = "dashed", color = "#E74C3C", linewidth = 0.8) +
    ggplot2::labs(
      title = "9. Diagnostico de Estacionariedad",
      subtitle = paste0("Analisis de Media y Varianza Movil (Ventana k = ", ventana_arma, ")"),
      x = "Tiempo (Periodos)",
      y = "Valor de la Serie",
      caption = paste0("INTERPRETACION:\n",
                       " LINEA VERDE: Media local. Debe ser horizontal y estable.\n",
                       " LINEA ROJA: Promedio global. La base del modelo.\n",
                       " BIEN: Si la franja verde mantiene un ancho constante.\n",
                       " NOTA: Se han omitido los extremos para evitar errores de calculo movil.")
    ) +
    mi_tema
  
  print(g9)
  
  # CORRECCION: Usar message y devolver el objeto x
  if (interactive()) {
    message("Recuerda que un buen modelo ARMA es el equilibrio entre parsimonia y precision.")
    message("Diagnostico ARMA completado.")
  }
  
  return(invisible(x))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Modelo_Autorregresivo_(AR).R
\name{modelo_ar}
\alias{modelo_ar}
\alias{print.ar_greenreg}
\alias{plot.ar_greenreg}
\title{Modelo Autorregresivo (AR)}
\usage{
modelo_ar(x, p = 1, nivel_confianza = 0.95)

\method{print}{ar_greenreg}(x, ...)

\method{plot}{ar_greenreg}(x, ...)
}
\arguments{
\item{x}{Objeto de clase ar_greenreg.}

\item{p}{Orden del modelo (numero de rezagos a considerar). Por defecto es 1.}

\item{nivel_confianza}{Nivel de significancia para las bandas de error (default 0.95).}

\item{...}{Argumentos adicionales.}
}
\value{
Un objeto de clase \code{"ar_greenreg"} que contiene:
\itemize{
\item \code{modelo}: Objeto \code{arima} original.
\item \code{orden_p}: El orden del modelo ajustado.
\item \code{coeficientes}: Matriz con estimaciones, errores estandar y p-valores.
\item \code{estabilidad}: Lista con las raaces inversas y diagnostico de estacionariedad.
\item \code{diagnosticos}: Resultados de las pruebas de Ljung-Box, Shapiro-Wilk y AIC.
\item \code{notas}: Reporte interpretado sobre estabilidad y ruido blanco.
\item \code{data_plot}: Dataframe listo para visualización avanzada.
}
}
\description{
Esta funcion ajusta un modelo de series de tiempo donde el valor actual \eqn{(Y_t)}
se explica mediante una combinacion lineal de sus propios valores pasados
\eqn{(Y_{t-1}, Y_{t-2}, \dots)}. Es la herramienta estandar para analizar la
"persistencia" o "memoria" de un fenomeno a traves del tiempo.

Genera una secuencia de 9 graficas de diagnostico para validar el modelo AR.
}
\details{
El flujo interno de la funcion se divide en cinco etapas diagnosticas:
\enumerate{
\item \strong{Estimacion (MLE):} Utiliza Maxima Verosimilitud para encontrar los
coeficientes \eqn{\phi} (Phi) que mejor describen la dependencia temporal.
\item \strong{Analisis de Estabilidad:} Calcula las raices inversas del polinomio
caracteristico. Un modelo AR solo es valido si es "Estacionario", es decir,
si las raices se encuentran dentro del circulo unitario.
\item \strong{Inferencia de Parametros:} Calcula errores estandar y p-valores para
determinar cuantos rezagos (lags) son realmente significativos.
\item \strong{Auditoria de Residuos:} Evalua si el error restante es "Ruido Blanco"
mediante la prueba de Ljung-Box (independencia) y Shapiro-Wilk (normalidad).
\item \strong{Traduccion Didactica:} Genera un reporte que explica si el modelo
"explota" (inestable) o si ha capturado exitosamente la memoria de la serie.
}
}
\section{Guia de las 9 Graficas de Diagnostico (\code{plot})}{

Al ejecutar \code{plot(modelo)}, se despliega una auditoria temporal completa:
\describe{
\item{\strong{1. Inspeccion Visual}}{Muestra la serie bruta con su media historica. Ayuda a detectar tendencias que el modelo AR no puede manejar por si solo.}
\item{\strong{2. Ajuste Temporal}}{Superpone el modelo sobre la realidad. La sombra roja indica la incertidumbre (Intervalo de Confianza).}
\item{\strong{3. Residuos en el Tiempo}}{Un "electrocardiograma" del error. No debe presentar ondas, ciclos ni tendencias.}
\item{\strong{4. Circulo Unitario}}{Grafica matematica critica. Si los rombos rojos salen del circulo, el modelo es inestable y no sirve para pronosticar.}
\item{\strong{5. ACF de Residuos}}{Busca correlaciones remanentes. Las barras deben ser pequenas para confirmar que el error es ruido aleatorio.}
\item{\strong{6. PACF de Residuos}}{Ayuda a confirmar si el orden \code{p} seleccionado fue suficiente para limpiar la dependencia directa.}
\item{\strong{7. Normalidad (Q-Q Plot)}}{Valida que los choques aleatorios sigan una distribucion normal, requisito para que los intervalos de confianza sean exactos.}
\item{\strong{8. Precision (Real vs Ajustado)}}{Muestra la correlacion directa entre la prediccion y el dato observado.}
\item{\strong{9. Estacionariedad Movil}}{Analiza si la media y la varianza cambian a lo largo de la serie. Una linea verde horizontal confirma un modelo robusto.}
}
}

\examples{

data("datos_anomalia_temperatura")
modelo_AR <- modelo_ar(datos_anomalia_temperatura$anomalia_c, p = 2)
modelo_AR
plot(modelo_AR)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Modelo_media_movil_(MA).R
\name{modelo_ma}
\alias{modelo_ma}
\alias{print.ma_greenreg}
\alias{plot.ma_greenreg}
\title{Modelo de Media Movil (MA)}
\usage{
modelo_ma(x, q = 1, include_mean = TRUE)

\method{print}{ma_greenreg}(x, ...)

\method{plot}{ma_greenreg}(x, ...)
}
\arguments{
\item{x}{Objeto de clase.}

\item{q}{Orden del modelo (numero de choques pasados a considerar). Por defecto es 1.}

\item{include_mean}{Logico. Incluir el promedio constante de la serie? Por defecto \code{TRUE}.}

\item{...}{Argumentos adicionales.}
}
\value{
Un objeto de clase \code{"ma_greenreg"}, que consiste en una lista con:
\itemize{
\item \code{modelo}: El objeto \code{arima} original (orden 0,0,q).
\item \code{orden_q}: El orden del modelo ajustado.
\item \code{coeficientes_tabla}: Matriz con estimaciones, errores estandar y p-valores.
\item \code{invertibilidad}: Lista con las raices inversas y diagnostico de invertibilidad.
\item \code{diagnosticos}: Resultados de las pruebas de Ljung-Box, Shapiro-Wilk y AIC.
\item \code{data_plot}: Dataframe listo para visualizacion con \code{ggplot2}.
}
}
\description{
Esta funcion ajusta un modelo de series de tiempo donde el valor actual \eqn{(Y_t)}
se explica mediante una combinacion lineal de los errores (shocks) pasados
(\eqn{\epsilon_{t-1}}, \eqn{\epsilon_{t-2}}, ...). Es ideal para modelar fenomenos con
efectos de memoria corta o impactos transitorios.

Genera una secuencia de 9 graficas de diagnostico para validar el modelo MA.
}
\details{
El flujo interno de la funcion se divide en cinco etapas fundamentales:
\enumerate{
\item \strong{Estimacion (MLE):} Utiliza Maxima Verosimilitud para estimar los
parametros \eqn{\theta} (Theta). A diferencia del AR, el MA requiere
algoritmos iterativos ya que los errores no son directamente observables.
\item \strong{Analisis de Invertibilidad:} Calcula las raices inversas del polinomio
caracteristico. Un modelo MA debe ser "Invertible" para que el error
pueda expresarse como una funcion convergente de los datos observados.
\item \strong{Identificacion Estructural:} Evalua la ACF (Autocorrelacion). En un
proceso MA(q), la ACF debe "cortarse" abruptamente despues del lag q.
\item \strong{Auditoria de Residuos:} Verifica que el error remanente sea Ruido Blanco
mediante la prueba de Ljung-Box y la distribucion Gaussiana (Shapiro-Wilk).
\item \strong{Traduccion Didactica:} Genera un reporte que explica la persistencia
de los choques y valida si el modelo es unico y matematicamente solido.
}
}
\section{Guia de las 9 Graficas de Diagnostico (\code{plot})}{

Al ejecutar \code{plot(modelo)}, se despliega una auditoria de "shocks" temporales:
\describe{
\item{\strong{1. Inspeccion Visual}}{Muestra la serie bruta con su media historica. El modelo MA supone que la serie siempre es atraida hacia este nivel central.}
\item{\strong{2. Ajuste Temporal}}{Superpone el modelo (azul) sobre la realidad. Permite ver como el modelo "suaviza" los picos de la serie.}
\item{\strong{3. Precision (Real vs Ajustado)}}{Grafica de dispersion para evaluar la correlacion. Los puntos deben seguir la diagonal roja.}
\item{\strong{4. Circulo Unitario}}{Prueba de Invertibilidad. Los rombos azules deben estar dentro del circulo para asegurar que el modelo sea unico.}
\item{\strong{5. Residuos en el Tiempo}}{Evaluacion de aleatoriedad. No debe haber patrones; los errores deben ser impredecibles.}
\item{\strong{6. ACF de Residuos}}{Prueba critica de Ruido Blanco. Si hay barras largas (numeros rojos), el orden \code{q} es insuficiente.}
\item{\strong{7. PACF de Residuos}}{Complemento de la ACF para verificar que no existan dependencias directas remanentes.}
\item{\strong{8. Normalidad (Q-Q Plot)}}{Valida que los choques sigan una campana de Gauss, asegurando la validez de los intervalos de confianza.}
\item{\strong{9. Estacionariedad Movil}}{Confirma que la media y varianza no se desplazan. Vital para asegurar que el modelo MA sea aplicable.}
}
}

\examples{
data("datos_anomalia_temperatura")
modelo_MA <- modelo_ma(datos_anomalia_temperatura$anomalia_c, q = 1, include_mean = TRUE)
modelo_MA
plot(modelo_MA)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datos_tala_ilegal_poisson.R
\docType{data}
\name{datos_tala_ilegal_poisson}
\alias{datos_tala_ilegal_poisson}
\title{Registros de tala ilegal en distintas regiones}
\format{
Un data frame con 200 observaciones y 4 variables:
\describe{
\item{numero_talas}{Numero de eventos de tala ilegal observados}
\item{densidad_poblacion}{Densidad poblacional de la region (hab/km2)}
\item{superficie_forestal_ha}{Superficie forestal en hectareas}
\item{dias_vigilancia}{Numero de dias de vigilancia reportados}
}
}
\source{
Elaboracion propia con fines didacticos tomando referencia de CONAFOR e INEGI.
}
\usage{
datos_tala_ilegal_poisson
}
\description{
Conjunto de datos que contiene el numero de eventos de tala ilegal observados
junto con variables explicativas relacionadas con la densidad poblacional,
superficie forestal y esfuerzo de vigilancia. Ideal para ejemplificar la aplicacion
de modelos de regresion Poisson.
}
\details{
Los datos fueron generados de manera simulada con base en valores reportados
por CONAFOR y SEMARNAT. Esto permite realizar ejercicios reproducibles y seguros
sin exponer datos sensibles o confidenciales.
}
\examples{
data(datos_tala_ilegal_poisson)
modelo_poisson <- reg_poisson(numero_talas ~ densidad_poblacion + 
                              superficie_forestal_ha + dias_vigilancia, 
                              data = datos_tala_ilegal_poisson)
                              

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datos_nivel_presa.R
\docType{data}
\name{datos_nivel_presa}
\alias{datos_nivel_presa}
\title{Niveles de agua en presas de Mexico}
\format{
Un data frame con 1825 observaciones y 2 variables:
\describe{
\item{fecha}{Fecha del registro (AAAA-MM-DD)}
\item{nivel_m}{Nivel de almacenamiento de la presa (metros)}
}
}
\source{
Elaboracion propia con fines didacticos basada en patrones de datos de CONAGUA.
}
\usage{
datos_nivel_presa
}
\description{
Conjunto de datos que presenta los niveles de almacenamiento de agua
registrados en distintas presas y fechas, util para analisis de series de tiempo
y modelos ARIMA.
}
\details{
Los datos fueron simulados a partir de patrones observados en reportes
de CONAGUA, manteniendo coherencia temporal y realismo. Se generaron con fines
didacticos para demostrar la aplicacion de modelos AR, MA y ARIMA en \pkg{GREENREG}.
}
\examples{
data("datos_nivel_presa")
ts_data <- datos_nivel_presa$nivel_m
modelo_ARIMA <- modelo_arima(ts_data, p = 1, d = 1, q = 1)
print(modelo_ARIMA)
plot(modelo_ARIMA)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Regresion_poisson.R
\name{reg_poisson}
\alias{reg_poisson}
\alias{print.poisson_greenreg}
\alias{plot.poisson_greenreg}
\title{Regresion de Poisson (Modelos de Conteo)}
\usage{
reg_poisson(formula, data)

\method{print}{poisson_greenreg}(x, ...)

\method{plot}{poisson_greenreg}(x, ...)
}
\arguments{
\item{formula}{Una formula de R (ej. \code{numero_frutos ~ riego + fertilizante}).}

\item{data}{Un \code{data.frame} con las observaciones (el conteo debe ser entero).}

\item{x}{Objeto de clase.}

\item{...}{Argumentos adicionales.}
}
\value{
Un objeto de clase \code{"poisson_greenreg"} que contiene:
\itemize{
\item \code{modelo}: Objeto \code{glm} original de la familia Poisson.
\item \code{irr_table}: Tabla con los Ratios de Tasas de Incidencia, intervalos y p-valores.
\item \code{dispersion}: El ratio de dispersion calculado (Pearson / GL).
\item \code{metricas}: Lista con AIC, Log-Verosimilitud y Devianzas.
\item \code{predicciones}: Dataframe con valores observados, lambda estimado y diagnosticos (leverage/residuos).
}
}
\description{
Esta funcion ajusta un Modelo Lineal Generalizado (GLM) de la familia Poisson con
funcion de enlace logaritmica \eqn{\ln(\lambda)}. Esta disenada para modelar variables
de respuesta que representan conteos enteros no negativos (0, 1, 2, 3...) y tasas
de incidencia en estudios agronomicos, forestales y sociales.

Genera una secuencia de 8 graficas de diagnostico para validar el modelo Poisson.
}
\details{
El flujo interno de la funcion se divide en seis etapas criticas:
\enumerate{
\item \strong{Ajuste Log-Lineal:} Estima la relacion entre los predictores y el logaritmo
de la tasa de eventos (\eqn{\lambda}). Utiliza Maxima Verosimilitud (MLE) para el ajuste.
\item \strong{Interpretacion de Tasas (IRR):} Los coeficientes originales (\eqn{\beta}) se
exponencian automaticamente \eqn{e^{\beta}} para obtener los \strong{Ratios de Tasas de Incidencia}.
Esto permite concluir en que porcentaje aumenta o disminuye la frecuencia de eventos
por cada unidad de cambio en X.
\item \strong{Diagnostico de Equidispersion:} Calcula el Ratio de Dispersion basado en el
estadistico de Pearson \eqn{(X^2 / gl)}. En una distribucion Poisson perfecta, este
valor debe ser 1. Valores > 1.5 indican \strong{Sobredispersion}, sugiriendo que los
datos varian mas de lo que el modelo permite.
\item \strong{Inferencia Robusta:} Genera intervalos de confianza para los IRR, permitiendo
identificar factores de riesgo (IRR > 1) y factores protectores (IRR < 1).
\item \strong{Evaluacion de Residuos:} Calcula residuos de devianza y Pearson para identificar
patrones de error sistematicos o falta de ajuste en conteos especificos (como el cero).
\item \strong{Traduccion Didactica:} Genera un reporte que advierte si el modelo es valido
o si se debe migrar a un modelo Binomial Negativa debido a la dispersion.
}
}
\section{Guia de las 8 Graficas de Diagnostico (\code{plot})}{

Al ejecutar \code{plot(modelo)}, se despliega una secuencia de validacion para conteos:
\describe{
\item{\strong{1. Distribucion de Conteos (Real vs Modelo)}}{Compara las frecuencias observadas contra las que el modelo "esperaba" predecir. Crucial para detectar el exceso de ceros.}
\item{\strong{2. Precision (Predicho vs Real)}}{Scatterplot entre el Lambda estimado y el conteo real. Los puntos deben seguir la tendencia de la linea roja diagonal.}
\item{\strong{3. Importancia de Variables (IRR)}}{Visualiza el impacto relativo de cada predictor. Los puntos verdes son significativos; si el intervalo cruza el 1, la variable es ruido.}
\item{\strong{4. Verificacion de Sobredispersion}}{Grafica la relacion Media-Varianza. Si la linea verde (datos) se eleva sobre la negra (teoria), el supuesto Poisson se rompe.}
\item{\strong{5. Distancia de Cook}}{Identifica casos atipicos en el conteo que tienen un peso excesivo en la estimacion de la tasa promedio.}
\item{\strong{6. Residuos de Devianza}}{Analiza la calidad del ajuste en escala logaritmica. Busca una nube uniforme sin formas de "U" o de embudo.}
\item{\strong{7. Apalancamiento (Leverage)}}{Detecta observaciones con valores de X inusuales que controlan la pendiente del modelo.}
\item{\strong{8. Densidad de Ajuste Global}}{Superpone las curvas de probabilidad del modelo y la realidad para evaluar el solapamiento total.}
}
}

\examples{
data(datos_tala_ilegal_poisson)
modelo_poisson <- reg_poisson(numero_talas ~ densidad_poblacion + 
                              superficie_forestal_ha + dias_vigilancia, 
                              data = datos_tala_ilegal_poisson)
plot(modelo_poisson)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datos_roya_cafe.R
\docType{data}
\name{datos_roya_cafe}
\alias{datos_roya_cafe}
\title{Presencia de roya del cafe en diferentes localidades}
\format{
Un data frame con 300 observaciones y 4 variables:
\describe{
\item{presencia_roya}{Presencia (1) o ausencia (0) de roya del cafe}
\item{humedad_relativa_pct}{Humedad relativa promedio (\%)}
\item{altitud_msnm}{Altitud de la finca (m s.n.m.)}
\item{manejo_sombra}{Porcentaje de cobertura de sombra en la plantacion}
}
}
\source{
Elaboracion propia con fines didacticos basada en informacion de SIAP e INIFAP.
}
\usage{
datos_roya_cafe
}
\description{
Base de datos simulada que presenta la presencia/ausencia de roya del cafe
junto con variables ambientales y de manejo agronomico relevantes.
Es util para ejercicios de regresion logistica.
}
\details{
Los datos fueron generados de forma controlada utilizando parametros
similares a los observados por SIAP y estudios agronomicos del INIFAP.
Esta base fue creada para ser utilizada en funciones de regresion logistica del paquete \pkg{GREENREG}.
}
\examples{
data(datos_roya_cafe)
modelo_log <- logistico(presencia_roya ~ humedad_relativa_pct + 
                        altitud_msnm + manejo_sombra, 
                        data = datos_roya_cafe)
                        

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in
%   R/Modelo_Autoregresivo_Integrado_de_Media_Movil_(ARIMA).R
\name{modelo_arima}
\alias{modelo_arima}
\alias{print.arima_greenreg}
\alias{plot.arima_greenreg}
\title{Modelo Integrado de Media Movil Autorregresivo (ARIMA)}
\usage{
modelo_arima(x, p = 1, d = 1, q = 1, include_mean = TRUE)

\method{print}{arima_greenreg}(x, ...)

\method{plot}{arima_greenreg}(x, ...)
}
\arguments{
\item{x}{Un objeto de clase "arima_greenreg".}

\item{p}{Orden Autorregresivo (memoria del pasado).}

\item{d}{Orden de Integracion (numero de diferenciaciones).}

\item{q}{Orden de Media Movil (memoria de los errores).}

\item{include_mean}{Logico. Incluir constante? (Suele ser \code{FALSE} si \eqn{d > 0}).}

\item{...}{Argumentos adicionales.}
}
\value{
Un objeto de clase \code{"arima_greenreg"} que contiene:
\itemize{
\item \code{modelo}: El objeto \code{arima} original de \code{stats}.
\item \code{ordenes}: Vector con los valores de p, d y q.
\item \code{coeficientes_tabla}: Matriz con estimaciones, errores estandar y p-valores.
\item \code{estatus}: Lista con el estado de estabilidad e invertibilidad.
\item \code{diagnosticos}: Resultados de Ljung-Box, Shapiro-Wilk y AIC.
\item \code{data_plot}: Dataframe listo para visualización con \code{ggplot2}.
\item \code{data_diff}: Dataframe de la serie diferenciada (si \eqn{d > 0}).
}
}
\description{
Esta funcion ajusta el modelo mas versatil para series de tiempo univariadas:
el ARIMA(p, d, q). Combina la memoria de largo plazo (AR), la estabilizacion
mediante diferenciacion (I) para eliminar tendencias, y la respuesta a
choques aleatorios de corto plazo (MA).

Genera un panel visual para evaluar el ajuste del modelo.
}
\details{
El flujo interno de la funcion ejecuta una auditoria de cinco fases:
\enumerate{
\item \strong{Diferenciacion (Fase I):} Si \eqn{d > 0}, el modelo resta los valores
consecutivos para eliminar la tendencia y lograr que la serie sea
"Estacionaria" (promedio constante).
\item \strong{Estimacion ARMA:} Sobre la serie ya estabilizada, estima los
parametros \eqn{\phi} (AR) y \eqn{\theta} (MA) mediante Maxima Verosimilitud.
\item \strong{Doble Validacion de Estructura:} Analiza las raices del polinomio
caracteristico para asegurar que la parte AR sea \strong{Estable} y la
parte MA sea \strong{Invertible}.
\item \strong{Auditoria de Residuos:} Verifica que el error final sea "Ruido Blanco"
mediante las pruebas de Ljung-Box (independencia) y Shapiro-Wilk (normalidad).
\item \strong{Diagnostico Dinamico:} Evalua si la media y la varianza se mantienen
estables a lo largo del tiempo despues de la integracion.
}
}
\section{Guia de las 10 Graficas de Diagnostico}{

Al ejecutar \code{plot(modelo)}, se despliega el diagnostico mas completo del paquete:
\itemize{
\item \strong{1. Serie Original:} Visualiza la serie bruta. Fundamental para identificar la tendencia inicial que justifica el valor de \eqn{d}.
\item \strong{2. Ajuste Integrado:} Superpone el modelo (purpura) sobre la realidad, incluyendo bandas de confianza del 95\\%.
\item \strong{3. Serie Diferenciada:} Muestra la serie despues de aplicar \eqn{d}. Si el modelo es correcto, esta serie debe verse plana y sin tendencia.
\item \strong{4. Circulo Unitario:} Grafica de convergencia matematica. Los puntos rojos (AR) y azules (MA) deben estar dentro del circulo gris.
\item \strong{5. Residuos en el Tiempo:} Busca patrones de error. Los puntos deben formar una nube aleatoria sin ciclos visibles.
\item \strong{6. ACF de Residuos:} Prueba de independencia. Las barras deben estar dentro de los limites para confirmar el Ruido Blanco.
\item \strong{7. PACF de Residuos:} Ayuda a identificar si el orden autorregresivo \eqn{p} es suficiente para explicar la inercia.
\item \strong{8. Normalidad (Q-Q Plot):} Valida que los errores sigan una distribucion gaussiana para asegurar pronosticos confiables.
\item \strong{9. Precision (Real vs Ajustado):} Evaluacion de la correlacion de Pearson entre los datos y la estimacion.
\item \strong{10. Estacionariedad Final:} Muestra la media y varianza movil. Una franja purpura estable certifica un modelo robusto.
}
}

\examples{
data("datos_nivel_presa")
ts_data <- datos_nivel_presa$nivel_m
modelo_ARIMA <- modelo_arima(ts_data, p = 1, d = 1, q = 1)
print(modelo_ARIMA)
plot(modelo_ARIMA)


}
\seealso{
\code{\link[stats]{arima}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Regresion_lineal_multiple.R
\name{rlm}
\alias{rlm}
\alias{print.rlm_greenreg}
\alias{plot.rlm_greenreg}
\title{Regresion Lineal Multiple (RLM)}
\usage{
rlm(formula, data, nivel_confianza = 0.95)

\method{print}{rlm_greenreg}(x, ...)

\method{plot}{rlm_greenreg}(x, ...)
}
\arguments{
\item{formula}{Formula modelo.}

\item{data}{Dataframe con los datos.}

\item{nivel_confianza}{Nivel de confianza para intervalos (default 0.95).}

\item{x}{Objeto de clase rlm_greenreg.}

\item{...}{Argumentos adicionales.}
}
\value{
Un objeto de clase \code{"rlm_greenreg"} que contiene:
\itemize{
\item \code{modelo}: Objeto \code{lm} original.
\item \code{variables}: Lista con los nombres de Y y los predictores X.
\item \code{coeficientes}: Matriz con estimaciones, p-valores e intervalos de confianza.
\item \code{vif}: Vector con los valores del Factor de Inflacion de la Varianza (VIF).
\item \code{supuestos}: Resultados de los tests de Shapiro-Wilk (Normalidad), Breusch-Pagan
(Homocedasticidad), Durbin-Watson (Independencia) y Rainbow (Linealidad).
\item \code{notas}: Reporte interpretado con recomendaciones de accion.
\item \code{data_plot}: Dataframe optimizado para visualizaciones.
}
}
\description{
Esta funcion ajusta un modelo de regresion lineal con multiples variables predictoras
\eqn{y = \beta_0 + \beta_1 x_1 + \beta_2 x_2 + \dots + \beta_k x_k} mediante Minimos Cuadrados Ordinarios (OLS).
Incluye un robusto motor de diagnostico de Multicolinealidad (VIF) y pruebas de supuestos
con recomendaciones de accion para el investigador.

Genera una secuencia de 10 graficas para validar el modelo.
}
\details{
El flujo interno de la funcion se divide en seis etapas:
\enumerate{
\item \strong{Validacion de Datos:} Asegura que la formula sea valida y que el dataframe no contenga errores estructurales.
\item \strong{Ajuste (Engine):} Utiliza \code{stats::lm}. En RLM, el foco principal es el \strong{R2 Ajustado},
que penaliza la inclusion de variables innecesarias que no aportan informacion real.
\item \strong{Deteccion de Multicolinealidad (VIF):} Calcula manualmente el Factor de Inflacion de
la Varianza \eqn{VIF = 1 / (1 - R^2_j)} para cada predictor. Esto identifica si las variables
X estan correlacionadas entre si, lo cual infla los errores estandar y vuelve inestables los coeficientes.
\item \strong{Auditoria de Supuestos:} Ejecuta de forma nativa pruebas de Normalidad (Shapiro),
Homocedasticidad (Breusch-Pagan), Independencia (Durbin-Watson) y Linealidad (Rainbow Test).
\item \strong{Traduccion Automatica:} Evalua los estadisticos y genera "Notas".
y sugerencias de mejora (ej. transformar variables o eliminar predictores redundantes).
\item \strong{Preparacion de Diagnostico:} Construye un dataframe optimizado para \code{ggplot2} que
incluye residuos estandarizados, distancias de Cook y apalancamiento (leverage).
}
}
\section{Guia de las 10 Graficas de Diagnostico (\code{plot})}{

Al ejecutar \code{plot(modelo)}, se despliega una secuencia educativa de 10 visualizaciones:
\describe{
\item{\strong{1. Distribucion de Variables (Y, Xi)}}{Muestra histogramas y densidades de todas las variables. Permite detectar sesgos o datos atipicos antes del ajuste.}
\item{\strong{2. Relacion de Variables (Y vs Xi)}}{Scatterplots individuales de la respuesta contra cada predictor con una curva LOESS para verificar la linealidad visualmente.}
\item{\strong{3. Ajuste (Observados vs Ajustados)}}{Muestra la precision global. Los puntos deben seguir la diagonal de 45 grados.}
\item{\strong{4. Q-Q Plot (Normalidad)}}{Valida si los residuos siguen una distribucion normal. Crucial para la validez de los P-valores.}
\item{\strong{5. Homocedasticidad}}{Residuos vs. Predichos. Busca una "nube de estrellas" sin forma de embudo.}
\item{\strong{6. Independencia}}{Residuos vs. Orden de recoleccion. Detecta patrones temporales o espaciales en los errores.}
\item{\strong{7. Prediccion vs Realidad (Identidad)}}{Version simplificada del ajuste para evaluar la correlacion de Pearson.}
\item{\strong{8. Multicolinealidad (VIF)}}{Grafica de barras exclusiva de RLM. Las barras rojas (VIF > 10) indican variables que deben ser eliminadas por redundancia.}
\item{\strong{9. Distancia de Cook}}{Identifica observaciones influyentes. Barras que superan el umbral $4/n$ indican datos que deforman los resultados.}
\item{\strong{10. Leverage vs Residuos}}{Cruza la "rareza" de X (Leverage) con el error en Y (Residuo). Detecta puntos que actuan como palancas sobre la recta.}
}
}

\examples{
  data(datos_rendimiento_maiz)
  modelo_rlm <- rlm(rendimiento_maiz_ton_ha ~ ., data = datos_rendimiento_maiz)
  print(modelo_rlm)
  plot(modelo_rlm)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Analisis_datos.R
\name{analisis_datos}
\alias{analisis_datos}
\title{Asistente Interactivo de Analisis de Datos}
\usage{
analisis_datos(data, variable_objetivo, verbose = TRUE)
}
\arguments{
\item{data}{Data frame o base de datos que contiene las variables.}

\item{variable_objetivo}{Cadena de texto con el nombre exacto de la columna que deseas analizar (ej. "Ventas").}

\item{verbose}{Logico. Si es TRUE, despliega los menus. Por defecto es TRUE.}
}
\value{
Devuelve de forma invisible un objeto de clase \code{ggplot2}.
La funcion se utiliza principalmente por su efecto secundario de imprimir
codigo sugerido en la consola y generar visualizaciones.
}
\description{
Esta es la funcion principal y punto de entrada del paquete. Actua como un consultor
automatizado que guia al usuario desde la carga de datos hasta la generacion del codigo final,
seleccionando la metodologia adecuada segun la naturaleza de la variable objetivo.
}
\section{Como funciona este Asistente paso a paso?}{

\enumerate{
\item \strong{Escaneo y Validacion:}
Analiza la base de datos para asegurar que la variable objetivo existe.
Ademas, realiza un test interno de autocorrelacion para verificar si tiene sentido aplicar modelos de Series de Tiempo (evitando analisis espurios).
\item \strong{Menu Interactivo:}
Despliega opciones en la consola basandose en los datos detectados, preguntando al usuario que enfoque desea (Regresion Clasica, Modelos Generalizados o Series de Tiempo).
\item \strong{Generacion de Estrategias (El "Motor"):}
\itemize{
\item \emph{Para Regresion y GLM:} Crea dos versiones del codigo.
\enumerate{
\item \strong{Modelo Full:} Incluye todas las variables disponibles.
\item \strong{Modelo Optimizado:} Ejecuta internamente un algoritmo "Stepwise" basado en AIC para limpiar el modelo y quedarse solo con los predictores relevantes.
}
\item \emph{Para Series de Tiempo:} Ejecuta una "Busqueda en Rejilla" (Grid Search) probando combinaciones de parametros (p, d, q). Identifica automaticamente si la serie necesita diferenciacion (ARIMA), si es puramente autoregresiva (AR), de media movil (MA) o mixta (ARMA), y genera el codigo para la funcion especifica.
}
}
}

\section{Guia de la Grafica Exploratoria Generada}{

Dependiendo de la opcion elegida, la funcion genera una visualizacion preliminar diferente para entender los datos antes de modelar:
\describe{
\item{\strong{Escenario 1: Regresion Lineal (Histograma + Gauss)}}{
Muestra un histograma azul de tus datos superpuesto con una curva roja (Campana de Gauss teorica).
\emph{Objetivo:} Evaluar visualmente si la variable respuesta se parece a una distribucion Normal (supuesto clave de la regresion lineal).
}
\item{\strong{Escenario 2: GLM (Barras o Histograma)}}{
Detecta el tipo de variable:
\itemize{
\item \emph{Si es Binaria (0/1):} Grafico de barras naranja. Ayuda a ver si las clases estan desbalanceadas (ej. muchos "0" y pocos "1").
\item \emph{Si es Conteo:} Histograma verde. Muestra la frecuencia de eventos (tipico de Poisson).
}
}
\item{\strong{Escenario 3: Serie de Tiempo (Linea de Tendencia)}}{
Grafica la evolucion de la variable a lo largo del indice temporal.
\emph{Objetivo:} Identificar tendencias (subidas/bajadas a largo plazo) o ciclos repetitivos antes de ajustar el modelo.
}
}
}

\examples{
  data(datos_rendimiento_maiz)
  analisis_datos(datos_rendimiento_maiz, "rendimiento_maiz_ton_ha")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Modelo_Autoregresivo_de_media_movil_(ARMA).R
\name{modelo_arma}
\alias{modelo_arma}
\alias{print.arma_greenreg}
\alias{plot.arma_greenreg}
\title{Modelo Autorregresivo de Media Movil (ARMA)}
\usage{
modelo_arma(x, p = 1, q = 1, include_mean = TRUE)

\method{print}{arma_greenreg}(x, ...)

\method{plot}{arma_greenreg}(x, ...)
}
\arguments{
\item{x}{Objeto de clase.}

\item{p}{Orden Autorregresivo (pasados de la variable).}

\item{q}{Orden de Media Movil (pasados del error).}

\item{include_mean}{Logico. Incluir constante/intercepto? Por defecto \code{TRUE}.}

\item{...}{Argumentos adicionales.}
}
\value{
Un objeto de clase \code{"arma_greenreg"} que contiene:
\itemize{
\item \code{modelo}: Objeto \code{arima} original (orden p, 0, q).
\item \code{ordenes}: Vector con los valores de p y q.
\item \code{coeficientes_tabla}: Matriz con estimaciones, errores estandar y p-valores.
\item \code{estatus}: Lista con el estado de estabilidad e invertibilidad.
\item \code{diagnosticos}: Resultados de Ljung-Box, Shapiro-Wilk y AIC.
\item \code{data_plot}: Dataframe listo para visualizacion con \code{ggplot2}.
}
}
\description{
Esta funcion ajusta un modelo hibrido ARMA(p, q) que combina la dependencia
de los valores pasados (AutoRegresivo) con la influencia de los errores
pasados (Media Movil). Es la herramienta estandar para modelar series de
tiempo estacionarias con estructuras de dependencia complejas.

Genera una secuencia de 9 graficas de diagnostico para validar el modelo ARMA.
}
\details{
El flujo interno de la funcion realiza una auditoria estructural en cinco pasos:
\enumerate{
\item \strong{Estimacion Conjunta:} Utiliza Maxima Verosimilitud (MLE) para encontrar
simultaneamente los coeficientes \eqn{\phi} (AR) y \eqn{\theta} (MA).
\item \strong{Estabilidad (Parte AR):} Verifica que el componente autorregresivo no sea
explosivo. Las raices inversas del polinomio AR deben estar dentro del circulo.
\item \strong{Invertibilidad (Parte MA):} Verifica que el componente de media movil sea
univoco. Las raices inversas del polinomio MA deben estar dentro del circulo.
\item \strong{Diagnostico de Residuos:} Evalua si el error resultante es Ruido Blanco
mediante el test de Ljung-Box, asegurando que no quede senal sin capturar.
\item \strong{Criterio de Informacion:} Proporciona el AIC para facilitar la comparacion
entre diferentes combinaciones de (p, q), buscando el equilibrio entre
complejidad y precision.
}
}
\section{Guia de las 9 Graficas de Diagnostico (\code{plot})}{

Al ejecutar \code{plot(modelo)}, se despliega una secuencia de validacion mixta:
\describe{
\item{\strong{1. Inspeccion de Serie}}{Muestra la serie bruta con su media. Fundamental para confirmar visualmente la estacionariedad.}
\item{\strong{2. Ajuste Mixto}}{Superpone el modelo (verde) sobre la realidad. Permite evaluar como interactuan la inercia y los shocks.}
\item{\strong{3. Residuos en el Tiempo}}{Evaluacion de la aleatoriedad del error. Los puntos deben formar una nube sin patrones.}
\item{\strong{4. Circulo Unitario Dual}}{Grafica critica exclusiva de ARMA. Muestra simultaneamente la Estabilidad (Puntos Rojos) e Invertibilidad (Rombos Azules).}
\item{\strong{5. ACF de Residuos}}{Busca autocorrelaciones remanentes. Las barras deben ser minimas para confirmar que el modelo es optimo.}
\item{\strong{6. PACF de Residuos}}{Ayuda a identificar si el orden autorregresivo (p) es suficiente.}
\item{\strong{7. Normalidad (Q-Q Plot)}}{Valida que los errores sigan una campana de Gauss, requisito para la validez de los intervalos de confianza.}
\item{\strong{8. Precision (Real vs Ajustado)}}{Muestra la correlacion directa. Un ARMA exitoso suele tener una alineacion estrecha sobre la diagonal.}
\item{\strong{9. Estacionariedad Movil}}{Prueba final de estabilidad. Una franja verde horizontal confirma que el modelo no se desplaza en el tiempo.}
}
}

\examples{
data("datos_anomalia_temperatura")
ts_data <- datos_anomalia_temperatura$anomalia_c
modelo <- modelo_arma(ts_data, p = 1, q = 1)
modelo
plot(modelo)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datos_rendimiento_maiz.R
\docType{data}
\name{datos_rendimiento_maiz}
\alias{datos_rendimiento_maiz}
\title{Rendimiento de maiz bajo distintos niveles de fertilizacion}
\format{
Un data frame con 250 observaciones y 4 variables:
\describe{
\item{rendimiento_maiz_ton_ha}{Rendimiento del cultivo de maiz (ton/ha)}
\item{precipitacion_mm}{Precipitacion registrada durante el ciclo agricola (mm)}
\item{temperatura_c}{Temperatura media (C)}
\item{fertilizante_kg_ha}{Cantidad de fertilizante aplicado (kg/ha)}
}
}
\source{
Elaboracion propia con fines didacticos tomando referencia de datos del SIAP e INEGI.
}
\usage{
datos_rendimiento_maiz
}
\description{
Conjunto de datos que presenta el rendimiento del maiz en funcion
de diferentes variables climaticas y de manejo del cultivo.
Este dataset es util para ejemplificar modelos de regresion lineal simple y multiple.
}
\details{
Los datos fueron generados de forma controlada, tomando como referencia
distribuciones y rangos reales reportados por SIAP e INEGI. Esta base se diseno
especificamente para probar las funciones de regresion del paquete \pkg{GREENREG}.
}
\examples{
data(datos_rendimiento_maiz)
modelo_rlm <- rlm(rendimiento_maiz_ton_ha ~., data = datos_rendimiento_maiz)
modelo_rlm       
plot(modelo_rlm)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datos_anomalia_temperatura.R
\docType{data}
\name{datos_anomalia_temperatura}
\alias{datos_anomalia_temperatura}
\title{Anomalias de temperatura media diaria}
\format{
Un data frame con 730 observaciones y 2 variables:
\describe{
\item{fecha}{Fecha del registro (AAAA-MM-DD)}
\item{anomalia_c}{Anomalia de temperatura respecto al promedio historico (C)}
}
}
\source{
Elaboracion propia con fines didacticos a partir de informacion publica de INEGI y CONAGUA.
}
\usage{
datos_anomalia_temperatura
}
\description{
Conjunto de datos que muestra las anomalias (diferencias respecto a la media historica)
de la temperatura media diaria registradas a lo largo del tiempo. Este dataset puede
utilizarse para ilustrar analisis de series de tiempo y modelos autorregresivos o ARIMA.
}
\details{
Los datos fueron construidos de manera simulada a partir de patrones reales
observados en bases climaticas del INEGI y de la CONAGUA. La creacion propia de esta
base permite garantizar que los valores sean adecuados para fines didacticos y
compatibles con las funciones del paquete \pkg{GREENREG}.
}
\examples{
data("datos_anomalia_temperatura")
modelo_AR <- modelo_ar(datos_anomalia_temperatura$anomalia_c, p = 2)
modelo_AR
plot(modelo_AR)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Regresion_logistica.R
\name{logistico}
\alias{logistico}
\alias{print.logistico_greenreg}
\alias{plot.logistico_greenreg}
\title{Regresion Logistica Binaria (MLG Logit)}
\usage{
logistico(formula, data, umbral = 0.5)

\method{print}{logistico_greenreg}(x, ...)

\method{plot}{logistico_greenreg}(x, ...)
}
\arguments{
\item{formula}{Una formula de R (ej. \code{presencia_plaga ~ humedad + altitud}).}

\item{data}{Un \code{data.frame} con las observaciones.}

\item{umbral}{Punto de corte para clasificar como "Evento (1)" (por defecto 0.5).}

\item{x}{Objeto de clase logistico_greenreg.}

\item{...}{Argumentos adicionales.}
}
\value{
Un objeto de clase \code{"logistico_greenreg"}, que consiste en una lista con los siguientes componentes:
\itemize{
\item \code{formula}: La formula de R aplicada al modelo.
\item \code{modelo}: El objeto de clase \code{glm} original con el ajuste binomial.
\item \code{variables}: Una lista que contiene el nombre de la variable respuesta (\code{y}).
\item \code{coeficientes}: Matriz con las estimaciones de los coeficientes, errores estandar, estadisticos z y p-valores.
\item \code{odds_ratios}: Tabla con los Odds Ratios (OR) calculados, sus intervalos de confianza y p-valores asociados.
\item \code{metricas}: Una lista que incluye el Pseudo-R2 de McFadden, el Criterio de Informacion de Akaike (AIC) y la precision global (\emph{Accuracy}).
\item \code{confusion}: Matriz de confusion que cruza los valores reales frente a los predichos segun el umbral especificado.
\item \code{predicciones}: Un \code{data.frame} con los valores observados, probabilidades predichas, clasificacion final y el apalancamiento (\emph{leverage}).
\item \code{diag_leverage}: Lista con informacion diagnostica sobre el apalancamiento y deteccion de puntos extremos.
\item \code{notas}: Lista de mensajes interpretativos sobre la bondad de ajuste y la precision.
\item \code{supuestos}: Un vector de caracteres con las validaciones tecnicas (semaforos) sobre significancia, multicolinealidad e influencia.
}
}
\description{
Esta funcion ajusta un Modelo Lineal Generalizado (GLM) de la familia binomial para
predecir la probabilidad de un evento dicotomico (Si/No, 0/1). A diferencia de la
regresion lineal, el motor de calculo estima los coeficientes mediante Maxima
Verosimilitud (MLE) y ofrece un diagnostico especializado en clasificacion.

Genera una secuencia de 9 graficas para validar el modelo de regresion logistica.
}
\details{
El flujo interno de la funcion se divide en cinco etapas fundamentales:
\enumerate{
\item \strong{Ajuste Logit:} Transforma la variable respuesta mediante la funcion de enlace
\eqn{logit(p) = ln(p / (1-p))}, permitiendo que la probabilidad se mantenga entre 0 y 1.
\item \strong{Conversion de Impacto:} Los coeficientes estimados se transforman automaticamente
a \strong{Odds Ratios (OR)} mediante \eqn{e^\beta}. Esto permite concluir cuanto
multiplica cada variable la posibilidad de que ocurra el evento.
\item \strong{Evaluacion de Bondad (McFadden):} Calcula el Pseudo-\eqn{R^2} comparando la
verosimilitud del modelo ajustado contra un modelo nulo (sin predictores).
\item \strong{Metricas de Clasificacion:} Genera la Matriz de Confusion basada en el \code{umbral}
especificado y calcula la Precision Global (Accuracy).
\item \strong{Diagnostico de Influencia:} Evalua si existen observaciones que deforman
las probabilidades mediante el calculo de la Distancia de Cook y el Apalancamiento (Leverage).
}
}
\section{Guia de las 9 Graficas de Diagnostico (\code{plot})}{

Al ejecutar \code{plot(modelo)}, se despliega una secuencia de validacion predictiva:
\describe{
\item{\strong{1. Capacidad de Separacion}}{Muestra si el modelo realmente distingue entre grupos. Buscamos "montanas" de densidad que no se traslapen.}
\item{\strong{2. Matriz de Confusion Visual}}{Mapa de calor que resume aciertos (diagonal) y errores (falsos positivos/negativos).}
\item{\strong{3. Odds Ratios (Forest Plot)}}{Visualiza la importancia de las variables. Los puntos a la derecha del 1 son factores de riesgo; a la izquierda, protectores.}
\item{\strong{4. Curva de Calibracion}}{Compara la probabilidad que "dice" el modelo contra la frecuencia real observada. Debe seguir la diagonal.}
\item{\strong{5. Curva ROC (AUC)}}{Mide la potencia global de diagnostico. Un AUC de 1.0 es una clasificacion perfecta; 0.5 es puro azar.}
\item{\strong{6. Distancia de Cook}}{Detecta casos atipicos en la clasificacion que tienen un peso excesivo en los coeficientes.}
\item{\strong{7. Trade-off de Umbrales}}{Permite ver como cambia la Sensibilidad y Especificidad al mover el punto de corte.}
\item{\strong{8. Residuos de Pearson}}{Busca patrones sistematicos en el error. Idealmente, la linea roja debe mantenerse cerca del cero.}
\item{\strong{9. Leverage vs Residuos}}{Identifica observaciones con combinaciones de variables X muy raras que "jalonean" las probabilidades.}
}
}

\examples{
data(datos_roya_cafe)
modelo_log <- logistico(presencia_roya ~ humedad_relativa_pct + 
                        altitud_msnm + manejo_sombra, 
                        data = datos_roya_cafe)
                        
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Regresion_lineal_simple.R
\name{rls}
\alias{rls}
\alias{print.rls_greenreg}
\alias{plot.rls_greenreg}
\title{Regresion Lineal Simple (RLS)}
\usage{
rls(formula, data, nivel_confianza = 0.95)

\method{print}{rls_greenreg}(x, ...)

\method{plot}{rls_greenreg}(x, ...)
}
\arguments{
\item{formula}{Una formula de R estandar (ej. \code{rendimiento ~ fertilizante}).}

\item{data}{El data.frame que contiene las variables.}

\item{nivel_confianza}{Nivel de confianza para los intervalos (por defecto 0.95).}

\item{x}{Objeto de clase rls_greenreg.}

\item{...}{Argumentos adicionales.}
}
\value{
Un objeto de clase \code{"rls_greenreg"} que es una lista con:
\itemize{
\item \code{modelo}: El objeto \code{lm} original.
\item \code{variables}: Nombres de X e Y.
\item \code{coeficientes}: Matriz con Estimaciones, Errores e Intervalos.
\item \code{supuestos}: Resultados de los tests de Shapiro, BP y DW.
\item \code{notas}: Diccionario de interpretaciones y sugerencias de mejora.
\item \code{data_plot}: Dataframe con residuos e indices de influencia.
}
}
\description{
Esta funcion ajusta un modelo de regresion lineal \eqn{y = \beta_0 + \beta_1 x} mediante
Minimos Cuadrados Ordinarios (OLS, por sus siglas en ingles 'Ordinary Least Squares').
A diferencia de \code{lm()} estandar, esta funcion devuelve un objeto enriquecido con
interpretaciones en lenguaje natural, verificaciones automaticas de supuestos y
sugerencias para el investigador.

Genera una secuencia de 9 graficas para validar el modelo.
}
\details{
El flujo interno de la funcion se divide en cinco etapas criticas:
\enumerate{
\item \strong{Preparacion:} Valida que los datos sean coherentes y extrae las variables de la formula.
\item \strong{Estimacion:} Calcula los coeficientes, el error estandar residual \eqn{(\sigma)} y la significancia global (F-test).
\item \strong{Inferencia:} Genera intervalos de confianza para los parametros \eqn{(\beta)} al nivel especificado.
\item \strong{Auditoria de Supuestos:} Ejecuta pruebas de Shapiro-Wilk (Normalidad), Breusch-Pagan (Homocedasticidad)
y Durbin-Watson (Independencia).
\item \strong{Traduccion:} Convierte los estadisticos abstractos en "Notas" de texto que recomiendan acciones
especificas (ej. aplicar logaritmos o revisar datos atipicos).
}
}
\section{Guia de las 9 Graficas de Diagnostico (\code{plot})}{

Al ejecutar \code{plot(modelo)}, se despliega una secuencia educativa:
\describe{
\item{\strong{1. Histogramas Exploratorios}}{Compara la distribucion de X e Y. Util para detectar sesgos iniciales.}
\item{\strong{2. Relacion Original (Scatterplot)}}{Muestra la nube de puntos pura para confirmar linealidad.}
\item{\strong{3. Recta de Regresion}}{Visualiza el ajuste final con bandas de confianza.}
\item{\strong{4. Q-Q Plot (Normalidad)}}{Grafica los cuantiles de los residuos contra una normal teorica.}
\item{\strong{5. Homocedasticidad}}{Residuos vs. Predichos para verificar varianza constante.}
\item{\strong{6. Independencia}}{Residuos vs. Orden de datos para detectar autocorrelacion.}
\item{\strong{7. Precision (Predicho vs Real)}}{Muestra la correlacion directa entre valores reales y estimados.}
\item{\strong{8. Distancia de Cook}}{Identifica observaciones influyentes o atipicos peligrosos.}
\item{\strong{9. Leverage vs Residuos}}{Diferencia entre errores en Y (Outliers) y valores extremos en X (Apalancamiento).}
}
}

\examples{
  data(datos_rendimiento_maiz)
  modelo <- rls(rendimiento_maiz_ton_ha ~ precipitacion_mm, data = datos_rendimiento_maiz)
  print(modelo)
  plot(modelo)

}
